// CFEMPlotter.cpp: implementation of the CFEMPlotter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CFEMPlotter.h"
#include "..\WinGUI\IWinGUI.h"										// IWinGUI
#include "..\..\PlugInSource\MessageHandler\IMessageHandler.h"		// IMessageHandler
#include "commdlg.h"												// OPENFILENAME, GetOpenFileName
#include "..\GraphLib\IGraphLib.h"									// I2DMemoryDC
#include "DVA\CuthillMcKee.h"										// CCuthillMcKee Algorithmus

#include "DVA\FemElement.h"											// eET_PARALLELOGRAMM

#include <mmsystem.h>
#include <stdio.h>
#include <assert.h>
#include <shellapi.h>
#include <math.h>													// floor

#include "..\..\shared\GUIAppData.h"								// CGUIAppData

// Extract the error code from an HRESULT
#define CODEFROMHRESULT(hRes) ((hRes) & 0x0000FFFFUL)


#define ERR if(g_App->GetErrorHandler())(g_App->GetErrorHandler())		// fr Fehlerausgabe
#define MESS if(g_App->GetMessageHandler())(g_App->GetMessageHandler())		// fr Logfileausgabe


extern IEditorApplication *g_App;


// Tippersparniss fr WinGUI
#define BEGIN	{ CHandle<ISceneGraphObject> nsich=n; n=
#define END		n=nsich; }


#define MAXE	(CFemElement::eElementType::eET_PARALLELOGRAMM2)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFEMPlotter::CFEMPlotter() : CDocObject(),m_scale(100.0f),m_femproc(&m_perftime),m_subx(0),m_suby(0)
{
	// Var Init
	InitVar();
	m_DetailLevel=4;
}



void CFEMPlotter::InitModule( void )
{
	BuildDialog(0);
}



// 0: wie nach dem Start
// 1: fr laminare Strmungen, Temperatur oder Spannungsverteilungen
// 2: fr Schwingungsaufgaben (EW-Berechnungen)
void CFEMPlotter::BuildDialog( int inMode )
{
	ReleaseSubObjects();

	assert(g_App);

	IWinGUI *gui=g_App->GetWinGUI();




	CRGUI_AlignMode m=alignl_stretch;
	CHandle<ISceneGraphObject> n;

	n=gui->CreateWindow_Dialog(eNone,/*IGUIDialogCallback*/this,"FEM Plotter",false,m,m);
	assert(n);


	m_MainGUIDialog=QUERYINTERFACE(IWinGUIDialog,n);

	n->Release();

	
	BEGIN gui->InsertButton_Group(n,"",false,m,m,borderm_none);

		BEGIN gui->CreateWindow_MenuBar(eNone,n,"");

			BEGIN gui->CreateWindow_MenuBar(eNone,n,			"Datei");
				gui->CreateWindow_MenuBar(eOpenFEMIN,n,				"FEMIN ffnen ...");

	if(inMode!=0)
	{
		if(inMode==1)
		{
				gui->CreateWindow_MenuBar(eOpenFEMOUT,n,			"FEMOUT ffnen ...");

		}
		else
		{
				gui->CreateWindow_MenuBar(eOpenFEMOUT,n,			"FEMOUT ffnen ...");

		}

	}
			END

				
			if(inMode!=0)
			{
				BEGIN gui->CreateWindow_MenuBar(eNone,n,			"Darstellung");
					BEGIN gui->CreateWindow_MenuBar(eNone,n,			"Potentiallinienqualitt");
						gui->CreateWindow_MenuBar(eDetails1,n,				"(1) schlecht");
						gui->CreateWindow_MenuBar(eDetails4,n,				"(4) mittel");
						gui->CreateWindow_MenuBar(eDetails8,n,				"(8) gut");
						gui->CreateWindow_MenuBar(eDetails16,n,				"(16) sehr gut");
		//				gui->CreateWindow_MenuBar(eDetails64,n,				"(64) bertrieben");
					END
				END

				BEGIN gui->CreateWindow_MenuBar(eNone,n,			"HeatShade");
					gui->CreateWindow_MenuBar(eHeatShadeNone,n,				"aus");
					if(inMode==1)
					{
						gui->CreateWindow_MenuBar(eHeatShadeU,n,				"fr U-Werte (schwarz=min, wei=max)");
						gui->CreateWindow_MenuBar(eHeatShadeV,n,				"fr V-Werte (schwarz=min, wei=max)");
					}
					else
					{
						gui->CreateWindow_MenuBar(eHeatShadeU,n,				"fr Eigenvektoren  (schwarz=0, wei=-1,1)");
					}
				END
			}
		END
	END


	BEGIN gui->InsertButton_Group(n,"FEM Gebiet",false,m,m,borderm_edgedframe);
		gui->InsertButton_Static(eNodeCountOutput,n,"keine Daten geladen");

	END

	if(inMode!=0)
	{
		BEGIN gui->InsertButton_Group(n,"Anzeige",false,m,m,borderm_edgedframe);
			BEGIN gui->InsertButton_Group(n,"",false,m,m,borderm_none);

				if(inMode==1)
				{
					BEGIN gui->InsertButton_Group(n,"Potentiallinien",false,m,m,borderm_edgedframe);

						BEGIN gui->InsertButton_Group(n,"",true,m,m,borderm_none);
							gui->InsertButton_Check(eDisplayU,n,"U Werte");
							gui->InsertButton_Edit(eStepperU,n,"");
						END

						BEGIN gui->InsertButton_Group(n,"",true,m,m,borderm_none);
							gui->InsertButton_Check(eDisplayV,n,"V Werte");
							gui->InsertButton_Edit(eStepperV,n,"");
						END

					END
				}
				else
				{
					BEGIN gui->InsertButton_Group(n,"Eigenwert",false,m,m,borderm_edgedframe);
				
						BEGIN gui->InsertButton_Group(n,"",true,alignl_min,m,borderm_none);
							gui->InsertButton_Static(eNone,n,"EW Nr.");
							gui->InsertButton_Push(eEWDecrese,n," < ");
							gui->InsertButton_Static(eEWNumberOutput,n," xxx/xxx ");
							gui->InsertButton_Push(eEWIncrease,n," > ");
							gui->InsertButton_Static(eEWValueOutput,n," = x.xxxx");
						END

						// Dummy, um etwas Abstand zu erzeugen
						BEGIN gui->InsertButton_Group(n,"",true,m,m,borderm_none);
						END

						BEGIN gui->InsertButton_Group(n,"",true,m,m,borderm_none);
							gui->InsertButton_Check(eDisplayU,n,"U Werte");
							gui->InsertButton_Edit(eStepperU,n,"");
						END

						gui->InsertButton_Check(eDisplayZeroU,n,"U Werte nur Nulldurchgnge");
					END
				}

				gui->InsertButton_Check(eDisplayElements,n,"Elementnummern");

				BEGIN gui->InsertButton_Group(n,"",true,m,m,borderm_none);
					BEGIN gui->InsertButton_Group(n,"",false,m,m,borderm_none);
						gui->InsertButton_Check(eDisplayNodes,n,"Knotennummern");
						gui->InsertButton_Check(eDisplayMatrix,n,"Gesamtsteifigkeitsmatrix ");
					END

				END


				BEGIN gui->InsertButton_Group(n,"",true,alignl_min,m,borderm_none);
					gui->InsertButton_Check(eDisplayEdges,n,"Elementrnder");
						gui->InsertButton_Check(eDisplayEdgesHB,n,"Transparent");
				END
			END

		END
	}
	else
	{
		BEGIN gui->InsertButton_Group(n,"Info",false,m,m,borderm_edgedframe);
			gui->InsertButton_Static(eNone,n,"FEMPlotter V1.1\n\ngeschrieben von:\n\n\n");
			gui->InsertButton_Static(eNone,n,"           Tobias Lang\n       e-mail: Tobias@TwistingTornado.de\n");
			gui->InsertButton_Static(eNone,n,"                und\n\n");
			gui->InsertButton_Static(eNone,n,"          Martin Mittring\n      e-mail: Kosmokleaner@Kosmokleaner.de");
		END

		

		BEGIN gui->InsertButton_Group(n,"",true,alignl_max,m,borderm_none);
			gui->InsertButton_Static(eNone,n,"www.Kosmokleaner.de");
		END
	}


	CHandle<I2DAutoLayout> aut;

	aut=QUERYINTERFACE(I2DAutoLayout,n);
	assert(aut);

	aut->RecalcLayoutRecursive();
	{
		ULONG id=1;

		aut->CreateLayoutRecursive(id);
	}


	// Alle Anzeigen auf setzen
	{	
		IWinGUIButton *f;

		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayU);if(f)f->SetValue(1);m_displayValues[0]=true;
		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayV);if(f)f->SetValue(1);m_displayValues[1]=true;
		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayElements);if(f)f->SetValue(1);m_displayValues[2]=true;
		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayNodes);if(f)f->SetValue(1);m_displayValues[3]=true;
		f=m_MainGUIDialog->GetPtrFromUserId(eStepperU);if(f)f->SetText("1.0");m_StepperU=1.0f;
		f=m_MainGUIDialog->GetPtrFromUserId(eStepperV);if(f)f->SetText("1.0");m_StepperV=1.0f;
		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayEdges);if(f)f->SetValue(1);m_displayValues[4]=true;
		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayMatrix);if(f)f->SetValue(0);m_displayValues[5]=false;
		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayNodesOptimized);if(f)f->SetValue(0);m_displayValues[6]=false;
		f=m_MainGUIDialog->GetPtrFromUserId(eDisplayMatrixOptimized);if(f)f->SetValue(0);m_displayValues[8]=false;	// MatrixOpti
		m_displayValues[9]=false;		// Nulldurchgnge
		m_displayValues[10]=false;		// Transparent
	}

}


CFEMPlotter::~CFEMPlotter()
{

	// Free Resources, Cleanup

	long f=((CComponentBase *)((IUnknown_ND*)this))->GetRefCount();		// Debug
}


void CFEMPlotter::InitVar()
{
	m_HeatShadeMode=0;

	m_perftime.Init();

	m_MemNeeded=0;
	m_EWNumber=0;				// kleinsten EW anzeigen

	m_ResultEW.FreeData();
	m_ResultEV.FreeData();
}




bool CFEMPlotter::IsObjectAt(CPoint2I xy)
{
	return(m_rect.PtInRect(xy));
}

void CFEMPlotter::ReleaseSubObjects( void )
{
	IWinGUI *gui=g_App->GetWinGUI();

	CHandle<ISceneGraphObject> gui2=QUERYINTERFACE(ISceneGraphObject,gui);

	gui2->ReleaseSubObjects();


//	m_MainGUIDialog->ReleaseObject();
	m_MainGUIDialog=0;						// freigeben
//	CDocObject::ReleaseSubObjects();
}







void CFEMPlotter::HeatShadeInnerTriangle( I2DMemoryDC *pMC, CPoint2F p[3], FP_TYPE value[3] )
{
	CPoint2F fdat[3];
	float val[3];
	
	for(int i=0;i<3;i++)
	{
		fdat[i].Set(p[i].x,p[i].y);
		val[i]=value[i];
	}

	pMC->TriAASpectrum(fdat,val);
}




void CFEMPlotter::PlotInnerTriangle( I2DMemoryDC *pMC, CPoint2F p[3], FP_TYPE value[3], float step, CColor col, CColor col_neg )
{
	if(m_displayValues[9])
	{ 
		PlotInnerTriangle_OnlyZero(pMC,p,value,col);
		return; 
	}

	if(step<=0.0f)return;

	// Min
	float min=value[0];
	int mini=0;

	if(value[1]<min){ min=value[1];mini=1; }
	if(value[2]<min){ min=value[2];mini=2; }

	// Max
	float max=value[0];
	int maxi=0;

	if(value[1]>max){ max=value[1];maxi=1; }
	if(value[2]>max){ max=value[2];maxi=2; }

	if(fabs(min)>10000000.0)return;		// unsinnige Werte
	if(fabs(max)>10000000.0)return;
//	if(min==max)return;

	int fromi=mini;	if((mini+1)%3 != maxi)fromi=maxi;
	int toi=(fromi+1)%3;
	int gegenueber=(fromi+2)%3;

	// quipotentiallinien zeichnen
	float start=floor(min/step+1.0)*step;

	if((max-start)>1000.0f*step)return;
	if((max-min)<0.0000000001)return;				// EW Null sieht sonst schrottig aus

	float mul_alpha=(value[fromi]-value[toi]);

	if(mul_alpha==0.0f)return;

	mul_alpha=1.0f/mul_alpha;

	for(float f=start;f<=max;f+=step)
	{
		float ax,ay,bx,by;

		float alpha=(f-value[toi])*mul_alpha;

		ax=(float)p[fromi].x*alpha+(float)p[toi].x*(1.0f-alpha);
		ay=(float)p[fromi].y*alpha+(float)p[toi].y*(1.0f-alpha);

		if(value[gegenueber]<f)
		{
			float mul_beta=(value[maxi]-value[gegenueber]);
			if(mul_beta==0.0f)return;

			float beta=(f-value[gegenueber])/mul_beta;

			bx=(float)p[maxi].x*beta+(float)p[gegenueber].x*(1.0f-beta);
			by=(float)p[maxi].y*beta+(float)p[gegenueber].y*(1.0f-beta);
		}
		else
		{
			float mul_beta=(value[mini]-value[gegenueber]);
			if(mul_beta==0.0f)return;
			float beta=(f-value[gegenueber])/mul_beta;

			bx=(float)p[mini].x*beta+(float)p[gegenueber].x*(1.0f-beta);
			by=(float)p[mini].y*beta+(float)p[gegenueber].y*(1.0f-beta);
		}

		if(f>0.000001)pMC->Alpha_LineAA(ax,ay,bx,by,col,255);	
		else
		{
			if(f<-0.000001)pMC->Alpha_LineAA(ax,ay,bx,by,col_neg,255);
			else 
			{
				CColor mix;

				mix.MixColor(col,col_neg,127);
				pMC->Alpha_LineAA(ax,ay,bx,by,mix,255);	
			}
		}
	}
}


void CFEMPlotter::PlotInnerTriangle_OnlyZero( I2DMemoryDC *pMC, CPoint2F p[3], FP_TYPE value[3], CColor col )
{
	// Min
	float min=value[0];
	int mini=0;

	if(value[1]<min){ min=value[1];mini=1; }
	if(value[2]<min){ min=value[2];mini=2; }

	// Max
	float max=value[0];
	int maxi=0;

	if(value[1]>max){ max=value[1];maxi=1; }
	if(value[2]>max){ max=value[2];maxi=2; }

//	if(min==max)return;

	int fromi=mini;	if((mini+1)%3 != maxi)fromi=maxi;
	int toi=(fromi+1)%3;
	int gegenueber=(fromi+2)%3;

	if(max<0)return;
	if(min>0)return;

	if((max-min)<0.0000000001)return;				// EW Null sieht sonst schrottig aus

	// Nulldurchgnge zeichnen
	float f=0;

	float ax,ay,bx,by;

	float mul_alpha=(value[fromi]-value[toi]);

	if(mul_alpha==0.0f)return;

	float alpha=(f-value[toi])/mul_alpha;

	ax=(float)p[fromi].x*alpha+(float)p[toi].x*(1.0f-alpha);
	ay=(float)p[fromi].y*alpha+(float)p[toi].y*(1.0f-alpha);

	if(value[gegenueber]<f)
	{
		float mul_beta=(value[maxi]-value[gegenueber]);
		if(mul_beta==0.0f)return;

		float beta=(f-value[gegenueber])/mul_beta;

		bx=(float)p[maxi].x*beta+(float)p[gegenueber].x*(1.0f-beta);
		by=(float)p[maxi].y*beta+(float)p[gegenueber].y*(1.0f-beta);
	}
	else
	{
		float mul_beta=(value[mini]-value[gegenueber]);
		if(mul_beta==0.0f)return;
		float beta=(f-value[gegenueber])/mul_beta;

		bx=(float)p[mini].x*beta+(float)p[gegenueber].x*(1.0f-beta);
		by=(float)p[mini].y*beta+(float)p[gegenueber].y*(1.0f-beta);
	}

	pMC->Alpha_LineAA(ax,ay,bx,by,col,255);	
}







// Interface I2DDrawObject
// pos ist in Weltkoordinaten
void CFEMPlotter::Draw( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify ) 
{
	try
	{
		assert(pMC);

		CRect2I rect2;

		float mag=m_scale/(1<<inMagnify);
		
		float ho=m_rect.bottom-m_rect.top;					// Hhe

		rect2.left=		((m_rect.left+1-pos.x)>>inMagnify);
		rect2.right=	((m_rect.right-pos.x)>>inMagnify);
		rect2.top=		((m_rect.top+1-pos.y)>>inMagnify);
		rect2.bottom=	((m_rect.bottom-pos.y)>>inMagnify);


	//	pMC->Rect(rect2,3,0xa0a0d0);

		rect2.left-=3;
		rect2.right+=3;
		rect2.top-=3;
		rect2.bottom+=3;
		pMC->FillRect(rect2,0xffffff);						// Hintergrund

		assert(m_femproc.GetFemData().GetCalcCount()<=2);

		// HeatShade zeichnen
		if(m_HeatShadeMode!=0)
			HeatShadeElement(pMC,pos,inMagnify);

		// Potentiallinien zeichnen
		DrawElementPot(pMC,pos,inMagnify);

		// Rand zeichnen
		if(m_displayValues[4])
			DrawEdges(pMC,pos,inMagnify);

		// Elementrnder zeichnen
		if(m_displayValues[2])
			DrawElementNumbers(pMC,pos,inMagnify);

		// Knotennummern zeichnen
		if(m_displayValues[3] && !m_displayValues[6])
			DrawNodeNumbers(pMC,pos,inMagnify);



		// Matrix darstellen
		if(m_displayValues[5] && !m_displayValues[8])
			DrawMatrix(pMC,pos,inMagnify);


	}
	catch(char *txt)
	{
		MessageBox(0,txt,"Error",MB_OK);
	}
	catch(...)
	{
		MessageBox(0,"?","Error",MB_OK);
	}
}





// Methode ist ganz hnlich wie DrawElementPot augebaut
void CFEMPlotter::HeatShadeElement( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify )
{
	float mag=m_scale/(1<<inMagnify);
	float ho=m_rect.bottom-m_rect.top;					// Hhe

	const float Steps=m_DetailLevel;
	float InvSteps=1.0f/Steps;

	int iCalcNr;

	if(m_ResultEV.GetN()==0)
	{
		iCalcNr=m_HeatShadeMode-1;											// U oder V Werte
	}
	else
	{
		iCalcNr=m_EWNumber;
	}

	FP_TYPE ValueMin=0.0f,ValueMax=1.0f;		// fr EW eingestellt

	if(m_ResultEV.GetM()==0)				// fr nicht EW Berechnungen
		m_femproc.GetFemData().GetMinMaxValues(iCalcNr,ValueMin,ValueMax);

	FP_TYPE ValueScale=1.0/(ValueMax-ValueMin);

	{
		int i;

		CFemElement &el=m_femproc.GetFemData().GetTriangleElements();

		// -- Zeichnen der Dreiecks Elemente
		// 
		{
			for(i=0;i<el.GetCount();i++)									// Alle Elemente
			{	
				float x[MAXE],y[MAXE];
				CPoint2F p[MAXE];
				FP_TYPE value[MAXE];

				for (int j=0; j<el.GetType(); j++)
				{
					//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
					//
					int iNodeNr=el.GetNodeNr(i, j)-1;

					assert(iNodeNr >= 0);

					if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
						throw "GetNodeCoord-Error";

					if(m_ResultEV.GetM()==0)		// Potentialwerte in den Knoten
						value[j]=(m_femproc.GetFemData().GetUVValue(iCalcNr,iNodeNr)-ValueMin)*ValueScale;
					  else							// Eigenvektoren ber Matrix
						value[j]=m_ResultEV.P(iNodeNr,iCalcNr);

					p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						          (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
				}

				// Darstellung nur wenn Daten vorhanden
				{
					bool todo=false;

					for(int f=0;f<el.GetType();f++)
						if(value[f]!=0.0){ todo=true;break; }

					if(!todo)continue;		// falls noch nicht berechnet
				}

				if(el.GetType()==6)									// Potentiallinien High Quality
				{
					for(float Eta=0.0f;Eta<=1.01f;Eta+=InvSteps)
					for(float Xi=0.0f;Xi<=1.01f-Eta-InvSteps;Xi+=InvSteps)
					{
						float x[MAXE],y[MAXE];
						FP_TYPE uv[MAXE];
			
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi,Eta,x[0],y[0],uv[0]))
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi+InvSteps,Eta,x[1],y[1],uv[1]))
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi,Eta+InvSteps,x[2],y[2],uv[2]))
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi+InvSteps,Eta+InvSteps,x[3],y[3],uv[3]))
						{
							CPoint2F p2[3];
							FP_TYPE v2[3];

							p2[0]=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[1]=CPoint2F((x[1]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[1]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[2]=CPoint2F((x[2]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[2]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);

							v2[0]=uv[0];v2[1]=uv[1];v2[2]=uv[2];

							HeatShadeInnerTriangle(pMC,p2,v2);

							if(Xi+Eta+InvSteps<1.0)
							{
								p2[0]=CPoint2F((x[3]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
									(ho/m_scale-y[3]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							
								v2[0]=uv[3];

								HeatShadeInnerTriangle(pMC,p2,v2);
							}
						}
					}
				}
				else HeatShadeInnerTriangle(pMC,p,value);
			}
		}

		// -- Zeichnen der Vierecks Elemente
		// 
		{
			CFemElement &el=m_femproc.GetFemData().GetRectangleElements();

			for(i=0;i<el.GetCount();i++)									// Alle Elemente
			{	
				float x[MAXE],y[MAXE];
				CPoint2F p[MAXE];
				FP_TYPE value[MAXE];

				for (int j=0; j<el.GetType(); j++)
				{
					//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
					//
					int iNodeNr=el.GetNodeNr(i, j)-1;

					assert(iNodeNr >= 0);

					if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
						throw "GetNodeCoord-Error";

					if(m_ResultEV.GetM()==0)		// Potentailwerte in den Knoten
						value[j]=(m_femproc.GetFemData().GetUVValue(iCalcNr,iNodeNr)-ValueMin)*ValueScale;
					  else							// Eigenvektoren ber Matrix
						value[j]=m_ResultEV.P(iNodeNr,iCalcNr);
					
					p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						          (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
				}

				// Darstellung nur wenn Daten vorhanden
				{
					bool todo=false;

					for(int f=0;f<el.GetType();f++)
						if(value[f]!=0.0){ todo=true;break; }

					if(!todo)continue;		// falls noch nicht berechnet
				}

//				if(m_displayValues[7] || el.GetType()==8)									// Potentiallinien High Quality
				if(m_DetailLevel!=1)
				{
					for(float Xi=0.0f;Xi<1.0f;Xi+=InvSteps)
					for(float Eta=0.0f;Eta<1.0f;Eta+=InvSteps)
					{
						float x[MAXE],y[MAXE];
						FP_TYPE uv[MAXE];

						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi,Eta,x[0],y[0],uv[0]))
						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi+InvSteps,Eta,x[1],y[1],uv[1]))
						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi+InvSteps,Eta+InvSteps,x[2],y[2],uv[2]))
						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi,Eta+InvSteps,x[3],y[3],uv[3]))
						{
							CPoint2F p2[3];
							FP_TYPE v2[3];

							p2[0]=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[1]=CPoint2F((x[1]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[1]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[2]=CPoint2F((x[2]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[2]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);

							v2[0]=uv[0];v2[1]=uv[1];v2[2]=uv[2];

							HeatShadeInnerTriangle(pMC,p2,v2);

							p2[0]=CPoint2F((x[2]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[2]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[1]=CPoint2F((x[3]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[3]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[2]=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);

							v2[0]=uv[2];v2[1]=uv[3];v2[2]=uv[0];

							HeatShadeInnerTriangle(pMC,p2,v2);
						}
					}
				}
				else														// Potentiallinien low Quality
				{															// starke Nherung
					CPoint2F p2[4];											// 0 ist die Mitte
					FP_TYPE v2[4];

					p2[0]=(p[0]+p[1]+p[2]+p[3])/4;
					v2[0]=(value[0]+value[1]+value[2]+value[3])/4;

					p2[1]=p[1];p2[2]=p[2];p2[3]=p[3];
					v2[1]=value[1];v2[2]=value[2];v2[3]=value[3];
					HeatShadeInnerTriangle(pMC,p2,v2);
					p2[1]=p[2];p2[2]=p[3];p2[3]=p[0];
					v2[1]=value[2];v2[2]=value[3];v2[3]=value[0];
					HeatShadeInnerTriangle(pMC,p2,v2);
					p2[1]=p[3];p2[2]=p[0];p2[3]=p[1];
					v2[1]=value[3];v2[2]=value[0];v2[3]=value[1];
					HeatShadeInnerTriangle(pMC,p2,v2);
					p2[1]=p[0];p2[2]=p[1];p2[3]=p[2];
					v2[1]=value[0];v2[2]=value[1];v2[3]=value[2];
					HeatShadeInnerTriangle(pMC,p2,v2);
				}

			}
		}
	}
}





void CFEMPlotter::DrawElementPot( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify )
{
	float mag=m_scale/(1<<inMagnify);
	float ho=m_rect.bottom-m_rect.top;					// Hhe

	const float Steps=m_DetailLevel;
	float InvSteps=1.0f/Steps;

	int from_calc,to_calc;

	if(m_ResultEV.GetN()==0)
	{
		from_calc=0;
		to_calc=m_femproc.GetFemData().GetCalcCount();
	}
	else
	{
		from_calc=m_EWNumber;to_calc=from_calc+1;
	}


	for(int iCalcNr=from_calc;iCalcNr<to_calc;iCalcNr++)
	{
		CColor col,col_neg;
		float stepper;

		if(m_ResultEV.GetN()==0)											// keine EW
		{
			if(iCalcNr==0)stepper=m_StepperU;
				else stepper=m_StepperV;

			if(!m_displayValues[iCalcNr])continue;							// nur wenn Checkbox an ist

			switch(iCalcNr)													// Innen-Farbe
			{
				case 0:	col_neg=col=CColor(0x0000ff);break;
				case 1:	col_neg=col=CColor(0xff0000);break;

				default: break;
			}
		}
		else																// mit EW
		{	
			stepper=m_StepperU;
		
			if(!m_displayValues[0])continue;								// nur wenn Checkbox an ist

			col_neg=CColor(0x0088ff);
			col=CColor(0xff8800);

			if(m_displayValues[9])
				col=CColor(0xff4040);
		}

		int i;

		CFemElement &el=m_femproc.GetFemData().GetTriangleElements();

		// -- Zeichnen der Dreiecks Elemente
		// 
		{
			for(i=0;i<el.GetCount();i++)									// Alle Elemente
			{	
				float x[MAXE],y[MAXE];
				CPoint2F p[MAXE];
				FP_TYPE value[MAXE];

				for (int j=0; j<el.GetType(); j++)
				{
					//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
					//
					int iNodeNr=el.GetNodeNr(i, j)-1;

					assert(iNodeNr >= 0);

					if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
						throw "GetNodeCoord-Error";

					if(m_ResultEV.GetM()==0)		// Potentialwerte in den Knoten
						value[j]=m_femproc.GetFemData().GetUVValue(iCalcNr,iNodeNr);
					  else							// Eigenvektoren ber Matrix
						value[j]=m_ResultEV.P(iNodeNr,iCalcNr);

					p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						          (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
				}

				// Darstellung nur wenn Daten vorhanden
				{
					bool todo=false;

					for(int f=0;f<el.GetType();f++)
						if(value[f]!=0.0){ todo=true;break; }

					if(!todo)continue;		// falls noch nicht berechnet
				}

				if(el.GetType()==6)									// Potentiallinien High Quality
				{
					for(float Eta=0.0f;Eta<=1.01f;Eta+=InvSteps)
					for(float Xi=0.0f;Xi<=1.01f-Eta-InvSteps;Xi+=InvSteps)
					{
						float x[MAXE],y[MAXE];
						FP_TYPE uv[MAXE];
			
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi,Eta,x[0],y[0],uv[0]))
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi+InvSteps,Eta,x[1],y[1],uv[1]))
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi,Eta+InvSteps,x[2],y[2],uv[2]))
						if(m_femproc.GetFemData().UVInterpolationTriangle(value,i,Xi+InvSteps,Eta+InvSteps,x[3],y[3],uv[3]))
						{
							CPoint2F p2[3];
							FP_TYPE v2[3];

							p2[0]=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[1]=CPoint2F((x[1]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[1]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[2]=CPoint2F((x[2]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[2]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);

							v2[0]=uv[0];v2[1]=uv[1];v2[2]=uv[2];

							PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);

							if(Xi+Eta+InvSteps<1.0)
							{
								p2[0]=CPoint2F((x[3]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
									(ho/m_scale-y[3]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							
								v2[0]=uv[3];

								PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);
							}
						}
					}
				}
				else PlotInnerTriangle(pMC,p,value,stepper,col,col_neg);
			}
		}

		// -- Zeichnen der Vierecks Elemente
		// 
		{
			CFemElement &el=m_femproc.GetFemData().GetRectangleElements();

			for(i=0;i<el.GetCount();i++)									// Alle Elemente
			{	
				float x[MAXE],y[MAXE];
				CPoint2F p[MAXE];
				FP_TYPE value[MAXE];

				for (int j=0; j<el.GetType(); j++)
				{
					//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
					//
					int iNodeNr=el.GetNodeNr(i, j)-1;

					assert(iNodeNr >= 0);

					if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
						throw "GetNodeCoord-Error";

					if(m_ResultEV.GetM()==0)		// Potentailwerte in den Knoten
						value[j]=m_femproc.GetFemData().GetUVValue(iCalcNr,iNodeNr);
					  else							// Eigenvektoren ber Matrix
						value[j]=m_ResultEV.P(iNodeNr,iCalcNr);
					
					p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						          (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
				}

				// Darstellung nur wenn Daten vorhanden
				{
					bool todo=false;

					for(int f=0;f<el.GetType();f++)
						if(value[f]!=0.0){ todo=true;break; }

					if(!todo)continue;		// falls noch nicht berechnet
				}

//				if(m_displayValues[7] || el.GetType()==8)									// Potentiallinien High Quality
				if(m_DetailLevel!=1)
				{
					for(float Xi=0.0f;Xi<1.0f;Xi+=InvSteps)
					for(float Eta=0.0f;Eta<1.0f;Eta+=InvSteps)
					{
						float x[MAXE],y[MAXE];
						FP_TYPE uv[MAXE];

						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi,Eta,x[0],y[0],uv[0]))
						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi+InvSteps,Eta,x[1],y[1],uv[1]))
						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi+InvSteps,Eta+InvSteps,x[2],y[2],uv[2]))
						if(m_femproc.GetFemData().UVInterpolationRectangle(value,i,Xi,Eta+InvSteps,x[3],y[3],uv[3]))
						{
							CPoint2F p2[3];
							FP_TYPE v2[3];

							p2[0]=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[1]=CPoint2F((x[1]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[1]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[2]=CPoint2F((x[2]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[2]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);

							v2[0]=uv[0];v2[1]=uv[1];v2[2]=uv[2];

							PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);

							p2[0]=CPoint2F((x[2]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[2]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[1]=CPoint2F((x[3]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[3]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
							p2[2]=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
								(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);

							v2[0]=uv[2];v2[1]=uv[3];v2[2]=uv[0];

							PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);
						}
					}
				}
				else														// Potentiallinien low Quality
				{															// starke Nherung
					CPoint2F p2[4];											// 0 ist die Mitte
					FP_TYPE v2[4];

					p2[0]=(p[0]+p[1]+p[2]+p[3])/4;
					v2[0]=(value[0]+value[1]+value[2]+value[3])/4;

					p2[1]=p[1];p2[2]=p[2];p2[3]=p[3];
					v2[1]=value[1];v2[2]=value[2];v2[3]=value[3];
					PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);
					p2[1]=p[2];p2[2]=p[3];p2[3]=p[0];
					v2[1]=value[2];v2[2]=value[3];v2[3]=value[0];
					PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);
					p2[1]=p[3];p2[2]=p[0];p2[3]=p[1];
					v2[1]=value[3];v2[2]=value[0];v2[3]=value[1];
					PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);
					p2[1]=p[0];p2[2]=p[1];p2[3]=p[2];
					v2[1]=value[0];v2[2]=value[1];v2[3]=value[2];
					PlotInnerTriangle(pMC,p2,v2,stepper,col,col_neg);
				}

			}
		}
	}
}




void CFEMPlotter::DrawEdges( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify )
{
	const float Steps=m_DetailLevel;
	float InvSteps=1.0f/Steps;

	UBYTE density= m_displayValues[10] ? 64 : 255;	// Transparent
	float mag=m_scale/(1<<inMagnify);
	float ho=m_rect.bottom-m_rect.top;					// Hhe

	CColor rcol=CColor(0x000000);					// Randfarbe

	int i;

	// -- Zeichnen der Dreiecks Elemente
	// 
	{
		CFemElement &el=m_femproc.GetFemData().GetTriangleElements();

		for(i=0;i<el.GetCount();i++)									// Alle Elemente
		{	
			// Rand zeichnen
			if(el.GetType()==3)											// eET_TRIANGLE
			{
				float x[3],y[3];
				CPoint2F p[3];

				for (int j=0; j<3; j++)
				{
					//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
					//
					int iNodeNr=el.GetNodeNr(i, j)-1;

					assert(iNodeNr >= 0);

					if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
						throw "GetNodeCoord-Error";

					// fuck test
/*
					FP_TYPE vx,vy;
					
					vx=m_femproc.GetFemData().GetUVValue(0,iNodeNr);
					vy=m_femproc.GetFemData().GetUVValue(1,iNodeNr);

					x[j]+=vx*10000.0;
					y[j]+=vy*10000.0;
*/
					p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						          (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
				}

				pMC->Alpha_LineAA(p[0].x,p[0].y,p[1].x,p[1].y,rcol,density);
				pMC->Alpha_LineAA(p[1].x,p[1].y,p[2].x,p[2].y,rcol,density);
				pMC->Alpha_LineAA(p[2].x,p[2].y,p[0].x,p[0].y,rcol,density);
			}
			else														// eET_TRIANGLE2
			{
				CPoint2F oldpun(0,0),start;

				// effizienter implementierbar
				for(int j=0;j<3;j++)
				for(float lauf=0.0f;lauf<1.0f;lauf+=InvSteps)
				{
					float Xi,Eta;

					switch(j)
					{
						case 0: Xi=lauf;Eta=0.0f;break;
						case 1: Xi=1.0f-lauf;Eta=lauf;break;
						case 2: Xi=0.0f;Eta=1.0f-lauf;break;
						default: assert(0);
					}

					float x[1],y[1];
					FP_TYPE uv[1];
			
					if(!m_femproc.GetFemData().UVInterpolationTriangle(0,i,Xi,Eta,x[0],y[0],uv[0]))break;

					CPoint2F newpun;

					newpun=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
					
					if(j!=0 || lauf!=0.0f)pMC->Alpha_LineAA(oldpun.x,oldpun.y,newpun.x,newpun.y,rcol,density);
						else start=newpun;

					oldpun=newpun;
				}
				pMC->Alpha_LineAA(oldpun.x,oldpun.y,start.x,start.y,rcol,density);
			}
		}
	}


	// -- Zeichnen der Vierecks Elemente
	// 
	{
		CFemElement &el=m_femproc.GetFemData().GetRectangleElements();

		for(i=0;i<el.GetCount();i++)									// Alle Elemente
		{	
			// Rand zeichnen
			if(el.GetType()==4)											// eET_PARALLELOGRAMM
			{
				float x[4],y[4];
				CPoint2F p[4];
				
				for (int j=0; j<4; j++)
				{
					//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
					//
					int iNodeNr=el.GetNodeNr(i, j)-1;

					assert(iNodeNr >= 0);

					if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
						throw "GetNodeCoord-Error";

					p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						          (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
				}
					
				pMC->Alpha_LineAA(p[0].x,p[0].y,p[1].x,p[1].y,rcol,density);
				pMC->Alpha_LineAA(p[1].x,p[1].y,p[2].x,p[2].y,rcol,density);
				pMC->Alpha_LineAA(p[2].x,p[2].y,p[3].x,p[3].y,rcol,density);
				pMC->Alpha_LineAA(p[3].x,p[3].y,p[0].x,p[0].y,rcol,density);
			}
			else														// eET_PARALLELOGRAMM2
			{
				CPoint2F oldpun(0,0),start;

				// effizienter implementierbar
				for(int j=0;j<4;j++)
				for(float lauf=0.0f;lauf<1.0f;lauf+=InvSteps)
				{
					float Xi,Eta;

					switch(j)
					{
						case 0: Xi=lauf;Eta=0.0f;break;
						case 1: Xi=1.0f;Eta=lauf;break;
						case 2: Xi=1.0f-lauf;Eta=1.0f;break;
						case 3: Xi=0.0f;Eta=1.0f-lauf;break;
						default: assert(0);
					}

					float x[1],y[1];
					FP_TYPE uv[1];
			
					if(!m_femproc.GetFemData().UVInterpolationRectangle(0,i,Xi,Eta,x[0],y[0],uv[0]))break;

					CPoint2F newpun;

					newpun=CPoint2F((x[0]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
						(ho/m_scale-y[0]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
					
					if(j==0 && lauf==0.0f)start=newpun;
						else pMC->Alpha_LineAA(oldpun.x,oldpun.y,newpun.x,newpun.y,rcol,density);

					oldpun=newpun;
				}
				pMC->Alpha_LineAA(oldpun.x,oldpun.y,start.x,start.y,rcol,density);
			}
		}
	}
}

void CFEMPlotter::DrawElementNumbers( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify )
{
	float mag=m_scale/(1<<inMagnify);
	float ho=m_rect.bottom-m_rect.top;					// Hhe

	int i;
	int tri_count=m_femproc.GetFemData().GetTriangleElements().GetCount();

	// -- Zeichnen der Dreiecks Elemente
	// 
	HDC hdc=pMC->GetHDC();	

	SetTextColor(hdc,0x808000);
	SetBkMode(hdc,TRANSPARENT);
	SetTextAlign(hdc,TA_LEFT | TA_NOUPDATECP | TA_TOP);
	{
		CFemElement &el=m_femproc.GetFemData().GetTriangleElements();

		for(i=0;i<tri_count;i++)									// Alle Elemente
		{	
			float x[3],y[3];
			CPoint2F p[3];

			for (int j=0; j<3; j++)
			{
				//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
				//
				int iNodeNr=el.GetNodeNr(i, j)-1;

				assert(iNodeNr >= 0);

				if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
					throw "GetNodeCoord-Error";

				p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
					         (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
			}

			float mx=(p[0].x+p[1].x+p[2].x)/3.0f;
			float my=(p[0].y+p[1].y+p[2].y)/3.0f;

			char str[80];
			sprintf(str,"%d",i+1);

			// Elementnummer zeichnen
			SIZE s;
			GetTextExtentPoint32(hdc,str,strlen(str),&s);
			TextOut(hdc,mx-s.cx*0.5f,my-s.cy*0.5f,str,strlen(str));
		}
	}


	// -- Zeichnen der Vierecks Elemente
	// 
	{
		CFemElement &el=m_femproc.GetFemData().GetRectangleElements();

		for(i=0;i<el.GetCount();i++)									// Alle Elemente
		{	
			float x[4],y[4];
			CPoint2F p[4];
			
			for (int j=0; j<4; j++)
			{
				//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
				//
				int iNodeNr=el.GetNodeNr(i, j)-1;

				assert(iNodeNr >= 0);

				if (m_femproc.GetFemData().GetNodeCoord(iNodeNr, x[j], y[j]) == false)
					throw "GetNodeCoord-Error";

				p[j]=CPoint2F((x[j]-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag,
					          (ho/m_scale-y[j]-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag);
			}

			float mx=(p[0].x+p[1].x+p[2].x+p[3].x)/4.0f;
			float my=(p[0].y+p[1].y+p[2].y+p[3].y)/4.0f;

			char str[80];
			sprintf(str,"%d",tri_count+i+1);

			// Elementnummer zeichnen
			SIZE s;
			GetTextExtentPoint32(hdc,str,strlen(str),&s);
			TextOut(hdc,mx-s.cx*0.5f,my-s.cy*0.5f,str,strlen(str));
		}
	}
	pMC->ReleaseHDC(hdc);
}




void CFEMPlotter::DrawNodeNumbers( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify )
{
	float mag=m_scale/(1<<inMagnify);
	float ho=m_rect.bottom-m_rect.top;					// Hhe

	// Schwerpunkt ermitten ( fr einfache Berechnung der besten Position fr den Text )
	int nodecount=m_femproc.GetFemData().GetNodeCount();
	float midx=0.0f,midy=0.0;

	for(int i=0;i<nodecount;i++)
	{
		float x,y;

		if (m_femproc.GetFemData().GetNodeCoord(i, x, y) == false)
					throw "GetNodeCoord-Error";

		midx+=x;midy+=y;
	}
	midx=(midx/nodecount-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag;
	midy=(ho/m_scale-midy/nodecount-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag;

	HDC hdc=pMC->GetHDC();	
	SetTextColor(hdc,0x000080);
	SetBkMode(hdc,TRANSPARENT);
	SetTextAlign(hdc,TA_LEFT | TA_NOUPDATECP | TA_TOP);

	for(i=0;i<nodecount;i++)
	{
		float x,y;

		if (m_femproc.GetFemData().GetNodeCoord(i, x, y) == false)
					throw "GetNodeCoord-Error";

		float mx=(x-m_subx+m_rect.left/m_scale-pos.x/m_scale)*mag;
		float my=(ho/m_scale-y-m_suby+m_rect.top/m_scale-pos.y/m_scale)*mag;
		
		float normalx=mx-midx;
		float normaly=my-midy;
		float normal=sqrt(normalx*normalx+normaly*normaly);

		if(normal>0.1f)
		{
			normalx/=normal;
			normaly/=normal;
		}

		char str[80];

		sprintf(str,"%d",i+1);
//		sprintf(str,"%d",(int)(m_ResultEV.P(i,m_EWNumber)*100.0f));	// EV Werte ausgeben


		// Knotennummer zeichnen
		SIZE s;
		GetTextExtentPoint32(hdc,str,strlen(str),&s);
		TextOut(hdc,mx+normalx*11.0f-s.cx*0.5f,my+normaly*11.0f-s.cy*0.5f,str,strlen(str));
	}
	pMC->ReleaseHDC(hdc);
}	





void CFEMPlotter::DrawMatrix( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify )
{
	float mag=m_scale/(1<<inMagnify);

	float size=m_rect.Width();

	if(size>m_rect.Height())size=m_rect.Height();


	float el_width=size/(float)(m_femproc.GetFemData().GetNodeCount())/m_scale;
	float el_height=size/(float)(m_femproc.GetFemData().GetNodeCount())/m_scale;

	// -- Zeichnen der Dreiecks Elemente
	// 
	{
		CFemElement &el=m_femproc.GetFemData().GetTriangleElements();
		int typ=el.GetType();

		for(int i=0;i<el.GetCount();i++)									// Alle Elemente
		{	
			for (int j=0; j<typ; j++)
			for (int g=0; g<typ; g++)
			{
				//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
				//
				float x=el.GetNodeNr(i, j)-1;
				float y=el.GetNodeNr(i, g)-1;

				CPoint2F p[4];

				p[0].x=(el_width*x + m_rect.left/m_scale-pos.x/m_scale)*mag;
				p[0].y=(el_height*y + m_rect.top/m_scale-pos.y/m_scale)*mag;

				p[1].x=p[0].x + el_width*0.8f*mag;
				p[1].y=p[0].y;
				p[2].x=p[0].x + el_width*0.8f*mag;
				p[2].y=p[0].y + el_height*0.8f*mag;
				p[3].x=p[0].x;
				p[3].y=p[0].y + el_height*0.8f*mag;

				pMC->PolyAA(p,4,0x000000);
			}
		}
	}

	// -- Zeichnen der Vierecks Elemente
	// 
	{
		CFemElement &el=m_femproc.GetFemData().GetRectangleElements();
		int typ=el.GetType();

		for(int i=0;i<el.GetCount();i++)									// Alle Elemente
		{	
			for (int j=0; j<typ; j++)
			for (int g=0; g<typ; g++)
			{
				//-- -1 um Nullindizierte Knotenpunktsnummer zu erhalten
				//
				float x=el.GetNodeNr(i, j)-1;
				float y=el.GetNodeNr(i, g)-1;

				CPoint2F p[4];

				p[0].x=(el_width*x + m_rect.left/m_scale-pos.x/m_scale)*mag;
				p[0].y=(el_height*y + m_rect.top/m_scale-pos.y/m_scale)*mag;

				p[1].x=p[0].x + el_width*0.8f*mag;
				p[1].y=p[0].y;
				p[2].x=p[0].x + el_width*0.8f*mag;
				p[2].y=p[0].y + el_height*0.8f*mag;
				p[3].x=p[0].x;
				p[3].y=p[0].y + el_height*0.8f*mag;

				pMC->PolyAA(p,4,0x000000);
			}
		}
	}
}


void CFEMPlotter::MarkedDraw( I2DMemoryDC *pMC, int r, CPoint2I pos, int inMagnify )
{
	assert(pMC);

	CRect2I rect2;
	CRect2I rect;

	rect.left=		((m_rect.left+1-pos.x)>>inMagnify);
	rect.right=		((m_rect.right-pos.x)>>inMagnify);
	rect.top=		((m_rect.top+1-pos.y)>>inMagnify);
	rect.bottom=	((m_rect.bottom-pos.y)>>inMagnify);

	CPoint2I lefttop=GetWorldPosition();

	pos-=lefttop-CPoint2I(rect.left,rect.top);

	rect2.left=rect.left-r;
	rect2.right=rect.left+r;
	rect2.top=rect.top-r;
	rect2.bottom=rect.top+r;
	pMC->FillRect(rect2,0x0,128);
	rect2.left=rect.right-r;
	rect2.right=rect.right+r;
	rect2.top=rect.top-r;
	rect2.bottom=rect.top+r;
	pMC->FillRect(rect2,0x0,128);
	rect2.left=rect.left-r;
	rect2.right=rect.left+r;
	rect2.top=rect.bottom-r;
	rect2.bottom=rect.bottom+r;
	pMC->FillRect(rect2,0x0,128);
	rect2.left=rect.right-r;
	rect2.right=rect.right+r;
	rect2.top=rect.bottom-r;
	rect2.bottom=rect.bottom+r;
	pMC->FillRect(rect2,0x0,128);
}




ULONG CFEMPlotter::IsAnchorPointAt( int r, CPoint2I xy )	// nur fr markierte Objekte, 0, wenn dort kein Punkt, sonst ist dort ein Angriffspunkt (Skalieren, ...)
{
/*	CRect2I rect2;

	CPoint2I lefttop=GetWorldPosition();

	CPoint2I pos=CPoint2I(m_rect.left,m_rect.top)-lefttop;

	rect2.left=m_rect.left-r-pos.x;
	rect2.right=m_rect.left+r-pos.x;
	rect2.top=m_rect.top-r-pos.y;
	rect2.bottom=m_rect.top+r-pos.y;
	if(rect2.PtInRect(xy))return(1);
	rect2.left=m_rect.right-r-pos.x;
	rect2.right=m_rect.right+r-pos.x;
	rect2.top=m_rect.top-r-pos.y;
	rect2.bottom=m_rect.top+r-pos.y;
	if(rect2.PtInRect(xy))return(2);
	rect2.left=m_rect.left-r-pos.x;
	rect2.right=m_rect.left+r-pos.x;
	rect2.top=m_rect.bottom-r-pos.y;
	rect2.bottom=m_rect.bottom+r-pos.y;
	if(rect2.PtInRect(xy))return(3);
	rect2.left=m_rect.right-r-pos.x;
	rect2.right=m_rect.right+r-pos.x;
	rect2.top=m_rect.bottom-r-pos.y;
	rect2.bottom=m_rect.bottom+r-pos.y;
	if(rect2.PtInRect(xy))return(4);
*/
	return(0);
}





I2DDraw *CFEMPlotter::GetTopObjectAt( CPoint2I xy )
{
	if(IsObjectAt(xy))return(this);

	return(CDocObject::GetTopObjectAt(xy));
/*
	ISceneGraphObject *p=GetIn().GetPtr();
	I2DDraw *top=this;

	for(;p!=0;p=p->GetNext().GetPtr())
	{
		I2DDraw *m=(I2DDraw *)p->QueryInterface(eI2DDraw);

		I2DDraw *here=m->GetTopObjectAt(xy-CPoint2I(m_rect.left,m_rect.top));

		if(here)top=here;
	}
	return(top);
*/
}

I2DDraw *CFEMPlotter::GetTopAnchorPointAt( int r, CPoint2I xy, ULONG &anchorid )
{
	anchorid=IsAnchorPointAt(r,xy);

	if(anchorid!=0)return(this);

	return(CDocObject::GetTopAnchorPointAt(r,xy,anchorid));
/*
	ISceneGraphObject *p=GetIn().GetPtr();
	I2DDraw *top=this;

	for(;p!=0;p=p->GetNext().GetPtr())
	{
		I2DDraw *m=(I2DDraw *)p->QueryInterface(eI2DDraw);

		I2DDraw *here=m->GetTopObjectAt(xy-CPoint2I(m_rect.left,m_rect.top));

		if(here)top=here;
	}
	return(top);
*/
}


void CFEMPlotter::OnContextMenu( IPopUpMenu *k, CPoint2I xy )
{
	IPopUpMenu *p=MakeCreateMenu(k,xy);


	k->AppendMenu("Create",p);

	{
		char idstr[80];

		sprintf(idstr,"Delete(%p)",(ISceneGraphObject *)this);

		k->AppendItem("Delete",idstr);
	}
	

	k->TrackPopUpMenu();

	p->Release();
}


// hier wird nur das zu serialisierende Objekt betrachtet,
// nicht Objekte auf die das Objekt werweist oder beinhaltet.
void CFEMPlotter::Serialize( CSerializeArchivePointer &ar )
{
	// if(ar.GetMode==CSAM_saveADEF) ....

/*	ar.Dint(m_rect.left,"left");
	ar.Dint(m_rect.top,"top");
	ar.Dint(m_rect.bottom,"bottom");
	ar.Dint(m_rect.right,"right");

*/
	CDocObject::Serialize(ar);		// Vaterobjekt serialisieren (nur wenn erwnscht)
}



void CFEMPlotter::OnControlMessage( struct CRGUI_ControlMessage &m )
{
	bool redraw=false;
	
	try
	{
		switch(m.Cmd)
		{
			case cmd_ValueChanged:
				{
	//				bool redraw=false;
					ULONG val;

					redraw=true;
					
					switch(m.Caller_UserId)
					{
						case eDisplayEdgesHB:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[10]=val==1;
							}
							break;

						case eDisplayZeroU:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[9]=val==1;
							}
							break;

						case eDisplayU:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[0]=val==1;
							}
							break;
						case eDisplayV:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[1]=val==1;
							}
							break;
						case eDisplayElements:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[2]=val==1;
							}
							break;
						case eDisplayNodes:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[3]=val==1;
							}
							break;
						case eDisplayEdges:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[4]=val==1;
							}
							break;
						case eDisplayMatrix:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[5]=val==1;
							}
							break;
						case eDisplayNodesOptimized:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[6]=val==1;
							}
							break;
						case eDisplayMatrixOptimized:
							{
								m.Caller_Ptr->GetValue(val);m_displayValues[8]=val==1;
							}
							break;
						case eStepperU:
							{
								char str[256];

								m.Caller_Ptr->GetText(str,256);

								if(sscanf(str,"%f",&m_StepperU)!=1)
									m_StepperU=1;
							}
							break;
						case eStepperV:
							{
								char str[256];

								m.Caller_Ptr->GetText(str,256);

								if(sscanf(str,"%f",&m_StepperV)!=1)
									m_StepperV=1;
							}
							break;
					}
				}
				break;

			case cmd_Trigger:								// z.B. Button pressed oder Menauswahl
				switch(m.Caller_UserId)
				{
					case eLoeschen:				m_MainGUIDialog->ReleaseWindow();ReleaseObject();Release();break;
					case eOpenFEMIN:			OpenFEMIN();break;
					case eOpenFEMOUT:			OpenFEMOUT();break;

					case eDetails1:				redraw=true;m_DetailLevel=1;break;
					case eDetails4:				redraw=true;m_DetailLevel=4;break;
					case eDetails8:				redraw=true;m_DetailLevel=8;break;
					case eDetails16:			redraw=true;m_DetailLevel=16;break;
					case eDetails64:			redraw=true;m_DetailLevel=64;break;

					case eHeatShadeNone:		redraw=true;m_HeatShadeMode=0;break;
					case eHeatShadeU:			redraw=true;m_HeatShadeMode=1;break;
					case eHeatShadeV:			redraw=true;m_HeatShadeMode=2;break;

					case eEWDecrese:
						if(m_EWNumber>0){ m_EWNumber--;redraw=true;UpdateControlWindow(); }
						// todo eEWEWValue
						break;

					case eEWIncrease:
						if(m_EWNumber+1<m_ResultEW.GetN()){ m_EWNumber++;redraw=true;UpdateControlWindow(); }
						// todo eEWEWValue				
						break;

					default:	MessageBox(0,"Not Done","Message",MB_OK);
				}

				break;

			case cmd_DialogMessage:							// Debugging
	//			sprintf(str,"ApplicationGetsMessage: Dialog Message %s",m.String_Ptr);
	//			AddConsoleLine(str);
				break;

			case cmd_Open:			// fr Tree
				break;

			case cmd_Close:			// fr Tree
				break;

			default:
				MessageBox(0,"Unknown Control Message","Error",MB_OK);
		}

		if(redraw)
		{
			// Redraw
			CHandle<ISceneGraphObject> par=GetTopmostParent();
			if(par)
			{
				CHandle<ISceneGraphRootObject> ro=QUERYINTERFACE(ISceneGraphRootObject,par);
				if(ro)
				{
					ro->RedrawAll();
				}
			}
		}
	}
	catch(char *txt)
	{
		MessageBox(0,txt,"Error",MB_OK);
	}
	catch(...)
	{
		MessageBox(0,"?","Error",MB_OK);
	}
}



void CFEMPlotter::OpenFEMIN( void )
{
	char back[1024];
	if(GetCurrentDirectory(1024,back)==0)
	{
		ERR->Add("CFEMPlotter::OpenFEMIN failed");
		MessageBox(0,"GetCurrentDirectory","failed",MB_OK);
		return;
	}

	char filestr[1024];		filestr[0]=0;

	CGUIAppData gapp;

	g_App->GetGUIAppData(gapp);

	OPENFILENAME ofn;

	ofn.lStructSize=sizeof(OPENFILENAME);
	ofn.hwndOwner=gapp.m_hMainFrame;
	ofn.hInstance=gapp.m_hInstance;
	ofn.lpstrFilter="Alle Dateien  (*.*)\0*.*\0";
	ofn.lpstrCustomFilter = (LPTSTR)NULL;
	ofn.nMaxCustFilter = 0L;
	ofn.nFilterIndex = 1L;
	ofn.lpstrInitialDir=NULL;
	ofn.lpstrTitle="Whlen der FEMIN Quelldatei";

	ofn.lpstrFileTitle=0;
	ofn.nMaxFileTitle=0;

	ofn.lpstrFile=filestr;
	ofn.nMaxFile=sizeof(filestr);

	ofn.lCustData=0;
	ofn.lpstrDefExt=".txt";
	ofn.Flags=OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_ENABLESIZING;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;

	if(GetOpenFileName(&ofn))
	{
		try
		{

			InitVar();

			m_femproc.LoadFemIn(filestr,true,&m_ResultEW,&m_ResultEV);

			if(m_ResultEV.GetN()!=0)
			{
				if(m_ResultEV.GetN()!=m_femproc.GetFemData().GetNodeCount())
				{
					m_ResultEV.FreeData();
					m_ResultEW.FreeData();
					MessageBox(0,"Out Datei hat mehr oder weniger Werte als Knoten im Gebiet der In Datei sind","Error",MB_OK);
				}
				if(m_ResultEV.GetM()!=m_ResultEW.GetN())
				{
					throw "Anzahl der Eigenvektoren stimmt nicht mit der Anzahl der Eigenwerte berein";
				}
			}


			float minx,maxx,miny,maxy;

			m_femproc.GetMinMaxCoord(minx,maxx,miny,maxy);

			float br=(maxx-minx)*m_scale;
			float ho=(maxy-miny)*m_scale;

			m_subx=(maxx+minx)/2			- (maxx-minx)/2;
			m_suby=-(maxy+miny)/2			+ (maxy-miny)/2;

			SetRect(CRect2I(m_rect.left,m_rect.top,m_rect.left+br,m_rect.top+ho));

			CHandle<ISceneGraphObject> par=GetTopmostParent();
			assert(par);
			CHandle<ISceneGraphRootObject> ro=QUERYINTERFACE(ISceneGraphRootObject,par);
			assert(ro);
			ro->RedrawAll();

			int mode=(m_femproc.GetFemData().GetFEMVersion()<=3) ? 1 : 2;
			if(m_femproc.GetFemData().GetFEMVersion()==6)mode=1;

			BuildDialog(mode);

			if(m_femproc.GetFemData().GetFEMVersion()<=3) 
				SetUPotValue(1.0);							// meist grer
			  else 
				SetUPotValue(0.1);							// meist zwischen -1 und 1 normiert


			UpdateControlWindow();
		}
		catch(CFemDataError &e)
		{
//			cout << e.GetLineNr() << " Error: " << e.GetMessage() << endl;
			ERR->Add("Parse Error at line: %d     Error: %s",e.GetLineNr(),e.GetErrorMessage());
		}



	}

	SetCurrentDirectory(back);
}



void CFEMPlotter::SetUPotValue( double inVal )
{
	char str[256];
	IWinGUIButton *n;

	n=m_MainGUIDialog->GetPtrFromUserId(eStepperU);assert(n);
	sprintf(str,"%.2f",inVal);
	n->SetText(str);
}




void CFEMPlotter::OpenFEMOUT( void )
{

	char back[1024];
	if(GetCurrentDirectory(1024,back)==0)
	{
		ERR->Add("CFEMPlotter::OpenFEMOUT failed");
		MessageBox(0,"GetCurrentDirectory","failed",MB_OK);
		return;
	}


	char filestr[1024];		filestr[0]=0;

	CGUIAppData gapp;

	g_App->GetGUIAppData(gapp);

	OPENFILENAME ofn;

	ofn.lStructSize=sizeof(OPENFILENAME);
	ofn.hwndOwner=gapp.m_hMainFrame;
	ofn.hInstance=gapp.m_hInstance;
	ofn.lpstrFilter="Alle Dateien  (*.*)\0*.*\0";
	ofn.lpstrCustomFilter = (LPTSTR)NULL;
	ofn.nMaxCustFilter = 0L;
	ofn.nFilterIndex = 1L;
	ofn.lpstrInitialDir=NULL;
	ofn.lpstrTitle="Whlen der FEMOUT Quelldatei";

	ofn.lpstrFileTitle=0;
	ofn.nMaxFileTitle=0;

	ofn.lpstrFile=filestr;
	ofn.nMaxFile=sizeof(filestr);

	ofn.lCustData=0;
	ofn.lpstrDefExt=".txt";
	ofn.Flags=OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_ENABLESIZING;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;

	if(GetOpenFileName(&ofn))
	{
		try
		{
			m_femproc.GetFemData().LoadFemOut(filestr);

			CHandle<ISceneGraphObject> par=GetTopmostParent();
			assert(par);
			CHandle<ISceneGraphRootObject> ro=QUERYINTERFACE(ISceneGraphRootObject,par);
			assert(ro);
			ro->RedrawAll();

		}
		catch(CFemDataError &e)
		{
			ERR->Add("Parse Error at line: %d     Error: %s",e.GetLineNr(),e.GetErrorMessage());
		}
	}

	SetCurrentDirectory(back);
}





// nicht skalierbar
void CFEMPlotter::ScaleObject( CPoint2I rel )
{
}







void CFEMPlotter::UpdateControlWindow( void )
{
	char str[256];
	IWinGUIButton *n;

	int nc=m_femproc.GetFemData().GetNodeCount();

	n=m_MainGUIDialog->GetPtrFromUserId(eNodeCountOutput);assert(n);
	sprintf(str,"Knotennanzahl: %d",nc);
	n->SetText(str);

	if(m_ResultEW.GetN()!=0)
	{
		n=m_MainGUIDialog->GetPtrFromUserId(eEWNumberOutput);assert(n);
		sprintf(str,"%.3d/%.3d",m_EWNumber+1,m_ResultEW.GetN());	
		n->SetText(str);

		n=m_MainGUIDialog->GetPtrFromUserId(eEWValueOutput);assert(n);
		sprintf(str,"%.5f",m_ResultEW.P(m_EWNumber));	
		n->SetText(str);
	}
	else
	{
		n=m_MainGUIDialog->GetPtrFromUserId(eEWNumberOutput);
		if(n)
		{
			sprintf(str,"xxx/xxx");	
			n->SetText(str);
		}

		n=m_MainGUIDialog->GetPtrFromUserId(eEWValueOutput);
		if(n)
		{
			sprintf(str,"x.xxxxx");	
			n->SetText(str);
		}
	}
}
