// CFEMPlotter.h: interface for the CFEMPlotter class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CFEMPlotter_H__8E259A54_E0B0_4C17_B1A6_C443A6B154DF__INCLUDED_)
#define AFX_CFEMPlotter_H__8E259A54_E0B0_4C17_B1A6_C443A6B154DF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif

#include "..\..\SHARED\CComponentBase.h"

#include "IFEMPlotter.h"
#include "..\SimpleBox\DocObject.h"					// CDocObject
#include "..\..\PlugInSource\WinGUI\IWinGUI.h"		// IGUIDialogCallback

#include "DVA\FemProcessor.h"						// CFemProcessor
	
class CFEMPlotter : public CDocObject, public IFEMPlotter, public IGUIDialogCallback
{
	public:
		CFEMPlotter();
		virtual ~CFEMPlotter();

		DECLARE_IUNKNOWN

		BEGIN_INTERFACES
			INTERFACE_ENTRY(IFEMPlotter)
			INTERFACE_ENTRY(I2DDraw)
			INTERFACE_ENTRY(ISceneGraphObject)
			INTERFACE_ENTRY(ISerialize)
			INTERFACE_ENTRY(IGUIDialogCallback)
		END_INTERFACES

		STATIC_CREATE_COMPONENT(CFEMPlotter)

		static const GUID &_GetGUID( void )
		{ 
			static const GUID guid = { 0x5f6fabb0, 0x2105, 0x4e4d, { 0x99, 0xe3, 0x2b, 0x94, 0x70, 0xf7, 0xfc, 0x96 } };

			return(guid); 
		}

	void								InitVar					( void );
	void								OpenFEMIN				( void );
	void								OpenFEMOUT				( void );

	void								SetUPotValue			( double );
	void								UpdateControlWindow		( void );
	void								BuildDialog				( int inMode );	// 0, ..

	void								DrawMatrix				( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	void								DrawOptimizedMatrix		( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	void								DrawNodeNumbers			( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	void								DrawNodeNumbersOptimized( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	void								DrawEdges				( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	void								DrawElementNumbers		( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	void								DrawElementPot			( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	void								HeatShadeElement		( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	
// Interface IFEMPlotter
	virtual void __stdcall				InitModule				( void );

	// Interface I2DDraw
	virtual void						Draw					( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify );
	virtual void						MarkedDraw				( I2DMemoryDC *pMC, int r, CPoint2I pos, int inMagnify );	// wird zusttzlich zu Draw aufgerufen, wenn Objekt markiert ist
	virtual bool						IsObjectAt				( CPoint2I xy );
	virtual I2DDraw *					GetTopObjectAt			( CPoint2I xy );
	virtual	void						OnContextMenu			( IPopUpMenu *p, CPoint2I xy );
	virtual ULONG						IsAnchorPointAt			( int r, CPoint2I xy );	// nur fr markierte Objekte, 0, wenn dort kein Punkt, sonst ist dort ein Angriffspunkt (Skalieren, ...)
	virtual I2DDraw *					GetTopAnchorPointAt		( int r, CPoint2I xy, ULONG &anchorid );
	virtual void						ScaleObject				( CPoint2I rel );

	// Interface ISerialize
	virtual void						Serialize				( CSerializeArchivePointer &ar );
	virtual const GUID &				GetGUID					( void ){ return(_GetGUID()); }

	// Interface IGUIDialogCallback
	virtual void __stdcall				OnControlMessage		( CRGUI_ControlMessage &m );

	// Interface ISceneGraphObject
	virtual void						ReleaseSubObjects		( void );											// Austragen



private:
	CHandle<IWinGUIDialog>		m_MainGUIDialog;		// GUI


	// private Data Elements and Methods

	CFemProcessor				m_femproc;
	CPerfTimer					m_perftime;
	bool						m_displayValues[11];		// 7 ist noch frei
	const float					m_scale;
	float						m_subx;
	float						m_suby;
	float						m_StepperU;
	float						m_StepperV;
	int							m_HeatShadeMode;			// 0:none, 1:U-Werte, 1:V-Werte

	int							m_EWNumber;					// 0 fr den kleinsten EW, ...
	DWORD						m_MemNeeded;
	int							m_DetailLevel;				// 1low.. 16high

	CVectorNArray<FP_TYPE>		m_ResultEW;
	CMatrixNMArray<FP_TYPE>		m_ResultEV;

	void	PlotInnerTriangle			( I2DMemoryDC *pMC, CPoint2F p[3], FP_TYPE value[3], float step, CColor col, CColor col_neg );
	void	PlotInnerTriangle_OnlyZero	( I2DMemoryDC *pMC, CPoint2F p[3], FP_TYPE value[3], CColor col );
	void	HeatShadeInnerTriangle		( I2DMemoryDC *pMC, CPoint2F p[3], FP_TYPE value[3] );
};



// explizite Nummerierung ist nur dan wichtig, wenn andere Komponenten auf konkrete
// Nummerierungen angewiesen sind
enum Dlg_UserId
{
	eNone=0,

	eLoeschen,
	eOpenFEMIN,
	eOpenFEMOUT,
	eCalcAndSave_Array,
	eCalcAndSave_Hull,
	eCalcAndSave_List,

	eDisplayU,
	eStepperU,
	eDisplayV,
	eStepperV,
	eDisplayNodes,
	eDisplayNodesOptimized,
	eDisplayElements,
	eDisplayEdges,
	eDisplayMatrix,
	eDisplayMatrixOptimized,
	eSubdiv,
	eNodeCountOutput,
	eMemoryOutput,
	eEWIncrease,
	eEWDecrese,
	eEWNumberOutput,
	eEWValueOutput,
	eDisplayZeroU,		// Nur 0-Durchgnge anzeigen
	eDisplayEdgesHB,	// Transparent
	eDetails1,
	eDetails4,
	eDetails8,
	eDetails16,
	eDetails64,
	eHeatShadeNone,
	eHeatShadeU,
	eHeatShadeV,
	eSaveFEMIN
};

#endif
