//-------------------------------------------------------------------+
// FemCalc.h                                                         |
//-------------------------------------------------------------------+
// Definiton of CFemCalc-class.                                      |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 22.02.2001
#ifndef _DVA_FEM_CALC_H__
#define _DVA_FEM_CALC_H__


//! Hilfsklasse f&uuml;r die Zuordnung der Randbedingungen zu den dazugeh&ouml;rigen RECHNEN-Kommandos
//
class CFemCalc
{
private:
    int     m_iConditionCount;  //!< Anzahl der Bedingungen, die zu der Berechnung gehren
    int     m_iEdgeCount;       //!< Anzahl der Rnder
    int*    m_pEdgeNr;          //!< Array mit den Rand-Nr., welche die Bedingung hlt und zu der Berechnung gehrt

public:
    //! Defaultkonstruktor setzt alle Element auf Null
    CFemCalc():
      m_iConditionCount(0),
      m_pEdgeNr(NULL)
    {}
    ~CFemCalc()
    {
        delete [] m_pEdgeNr;
    }

    //! Zum Initialisieren der Klassenelemente. Attribute werden alle auf Null gesetzt.
    void Init(const int in_iEdgeCount)
    {
        delete [] m_pEdgeNr;
        m_iEdgeCount=in_iEdgeCount;
        m_pEdgeNr= new int[m_iEdgeCount];
        assert(m_pEdgeNr != NULL);
    }

    //! Gibt alle angeforderten Ressourcen frei
    void DeInit()
    {
        delete [] m_pEdgeNr;
        m_pEdgeNr=NULL;
        m_iEdgeCount=m_iConditionCount=0;
    }

    //! Fgt dem RECHNEN-Kommando eine Randbedingung hinzu
    //! \param in_iEdgeNr Ist die Randnummer, zu der eine Bedingung vorliegt. Die Bedingung selbst ist in CFemEdge abgelegt.
    //! \return true bei Erfolg, false falls keine Bedingungen mehr hinzugefgt werden knnen.
    bool AddCondition(const int in_iEdgeNr)
    {
        if (m_pEdgeNr && (m_iConditionCount<m_iEdgeCount))
        {
            m_pEdgeNr[m_iConditionCount]=in_iEdgeNr;
            m_iConditionCount++;
            return true;
        }
        return false;
    }

    //! Liefert die Anzahl der Randbedingungen zurck. In der Regel 2.
    //! \return Anzahl der Randbedingungen
    int GetConditionCount() const { return m_iConditionCount; }

    //! Liefert zu einer Bedingung die Randnummer, die den Wert der Bedingung hlt. MARKER: ist nicht sauber
    //! \param in_iConditionIndex Nullindizierter Index von 0 bis GetConditionCount()-1
    //! \return Randnummer, bei einem Fehler -1 
    int GetEdgeNr(const int in_iConditionIndex) const
    {
        if (m_pEdgeNr && (in_iConditionIndex<m_iEdgeCount))
            return m_pEdgeNr[in_iConditionIndex];
        return -1;
    }
};

//-- Anzahl der max. untersttzten RECHNEN-Kommandos
//
const int g_iMAX_CALC=2;


#endif//_DVA_FEM_CALC_H__