#include "FemData.h"
#include "TokenParser.h"

#include <float.h>

#include <iomanip>
#include <fstream>
using namespace std;

//! Defaultkonstruktor
CFemData::CFemData():
    m_iFEMVersion(0),
    m_iNodeCount(0),
    m_iEdgeCount(0),
    m_pEdge(NULL),
    m_iCalcCount(0),
	m_bOutDataAvailable(false)
{
    m_pUVValue[0]=NULL;
    m_pUVValue[1]=NULL;
}

CFemData::~CFemData()
{
    FreeData();
}

//! Gibt alle Resourcen wieder frei und setzt die Daten auf Null zur&uuml;ck
void CFemData::FreeData()
{
    //-- Knotenpunkte lschen
    //
    m_iNodeCount=0;
    m_vNodeCoord.clear();

    //-- Rnder lschen
    //
    delete [] m_pEdge;
    m_pEdge=NULL;

    //-- Name der Ausgabedatei lschen
    //
    m_sFEMOutFile="";

    //-- Finite Elemente lschen
    //
    m_TriElement.SetType(CFemElement::eET_UNKNOWN);
    m_RecElement.SetType(CFemElement::eET_UNKNOWN);

    //-- Rechen-Kommandos lschen
    //
    m_iCalcCount=0;
    m_pCalculation[0].DeInit();
    m_pCalculation[1].DeInit();

    //-- Arrays mit den UV-Ergebnissen lschen
    //
    delete [] m_pUVValue[0];
    delete [] m_pUVValue[1];

    m_pUVValue[0]=NULL;
    m_pUVValue[1]=NULL;

    m_bOutDataAvailable=false;
}

//-- Parst eine FEM-IN-Datei
//
void CFemData::LoadFemIn(const char* in_pFileName, const bool in_bTryToLoadFemOut, CVectorNArray<FP_TYPE>* out_pEW, CMatrixNMArray<FP_TYPE>* out_pEV)
{
    //-- Text-Parser mit Semikolon als einleitendes Symbol fr einzeilige Kommentare
    //
    CTokenParser femParser(";");
    if (femParser.LoadFile(in_pFileName))
    {
        //-- zum mitprotokolieren der aktuellen Zeilennummer
        //
        int iLineNr;

        //-- Alte Daten freigeben
        //
        FreeData();

        //-- Dateiheader einlesen
        //
        if (femParser.GetToken("FEM", true, iLineNr))
        {
            if (!femParser.GetIntegerValue(m_iFEMVersion, iLineNr))
                throw CFemDataError("FEM-Version-Number missing", iLineNr);
        }
        else
            throw CFemDataError("Token FEM missing", iLineNr);

        if (femParser.GetToken("FEMOUT", true, iLineNr))
            m_sFEMOutFile=femParser.GetOptionalStringValue(iLineNr);
        else
		{
			m_sFEMOutFile=in_pFileName;
			int iPos=m_sFEMOutFile.rfind(".");
			if (iPos != string::npos)
				m_sFEMOutFile=m_sFEMOutFile.substr(0, iPos);

			m_sFEMOutFile.append(".out");
            femParser.SetBack();
		}

        //-- Koordinaten der Knotenpunkte einlesen
        //
        if (femParser.GetToken("KOORDINATEN", true, iLineNr))
        {
            if (!femParser.GetIntegerValue(m_iNodeCount, iLineNr))
                throw CFemDataError("Number for KOORDINATEN missing", iLineNr);

            //-- Anzahl der Koordinaten bekannt => Speicher allokieren
            //
            m_vNodeCoord.reserve(m_iNodeCount);
            if (m_vNodeCoord.capacity() < m_iNodeCount)
                //MARKER: neue Fehlerklasse erstellen: FatalError
                throw CFemDataError("Out of Memory for m_vNodeCoord", __LINE__);

            //-- Speicher fr die Ergebnis-Vektoren
            //
            m_pUVValue[0]=new FP_TYPE[m_iNodeCount];
            m_pUVValue[1]=new FP_TYPE[m_iNodeCount];

            //-- UV-Werte auf Null initialisieren
            //
            for (int i=0; i<m_iNodeCount; i++)
                m_pUVValue[0][i]=m_pUVValue[1][i]=FP_TYPE(0);
        }
        else
            throw CFemDataError("Token KOORDINATEN missing", iLineNr);

        tagFEM3Coord*   pFEM3Coord=NULL;
        if (
            (m_iFEMVersion==3) ||       // FEM3 quadratischer Ansatz
            (m_iFEMVersion==4)          // FEM4 quadratischer Ansatz, Interpretation als Membrangebiet
           )
        {
            //----------------------------
            //-- FEM3 oder FEM4 VERSION --
            //----------------------------

            //-- Bei FEM3 enthlt das Token KOORDINATEN einen zweiten Parameter:
            //   Anzahl der angegebenen Knotenpunkte in der FEM-Datei. (Fehlende
            //   Zwischenpunkte werden interpoliert.)
            //
            int iRealNodeCount;
            if (!femParser.GetIntegerValue(iRealNodeCount, iLineNr))
                throw CFemDataError("FEM3: Missing second NodeCount for KOORDINATEN", iLineNr);

            //-- Die Knotenpunkte werden vorbergehend in das Array eingelesen.
            //   Whrend dem Einlesevorgang der Elemente, werden die fehlenden Knotenpunkte
            //   bestimmt, die zwischen den vorhanden Punkten liegen. Die Zuordnung der
            //   richtigen Knotenpunktsnummern erfolgt mit Hilfe der Elementnummerierung.
            //
            pFEM3Coord=new tagFEM3Coord[m_iNodeCount];
            assert(pFEM3Coord != NULL);

            int iNodeNumber;
            int iArrayIndex;
            for (int i=0; i<iRealNodeCount; i++)
            {
                //-- Format: {NR x y}
                //
                if (!femParser.GetIntegerValue(iNodeNumber, iLineNr))
                    throw CFemDataError("FEM3: Missing Node-Number for KOORDINATEN", iLineNr);

                iArrayIndex=iNodeNumber-1;
                assert(iArrayIndex<m_iNodeCount);
                assert(iArrayIndex>=0);
                pFEM3Coord[iArrayIndex].iNodeNr=iNodeNumber;

                if (!femParser.GetFloatValue(pFEM3Coord[iArrayIndex].x, iLineNr))
                    throw CFemDataError("FEM3: Missing x-Position for KOORDINATEN", iLineNr);
                if (!femParser.GetFloatValue(pFEM3Coord[iArrayIndex].y, iLineNr))
                    throw CFemDataError("FEM3: Missing y-Position for KOORDINATEN", iLineNr);
            }
        }
        else
        {
            if (
                (m_iFEMVersion==2) ||       // FEM2 linearer Ansatz
                (m_iFEMVersion==5) ||       // FEM5 linearer Ansatz, Interpretation als Membrangebiet
                (m_iFEMVersion==6)          // FEM6 linearer Ansatz, Schraubenschlssel
               )
            {
                //----------------------------
                //-- FEM2 oder FEM5 VERSION --
                //----------------------------

                //-- Nun werden m_iCoordCount Zahlenpaare erwartet
                //
                for (int i=0; i<m_iNodeCount; i++)
                {
                    tagCoord tCoord;
                    if (!femParser.GetFloatValue(tCoord.x, iLineNr))
                        throw CFemDataError("Missing x-Position for KOORDINATEN", iLineNr);
                    if (!femParser.GetFloatValue(tCoord.y, iLineNr))
                        throw CFemDataError("Missing y-Position for KOORDINATEN", iLineNr);

                    m_vNodeCoord.push_back(tCoord);
                }
            }
            else
                throw CFemDataError("File-Format not supported", -1);
        }

        //-- Elemente einlesen
        //
        if (femParser.GetToken("ELEMENTE", true, iLineNr))
        {
            int iTriCount;
            if (!femParser.GetIntegerValue(iTriCount, iLineNr))
                throw CFemDataError("Number of TriAngles after the Token ELEMENTE missing", iLineNr);

            int iRecCount;
            if (!femParser.GetIntegerValue(iRecCount, iLineNr))
                throw CFemDataError("Number of RectAngles after the Token ELEMENTE missing", iLineNr);

            //-- Anzahl der Elemente sind nun bekannt => Elemente entsprechend der FEM-Version initialisieren
            //
            switch (m_iFEMVersion)
            {
            case 2:
            case 5:
            case 6:
                {
                    //-- Dreiecke oder Vierecke mit linearem Ansatz
                    //
                    if (iTriCount)
                    {
                        if (m_TriElement.Init(CFemElement::eET_TRIANGLE, iTriCount) == false)
                            throw CFemDataError("Couldn't initialize triangle elements", __LINE__);
                    }

                    if (iRecCount)
                    {
                        if (m_RecElement.Init(CFemElement::eET_PARALLELOGRAMM, iRecCount) == false)
                            throw CFemDataError("Couldn't initialize rectangle elements", __LINE__);
                    }
                    break;
                }
            case 3:
            case 4:
                {
                    //-- Dreiecke oder Vierecke mit Zwischenpunkten
                    //
                    if (iTriCount)
                    {
                        if (m_TriElement.Init(CFemElement::eET_TRIANGLE2, iTriCount) == false)
                            throw CFemDataError("Couldn't initialize triangle elements", __LINE__);
                    }

                    if (iRecCount)
                    {
                        if (m_RecElement.Init(CFemElement::eET_PARALLELOGRAMM2, iRecCount) == false)
                            throw CFemDataError("Couldn't initialize rectangle elements", __LINE__);
                    }
                    break;
                }
            default:
                {
                    throw CFemDataError("FEM-Version not supported", __LINE__);
                }
            }

            //-- Fr jedes Dreiecks-Element die dazugehrigen Knotenpunkte auslesen und setzen
            //
            for (int i=0; i<iTriCount; i++)
            {
                //-- Knotenpunkte lesen
                //
                for (int j=0; j<m_TriElement.GetType(); j++)
                {
                    int iNodeNr;
                    if (femParser.GetIntegerValue(iNodeNr, iLineNr) == false)
                    {
                        char cBuffer[256];
                        sprintf(cBuffer, "Missing Coord-Nr. %d for triangle Element %d", j+1, i+1);
                        throw CFemDataError(cBuffer, iLineNr);
                    }
                    if (m_TriElement.SetNodeNr(i, j, iNodeNr, pFEM3Coord) == false)
                        throw CFemDataError("Setting Node-Nr. for triangle Element failed", __LINE__);
                }
            }

            //-- Fr jedes Vierecks-Element die dazugehrigen Knotenpunkte auslesen und setzen
            //
            for (i=0; i<iRecCount; i++)
            {
                //-- Knotenpunkte lesen
                //
                for (int j=0; j<m_RecElement.GetType(); j++)
                {
                    int iNodeNr;
                    if (femParser.GetIntegerValue(iNodeNr, iLineNr) == false)
                    {
                        char cBuffer[256];
                        sprintf(cBuffer, "Missing Coord-Nr. %d for rectangle Element %d", j+1, i+1);
                        throw CFemDataError(cBuffer, iLineNr);
                    }
                    if (m_RecElement.SetNodeNr(i, j, iNodeNr, pFEM3Coord) == false)
                        throw CFemDataError("Setting Coord for recangle Element failed", __LINE__);
                }
            }
        }
        else
            throw CFemDataError("Token ELEMENTE missing", iLineNr);

        //-- Nachdem die eventuell fehlende Zwischenpunkte bestimmt wurden
        //   werden sie in das eigentliche Array in der richtigen Reihenfolge
        //   eingetragen.
        //
        if (pFEM3Coord)
        {
            ResortFEM3Coord(pFEM3Coord);
            delete [] pFEM3Coord;
            pFEM3Coord=NULL;
        }

        //-- Zusatzdaten des FEM6-Dateiformats einlesen
        //
        if (m_iFEMVersion == 6)
        {
            try
            {
                ReadFEM6Data(femParser);
            }
            catch(CFemDataError& e)
            {
                throw e;
            }
        }

        //-- Rnder einlesen
        //
        if (femParser.GetToken("NR", true, iLineNr))
        {
            //-- Angebene Anzahl der Rnder in der FEM-Datei auslesen
            //
            if (femParser.GetIntegerValue(m_iEdgeCount, iLineNr) == false)
                throw CFemDataError("Number after Token NR missing", iLineNr);

            //-- Anzahl der Rnder bekannt => Speicher allokieren
            //
            m_pEdge=new CFemEdge[m_iEdgeCount];
            assert(m_pEdge != NULL);
            if (m_pEdge == NULL)
                //MARKER: neue Fehlerklasse erstellen: FatalError
                throw CFemDataError("Out of Memory for m_pEdge", __LINE__);
        }
        else
            throw CFemDataError("Token NR missing", iLineNr);

        //-- Rnder laden
        //
        for (int i=0; i<m_iEdgeCount; i++)
        {
            if (femParser.GetToken("RAND", true, iLineNr))
            {
                //-- Anzahl der Knotenpunkte auf dem Rand einlesen
                //
                int iCount;
                if (femParser.GetIntegerValue(iCount, iLineNr))
                {
                    if (m_pEdge[i].SetNodeCount(iCount) == false)
                        throw CFemDataError("Out of Memory for m_pEdge.SetCoordCount", __LINE__);
                }
                else
                {
                    char cBuffer[256];
                    sprintf(cBuffer, "Missing Coord-Count for Edge-Condition %d", i+1);
                    throw CFemDataError(cBuffer, iLineNr);
                }

                //-- Nummer der Knotenpunkte einlesen
                //
                for (int j=0; j<iCount; j++)
                {
                    int iEdgeNodeNr;
                    if (femParser.GetIntegerValue(iEdgeNodeNr, iLineNr))
                    {
                        m_pEdge[i].BackInsert(iEdgeNodeNr);
                    }
                    else
                    {
                        char cBuffer[256];
                        sprintf(cBuffer, "Missing Node-Nr. %d for Edge-Condition %d", j+1, i+1);
                        throw CFemDataError(cBuffer, iLineNr);
                    }
                }
            }
            else
                CFemDataError("Token RAND missing", iLineNr);
        }

        //-- Randbedingungen laden und RECHNEN-Kommandos den RB zuordnen
        //
        int  iCondCount=0;
        bool bFoundStopToken=false;
        bool bFirstCondToken=false;

        //-- Solange keine STOP-Token gefunden wurde
        //   oder keine Exception geworfen wird, luft die Schleife durch
        //
        while (!bFoundStopToken)
        {
            //-- Bedingung fr einen Rand gefunden
            //
            if (femParser.GetToken("BEDINGUNG", true, iLineNr))
            {
                if (m_iCalcCount >= g_iMAX_CALC)
                    throw CFemDataError("MAX_CALC out of limit", __LINE__);

                if (bFirstCondToken==false)
                {
                    m_pCalculation[m_iCalcCount].Init(m_iEdgeCount);
                    bFirstCondToken=true;
                }

                //-- Randnummer aus der Datei lesen zu der die Bedingung gehrt
                //
                int iCondForEdge;
                if (femParser.GetIntegerValue(iCondForEdge, iLineNr) == false)
                {
                    char cBuffer[256];
                    sprintf(cBuffer, "Missing Edge-Number for Condition %d", iCondCount+1);
                    throw CFemDataError(cBuffer, iLineNr);
                }

                //-- Potential, das auf dem Rand herrschen soll einlesen
                //
                float fCondition;
                if (femParser.GetFloatValue(fCondition, iLineNr) == false)
                {
                    char cBuffer[256];
                    sprintf(cBuffer, "Missing Value for Condition %d", iCondForEdge);
                    throw CFemDataError(cBuffer, iLineNr);
                }

                //-- Potential dem Rand zuweisen.
                //   Die eingelesene Rand-Nummer wird auf seine Gltigkeit hin berprft.
                //
                if (
                    ((iCondForEdge-1) >= 0           ) &&
                    ((iCondForEdge-1) <  m_iEdgeCount)
                   )
                {
                    m_pCalculation[m_iCalcCount].AddCondition(iCondForEdge);

                    m_pEdge[iCondForEdge-1].SetCondition(fCondition);
                    iCondCount++;
                }
                else
                {
                    char cBuffer[256];
                    sprintf(cBuffer, "Edge-Number out of Range for %d", iCondCount+1);
                    throw CFemDataError(cBuffer, iLineNr);
                }
            }
            else
            {
                //-- Keine BEDINGUNG, dann evtl. STOP
                //
                if (femParser.CompareCurrentToken("STOP"))
                    bFoundStopToken=true;
                else
                {
                    //-- RECHNEN-Tokens gefunden, nun erfolgt die Zuordnung der Bedingungen an das nchste RECHNEN-Kommando
                    //   falls noch weitere vorhanden sind
                    //
                    if (femParser.CompareCurrentToken("RECHNEN"))
                    {
                        bFirstCondToken=false;
                        m_iCalcCount++;
                    }
                    else
                        throw CFemDataError("Unknown Token: ", femParser.GetCurrentToken(), iLineNr);
                }
            }
        }

        //-- Mindestanzahl der ntigen Bedingungen ist vom Dateiformat abhngig
        //
        if ((m_iFEMVersion==2) || (m_iFEMVersion==3))
        {
            if (iCondCount < 2)
                throw CFemDataError("Not enough conditions found", iLineNr);
        } // bei FEM 4+5 ist nur eine Bedingung erforderlich, bei FEM6 keine Einschrnkungen

    }
    else
        throw CFemDataError("Parser could not open ", in_pFileName);

    //-- Dateiname der Eingabedaten fr Zusatzinformationen bei einer spteren FEM-Out-Datei-Generierung
    //
    m_sFEMInFile=in_pFileName;

    if (in_bTryToLoadFemOut)
    {
        try
        {
            //-- Versuche Datei zuerst ber optional angegebenen Dateinamen beim FEMOUT-Token zu laden
            //
			if(m_iFEMVersion==4 || m_iFEMVersion==5)
	            LoadFemOut(m_sFEMOutFile.c_str(), out_pEW, out_pEV);
			  else
	            LoadFemOut(m_sFEMOutFile.c_str(), 0, 0);
        }
        catch(CFemDataError& e)
        {
            //-- Fehlerhafte-Datei
            //
            if (e.GetLineNr() != -1)
                throw e;

            //-- Versuche Datei analog zum Dateinamen der FEM-IN-Datei zu laden
            //
            try
            {
                string sFemOutFile=in_pFileName;
                sFemOutFile=sFemOutFile.substr(0, sFemOutFile.rfind("."))+".out";
                LoadFemOut(sFemOutFile.c_str(), out_pEW, out_pEV);
            }
            catch(CFemDataError&)
            {
                //-- Fehlerhafte-Datei
                //
                if (e.GetLineNr() != -1)
                    throw e;

                //keine passende out-Datei gefunden
                //
            }
        }
    }
}


//! Speichert die Daten in einer Textdatei im FEM-Format ab
//! \param in_pFileName Dateiname mit relativen oder absoluten Pfad
void CFemData::SaveFemIn(const char* in_pFileName)
{
    assert(in_pFileName != NULL);

    ofstream ofs(in_pFileName);
    if (ofs.is_open()==0)
       throw CFemDataError("Could not open file for writing ", in_pFileName);

    //-- FEM-Dateiheader
    //
    ofs << "FEM " << m_iFEMVersion << endl;

    //-- Optional -> Angabe des Dateinamens fr die Ergebnisdatei
    //
    if (!m_sFEMOutFile.empty())
        ofs << "FEMOUT " << m_sFEMOutFile << endl;

    if ((m_iFEMVersion==2) || (m_iFEMVersion==5) || (m_iFEMVersion==6))
    {
        //-- Koordinaten der Elemente ausgeben, linearer Ansatz
        //
        ofs << "KOORDINATEN " << m_iNodeCount << endl;

        for (int i=0; i<m_iNodeCount; i++)
        {
            ofs << setw(3) << m_vNodeCoord[i].x << " "
                << setw(3) << m_vNodeCoord[i].y << endl;
        }
    }
    else
    {
        //-- Koordinaten der Elemente ausgeben + Anzahl der vorhandenen Koordinaten in der FEM-Datei
        //
        ofs << "KOORDINATEN " << m_iNodeCount << " " << m_iNodeCount <<endl;

        for (int i=0; i<m_iNodeCount; i++)
        {
            ofs << setw(3) << i+1 << " "
                << setw(3) << m_vNodeCoord[i].x << " "
                << setw(3) << m_vNodeCoord[i].y << endl;
        }
    }

    //-- Abzahl der Elemente ausgeben, zuerst Dreiecke dann Rechtecke
    //
    ofs << "ELEMENTE " << m_TriElement.GetCount() << " " << m_RecElement.GetCount() << endl;

    //-- Knotenpunktnummern der Dreiecke
    //
    int iType=m_TriElement.GetType();
    for (int i=0; i<m_TriElement.GetCount(); i++)
    {
        for (int j=0; j<iType; j++)
        {
            ofs << m_TriElement.GetNodeNr(i, j);
            if ((j+1)<iType)
                ofs << " ";
        }
        ofs << endl;
    }

    //-- Knotenpunktnummern der Vierecke
    //
    for (i=0; i<m_RecElement.GetCount(); i++)
    {
        for (int j=0; j<m_RecElement.GetType(); j++)
        {
            ofs << m_RecElement.GetNodeNr(i, j) << " ";
        }
        ofs << endl;
    }

    if (m_iFEMVersion==6)
    {
        //-- Elastizittsmodul, Poissonzahl und Materialdicke
        //
        ofs << "EMODUL "  << m_ScrewDriver.m_dEModul    << endl;
        ofs << "POISSON " << m_ScrewDriver.m_fPoisson   << endl;
        ofs << "DICKE "   << m_ScrewDriver.m_fThickness << endl;

        //-- Angreifenden Lasten
        //
        ofs << "LAST " << m_ScrewDriver.GetForceCount() << endl;

        for (int i=0; i<m_ScrewDriver.GetForceCount(); i++)
        {
            ofs << m_ScrewDriver.GetForce(i).iNodeNr        << " "
                << m_ScrewDriver.GetForce(i).fXDirection    << " "
                << m_ScrewDriver.GetForce(i).fYDirection    << endl;
        }
    }

    //-- Rnder ausgeben
    //
    ofs << "NR " << m_iEdgeCount << endl;

    assert(m_pEdge != NULL);

    for (i=0; i<m_iEdgeCount; i++)
    {
        ofs << "RAND " << m_pEdge[i].GetNodeCount() << "  ";
        for (int j=0; j<m_pEdge[i].GetNodeCount(); j++)
            ofs << m_pEdge[i].GetNodeNr(j) << " ";
        ofs << endl;
    }

    //-- Bedingungs- und Rechnen-Kommandos ausgeben
    //
    for (i=0; i<m_iCalcCount; i++)
    {
        for (int j=0; j<m_pCalculation[i].GetConditionCount(); j++)
        {
            int iEdgeNr=m_pCalculation[i].GetEdgeNr(j);
            ofs << "BEDINGUNG " << iEdgeNr << "  " << m_pEdge[iEdgeNr-1].GetCondition() << endl;
        }
        ofs << "RECHNEN" << endl;
    }

    ofs << "STOP" << endl;
}


//! Hilfsroutine zum Einlesen der FEM6-Daten
void CFemData::ReadFEM6Data(class CTokenParser& in_rParser)
{
    int iLineNr;

    //-- Elastizittsmodul
    //
    if (in_rParser.GetToken("EMODUL", true, iLineNr))
    {
        float fDummy;
        if (in_rParser.GetFloatValue(fDummy, iLineNr) == false)
            throw CFemDataError("Number after Token EMODUL missing", iLineNr);

        m_ScrewDriver.m_dEModul=fDummy;
    }
    else
        throw CFemDataError("Token EMODUL missing", iLineNr);

    //-- Poissonzahl
    //
    if (in_rParser.GetToken("POISSON", true, iLineNr))
    {
        float fDummy;
        if (in_rParser.GetFloatValue(fDummy, iLineNr) == false)
            throw CFemDataError("Number after Token POISSON missing", iLineNr);

        m_ScrewDriver.m_fPoisson=fDummy;
    }
    else
        throw CFemDataError("Token POISSON missing", iLineNr);

    //-- Materialdicke
    //
    if (in_rParser.GetToken("DICKE", true, iLineNr))
    {
        float fDummy;
        if (in_rParser.GetFloatValue(fDummy, iLineNr) == false)
            throw CFemDataError("Number after Token DICKE missing", iLineNr);

        m_ScrewDriver.m_fThickness=fDummy;
    }
    else
        throw CFemDataError("Token DICKE missing", iLineNr);

    //-- Anzahl der Krfte bzw. Lasten
    //
    if (in_rParser.GetToken("LAST", true, iLineNr))
    {
        int iDummy;
        if (in_rParser.GetIntegerValue(iDummy, iLineNr) == false)
            throw CFemDataError("Number after Token LAST missing", iLineNr);

        m_ScrewDriver.SetForceCount(iDummy);
    }
    else
        throw CFemDataError("Token LAST missing", iLineNr);

    //-- angreifenden Lasten einlesen
    //
    for(int i=0; i<m_ScrewDriver.GetForceCount(); i++)
    {
        int iNodeNr;
        if (in_rParser.GetIntegerValue(iNodeNr, iLineNr) == false)
            throw CFemDataError("LAST: NodeNumber missing", iLineNr);

        float x,y;
        if (in_rParser.GetFloatValue(x, iLineNr) == false)
            throw CFemDataError("LAST: x-value missing", iLineNr);

        if (in_rParser.GetFloatValue(y, iLineNr) == false)
            throw CFemDataError("LAST: y-value missing", iLineNr);

        m_ScrewDriver.SetForce(i, iNodeNr, x, y);
    }
}



//! ffnet eine Textdatei im FEM-OUT-Format. Wirft bei einem Fehler eine Exception vom Typ CFemDataError.
//! \param in_pFileName Dateiname mit relativen oder absoluten Pfad
//! \param out_pEW Vektor mit den Eigenwerten von der erweiterten FEM-Out-Datei
//! \param out_pEV Matrix mit den dazugehrigen Eigenvektoren (spaltenweise orientiert).
void CFemData::LoadFemOut(const char* in_pFileName, CVectorNArray<FP_TYPE>* out_pEW, CMatrixNMArray<FP_TYPE>* out_pEV)
{
    assert(in_pFileName != NULL);

    //-- Text-Parser bestimmt im 1. Pass die Anzahl der UWERTE und im 2. Pass werden sie eingelesen
    //
    CTokenParser femParser(";");
    if (femParser.LoadFile(in_pFileName))
    {
        //-- Hilfsvariablen
        //
        int  iLineNr;
        bool bStopFound=false;
        int  iUSections=0;
        int  iNodeCount;        // Anzahl der Knoten innerhalb der akt. Sektion
        int  iMaxNodeCount=-1;  // Max. Anzahl der Knoten wird durch die erste Sektion vorgegeben

        //-- 1. Pass: Bestimmung der Parameter
        //
        while (!bStopFound)
        {
            //-- Abschnitt UWERTE einlesen
            //
            if (femParser.GetToken("UWERTE", true, iLineNr))
            {
                iUSections++;
                iNodeCount=0;

                if ((out_pEW==NULL) && (out_pEV==NULL))
                if (iUSections > g_iMAX_CALC)
                    throw CFemDataError("Too many UWERTE-Sections for this file-format. Only two sections supported.", iLineNr);

                float fDummy;
                while (femParser.GetFloatValue(fDummy, iLineNr))
                {
                    iNodeCount++;
                }

                if (iMaxNodeCount>0)
                {
                    if (iMaxNodeCount != iNodeCount)
                        throw CFemDataError("Current Nodecount differs from first UWERTE-section", iLineNr);
                }
                else
                {
                    //-- Derzeit nur die aktuelle Sektion gelesen => kein Vergleich mglich
                    //
                    iMaxNodeCount=iNodeCount;
                }

                //-- Parser zurcksetzen, damit im nchsten Durchlauf der while-Schleife
                //   GetToken auf das aktuelle LookAhead-Symbol arbeitet und noch nicht die
                //   nchste Zeile betrachtet.
                //
                femParser.SetBack();
            }
            else
            {
                if (femParser.CompareCurrentToken("STOP"))
                {
                    bStopFound=true;
                }
                else
                    throw CFemDataError("Unknown TOKEN found: ", femParser.GetCurrentToken(), iLineNr);
            }
        }

        //-- 2. Pass: Speicher fr die Variablen allokieren und einlesen
        //
        if (femParser.LoadFile(in_pFileName)==false)
            throw CFemDataError("Could not open FEM-Out file for second pass ", in_pFileName);

        if ((out_pEW==NULL) && (out_pEV==NULL))
        {
            //-- Nur Potential und Spannungswerte einlesen
            //
            for (int i=0; i<iUSections; i++)
            {
				//-- Speicher der alten UV-Werte freigeben
				//
				delete [] m_pUVValue[i];

                m_pUVValue[i]=new FP_TYPE[iNodeCount];
                assert(m_pUVValue[i] != NULL);

                if (femParser.GetToken("UWERTE", true, iLineNr))
                {
                    for (int j=0; j<iNodeCount; j++)
                    {
                        float fValue;
                        if (femParser.GetFloatValue(fValue, iLineNr) == false)
                        {
                            char cBuffer[256];
                            sprintf(cBuffer, "Could not get %d. U-Value", j+1);
                            throw CFemDataError(cBuffer, iLineNr);
                        }
                        m_pUVValue[i][j]=fValue;
                    }
                }
                else
                    throw CFemDataError("Token UWERTE not found: ", femParser.GetCurrentToken(), iLineNr);
            }
        }
        else
        {
            //-- Eigenwerte und Eigenvektoren einlesen
            //
            iNodeCount--;   // Wird um eins (=der mitgezhlte EW von oben) verringert, um die Anzahl der Komponenten pro EV zu erhalten

            out_pEW->Resize(iUSections);
            out_pEV->Resize(iNodeCount, iUSections);

            for (int i=0; i<iUSections; i++)
            {
                if (femParser.GetToken("UWERTE", true, iLineNr))
                {
                    float fValue;
                    if (femParser.GetFloatValue(fValue, iLineNr))
                    {
                        //-- Eigenwert hinzufgen
                        //
                        out_pEW->P(i, fValue);

                        //-- Eigenvektor auslesen
                        //
                        for (int j=0; j<iNodeCount; j++)
                        {
                            if (femParser.GetFloatValue(fValue, iLineNr) == false)
                            {
                                char cBuffer[256];
                                sprintf(cBuffer, "Could not get the %d. component of the Eigenvektor", j+1);
                                throw CFemDataError(cBuffer, iLineNr);
                            }

                            out_pEV->P(j, i, fValue);
                        }
                    }
                    else
                        throw CFemDataError("Missing Eigenwert after UWERTE: ", femParser.GetCurrentToken(), iLineNr);
                }
                else
                    throw CFemDataError("Token UWERTE not found: ", femParser.GetCurrentToken(), iLineNr);
            }
        }
        m_bOutDataAvailable=true;
    }
    else
        throw CFemDataError("Could not open FEM-Out file ", in_pFileName);
}



//! Speichert die UV-Werte im FEM-Format ab. Werden EW und EV bergeben, werden diese abgespeichert.
//! \param in_pFileName Dateiname mit relativen oder absoluten Pfad
//! \param out_pEW Vektor mit den Eigenwerten von der erweiterten FEM-Out-Datei
//! \param out_pEV Matrix mit den dazugehrigen Eigenvektoren (spaltenweise orientiert).
void CFemData::SaveFemOut(const char* in_pFileName, CVectorNArray<FP_TYPE>* in_pEW, CMatrixNMArray<FP_TYPE>* in_pEV) const
{
    ofstream ofs;

	if (in_pFileName == NULL)
	{
		ofs.open(m_sFEMOutFile.c_str());
		if (ofs.is_open()==0)
		   throw CFemDataError("Could not open file for writing ", m_sFEMOutFile.c_str());
	}
	else
	{
		ofs.open(in_pFileName);
		if (ofs.is_open()==0)
		   throw CFemDataError("Could not open file for writing ", in_pFileName);
	}


    long ofsFlags=ofs.flags();
    ofs.flags(ios::right|ios::fixed);

    if ((in_pEW==NULL) && (in_pEV==NULL))
    {
        //-- Potential und Spannungswerte
        //
        ofs << "; Potential und Spannungswerte von [" << m_sFEMInFile.c_str() << "]\n";

        for (int i=0; i<m_iCalcCount; i++)
        {
            assert(m_pUVValue[i] != NULL);

            ofs << "UWERTE" << endl;

            for (int j=0; j<m_iNodeCount; j++)
                ofs << setw(15) << m_pUVValue[i][j] << endl;
        }
    }
    else
    {
        //-- Eigenwerte und Eigenvektoren
        //
        ofs << "; Eigenwerte und Eigenvektoren von [" << m_sFEMInFile.c_str() << "]\n";
        ofs << ";\n";
        ofs << "; Syntax:\n";
        ofs << "; UWERTE {Eigenwert}\n";
        ofs << ";        {n Komponenten des dazugehrigen Eigenvektors}\n";

        for (int i=0; i<in_pEW->GetN(); i++)
        {
            ofs << ";\n";
            ofs << "; " << i+1 << ". Eigenwert und dazugehriger Eigenvektor\n";
            ofs << "UWERTE " << in_pEW->P(i) << endl;

            for (int j=0; j<in_pEV->GetN(); j++)
                ofs << setw(15) << in_pEV->P(j,i) << endl;
        }
    }

    ofs << "STOP" << endl;
    ofs.flags(ofsFlags);
}

//! Reserviert fr die Knotenpunkte vom FEM-Gebiet fr n Punkte Speicher. Die alten Knotenpunkte bleiben erhalten.
//! \param in_iReserveNodes Anzahl der zu reservierenden Knotenpunkte. Muss gr&ouml;sser als NodeCount sein.
//! \return true bei Erfolg, false falls Speicherallokation fehl schlug
bool CFemData::ReserveNodeCount(const int in_iReserveNodes)
{
    if (m_iNodeCount >= in_iReserveNodes)
        return false;

    m_vNodeCoord.reserve(in_iReserveNodes);
    return true;
}


//! F&uuml;gt einen neuen Knoten am Ende der Liste hinzu.
//! \param in_iNodeNr Nullindizierte Nummer des neuen Knotens. Wird zum &Uuml;berpr&uuml;fen benutzt.
//! \param in_fXPos x-Koordinate des Knotenpunkts
//! \param in_fYPos y-Koordinate des Knotenpunkts
//! \return true bei Erfolg, false falls in_iNodeNr != m_iNodeCount sein sollte
bool CFemData::BackInsertNode(const int in_iNodeNr, const float in_fXPos, const float in_fYPos)
{
    if ((in_iNodeNr-1) != m_iNodeCount)
        return false;

    tagCoord tempPos;
    tempPos.x=in_fXPos;
    tempPos.y=in_fYPos;
    m_vNodeCoord.push_back(tempPos);
    m_iNodeCount++;
    return true;
}


//! Liefert zu einem bestimmten Knotenpunkt dessen (x,y)-Koordinaten
//! \param in_iNodeNr Nullindizierte Knotenpunktnummer, Wertebereich [0 bis GetCoordCount()-1]
//! \param out_fXCoord X-Komponente
//! \param out_fYCoord Y-Komponente
//! \return true bei Erfolg, false falls die Knotenpunktnummer ausserhalb des Wertebereichs liegt, oder keine Knoten vorhanden sind
bool CFemData::GetNodeCoord(const int in_iNodeNr, float &out_fXCoord, float &out_fYCoord)
{
    if ((in_iNodeNr >= 0) && (in_iNodeNr < m_iNodeCount))
    {
        out_fXCoord=m_vNodeCoord[in_iNodeNr].x;
        out_fYCoord=m_vNodeCoord[in_iNodeNr].y;
        return true;
    }
    return false;
}


//! Weist einem Knotenpunkt das Potential (U- oder V-Wert) zu
//! \param in_iCalcNr Nullindizierter Index der Berechnung, Wertebereich [0 bis GetCalcCount()-1]
//! \param in_iNodeNr Nullindizierte Knotenpunktnummer, Wertebereich [0 bis GetCoordCount()-1]
//! \param in_fUVValue Potential welches auf dem Knoten herrscht
void CFemData::SetUVValue(const int in_iCalcNr, const int in_iNodeNr, const FP_TYPE in_fUVValue)
{
    assert(m_pUVValue[in_iCalcNr] != NULL);
    assert(in_iCalcNr < m_iNodeCount);

    m_pUVValue[in_iCalcNr][in_iNodeNr]=in_fUVValue;
}

//! Liefert zu einem Knotenpunkt dessen Potential (U- oder V-Wert)
//! \param in_iCalcNr Nullindizierter Index der Berechnung, Wertebereich [0 bis GetCalcCount()-1]
//! \param in_iNodeNr Nullindizierte Knotenpunktnummer, Wertebereich [0 bis GetCoordCount()-1]
//! \return Potential welches auf dem Knoten herrscht
FP_TYPE CFemData::GetUVValue(const in_iCalcNr, const int in_iNodeNr) const
{
    assert(m_pUVValue[in_iCalcNr] != NULL);
    assert(in_iNodeNr < m_iNodeCount);

    return m_pUVValue[in_iCalcNr][in_iNodeNr];
}


//! Sobald die Anzahl der Knotenpunkte zum Beispiel durch eine Gebietsverfeinerung ver&auml;ndert wird
//! muss danach das Array mit den Ergebniswerten angepasst werden. ResizeUVCount passt das Array an und setzt die Werte auf Null.
//! \return true bei Erfolg, false sonst
bool CFemData::ResizeUVCount()
{
    delete [] m_pUVValue[0];
    delete [] m_pUVValue[1];

    m_pUVValue[0]=new FP_TYPE[m_iNodeCount];
    assert(m_pUVValue[0] != NULL);
    m_pUVValue[1]=new FP_TYPE[m_iNodeCount];
    assert(m_pUVValue[1] != NULL);

    //-- UV-Werte auf Null initialisieren
    //
    for (int i=0; i<m_iNodeCount; i++)
        m_pUVValue[0][i]=m_pUVValue[1][i]=FP_TYPE(0);

    return (m_pUVValue[0] != NULL) && (m_pUVValue[1] != NULL);
}


//! Berechnet zu einem beliebigen Xi und Eta, die x,y-Koordinaten und das Potential in diesem Punkt.
//! \param in_fUVArray 
//! \param in_iElement Nullindizierte Element-Nr. [0 bis CFemElement::GetCount()-1]
//! \param in_fXi X-Position im Einheitselement [0 bis 1.0]
//! \param in_fEta Y-Position im Einheitselement [0 bis 1.0]
//! \param out_fXValue Errechneter X-Wert im FEM-Gebiet G
//! \param out_fYValue Errechneter Y-Wert im FEM-Gebiet G
//! \param out_fUVValue Potential, dass im errechneten Punkt herscht.
//! \return true falls die UV-Werte in den St&uuml;tzpunkten vorliegen, ansonsten false
bool CFemData::UVInterpolationTriangle( FP_TYPE *in_fUVArray, const int in_iElement, const float in_fXi, const float in_fEta,
                     float &out_fXValue, float &out_fYValue, FP_TYPE &out_fUVValue)
{
    return UVInterpolation(in_fUVArray, m_TriElement, in_iElement, in_fXi, in_fEta, out_fXValue, out_fYValue, out_fUVValue);
}

//! Berechnet zu einem beliebigen Xi und Eta, die x,y-Koordinaten und das Potential in diesem Punkt.
//! \param in_fUVArray
//! \param in_iElement Nullindizierte Element-Nr. [0 bis CFemElement::GetCount()-1]
//! \param in_fXi X-Position im Einheitselement [0 bis 1.0]
//! \param in_fEta Y-Position im Einheitselement [0 bis 1.0]
//! \param out_fXValue Errechneter X-Wert im FEM-Gebiet G
//! \param out_fYValue Errechneter Y-Wert im FEM-Gebiet G
//! \param out_fUVValue Potential, dass im errechneten Punkt herscht.
//! \return true falls die UV-Werte in den St&uuml;tzpunkten vorliegen, ansonsten false
bool CFemData::UVInterpolationRectangle( FP_TYPE *in_fUVArray, const int in_iElement, const float in_fXi, const float in_fEta,
                     float &out_fXValue, float &out_fYValue, FP_TYPE &out_fUVValue)
{
    return UVInterpolation(in_fUVArray, m_RecElement, in_iElement, in_fXi, in_fEta, out_fXValue, out_fYValue, out_fUVValue);
}


//! Berechnet zu einem beliebigen Xi und Eta, die x,y-Koordinaten und das Potential in diesem Punkt.
//! \param in_pOrgUVValue Potential oder Stromwert
//! \param in_FemElement FemElement-Klasse mit den Elementen
//! \param in_iElement Nullindizierte Element-Nr. [0 bis CFemElement::GetCount()-1]
//! \param in_fXi X-Position im Einheitselement [0 bis 1.0]
//! \param in_fEta Y-Position im Einheitselement [0 bis 1.0]
//! \param out_fXValue Errechneter X-Wert im FEM-Gebiet G
//! \param out_fYValue Errechneter Y-Wert im FEM-Gebiet G
//! \param out_fUVValue Potential, dass im errechneten Punkt herscht.
//! \return true falls die UV-Werte in den St&uuml;tzpunkten vorliegen, ansonsten false
bool CFemData::UVInterpolation(const FP_TYPE* in_pOrgUVValue, const CFemElement& in_FemElement, const int in_iElement, const float in_fXi, const float in_fEta,
                     float &out_fXValue, float &out_fYValue, FP_TYPE &out_fUVValue)
{
    static tagCoord pElementCoord[CFemElement::eElementType::eET_PARALLELOGRAMM2];  //Maximale Anzahl an mglichen Punkten
    static FP_TYPE  pAlphaVektor[CFemElement::eElementType::eET_PARALLELOGRAMM2];

    //-- Koordinaten der Eckpunkte vom Element und UV-Wert in diesen Punkten bestimmen
    //
    for (int i=0; i<in_FemElement.GetType(); i++)
    {
        int iNodeNr=in_FemElement.GetNodeNr(in_iElement, i)-1;

        pElementCoord[i]=m_vNodeCoord[iNodeNr];

        if (in_pOrgUVValue != NULL)pAlphaVektor[i]=in_pOrgUVValue[i];
			else pAlphaVektor[i]=0;
    }
    
	in_FemElement.UVInterpolation(in_fXi, in_fEta, pElementCoord, pAlphaVektor, out_fXValue, out_fYValue, out_fUVValue);

    return true;
}


//! Pr&uuml;ft ob eine Kante gegeben durch Start- und Endpunkt auf einem der R&auml;nder liegt.
//! \param in_iStartNr Knotenpunktnummer (nicht Nullindiziert!) des Startpunktes.
//! \param in_iEndNr Knotenpunktnummer (nicht Nullindiziert!) des Endpunktes.
//! \param out_iEdgeNr Randnummer auf der die Kante liegt.
//! \param out_iStartIndex Index der Knotenpunktnummer, die innerhalb der CFemEdge-Klasse benutzt wird.
//! \param out_iEndIndex Index der Knotenpunktnummer, die innerhalb der CFemEdge-Klasse benutzt wird.
//! \return true falls die Kante auf einem Rand liegt, sonst false.
bool CFemData::PartOfEdge(const int in_iStartNr, const int in_iEndNr, int& out_iEdgeNr, int& out_iStartIndex, int& out_iEndIndex) const
{
    //-- Alle Rnder durchsuchen
    //
    for (int i=0; i<m_iEdgeCount; i++)
    {
        bool bFoundStart=false;
        bool bFoundEnd  =false;

        int iNode=m_pEdge[i].GetNodeCount();

        while (
                (iNode) &&
               !(bFoundEnd && bFoundStart)
              )
        {
            //-- Wir gehen die Liste rckwrts durch
            //
            iNode--;

            //-- Solange ein Teilergebnis noch nicht bekannt ist weitersuchen
            //   sonst das Teilergebnis nicht mehr verndern.
            //
            if (!bFoundEnd)
            {
                bFoundEnd=m_pEdge[i].GetNodeNr(iNode)==in_iEndNr;
                out_iEndIndex=iNode;
            }
            if (!bFoundStart)
            {
                bFoundStart =m_pEdge[i].GetNodeNr(iNode)==in_iStartNr;
                out_iStartIndex=iNode;
            }
        }

        if (bFoundEnd && bFoundStart)
        {
            out_iEdgeNr=i;
            return true;
        }
    }
    return false;
}

//! liefert das Minimum und Maximum der U oder V Werte (bei EW-Berechnungen nutzlos)
//! \param in_iCalcNr Nullindizierter Index der Berechnung, Wertebereich [0 bis GetCalcCount()-1]
//! \param outMin Minimum
//! \param outMin Maximum
void CFemData::GetMinMaxValues( const in_iCalcNr, FP_TYPE &outMin, FP_TYPE &outMax )
{
    assert(m_pUVValue[in_iCalcNr] != NULL);

	outMin=FLT_MAX;
	outMax=-FLT_MAX;

	for(int i=0;i<m_iNodeCount;i++)
	{
		if(outMax<m_pUVValue[in_iCalcNr][i])outMax=m_pUVValue[in_iCalcNr][i];
		if(outMin>m_pUVValue[in_iCalcNr][i])outMin=m_pUVValue[in_iCalcNr][i];
	}
}


void CFemData::ResortFEM3Coord(const tagFEM3Coord* in_pFEM3Coord)
{
    assert(in_pFEM3Coord != NULL);
    tagCoord tCoord;
    for (int i=0; i<m_iNodeCount; i++)
    {
        //-- NodeNr darf nicht Null sein, falls doch wurde ein Zwischenpunkt
        //   nicht berechnet.
        //
        assert(in_pFEM3Coord[i].iNodeNr != 0);

        tCoord.x=in_pFEM3Coord[i].x;
        tCoord.y=in_pFEM3Coord[i].y;
        m_vNodeCoord.push_back(tCoord);
    }
    int iSize=m_vNodeCoord.size();
    iSize++;
}

