//-------------------------------------------------------------------+
// FemDataError.h                                                    |
//-------------------------------------------------------------------+
// Definiton of CFemDataError. Exception-Handling of FemDate-Errors  |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 31.01.2001
#ifndef _DVA_FEM_DATA_ERROR_H__
#define _DVA_FEM_DATA_ERROR_H__

#include <assert.h>
#include <string>
using namespace std;

/*!
    \brief Klasse zum Fehlerbehandlung von fehlerhaften FEM-Dateien
    \author Tobias Lang
    \version 1.0
    \date    2001
*/
class CFemDataError
{
private:
    string  m_sMessage;
    int     m_iLineNr;

public:
    //! Defaultkonstruktor
    CFemDataError():m_iLineNr(-1)
    {}

    //! Fehlerkonstruktur mit einer Textangabe und Zeilenummer
    //! \param in_pMessage Fehlermeldung
    //! \param in_iLineNr Zeilennummer
    CFemDataError(const char* in_pMessage, const int in_iLineNr):m_iLineNr(in_iLineNr)
    {
        assert(in_pMessage != NULL);
        m_sMessage=in_pMessage;
    }

    //! Fehlerkonstruktur mit zwei Textangaben ohne Zeilenummer
    //! \param in_pMessage1 erste Fehlermeldung
    //! \param in_pMessage2 zweite Fehlermeldung
    CFemDataError(const char* in_pMessage1, const char* in_pMessage2):m_iLineNr(-1)
    {
        assert(in_pMessage1 != NULL);
        assert(in_pMessage2 != NULL);
        m_sMessage=in_pMessage1;
        m_sMessage.append(in_pMessage2);
    }

    //! Fehlerkonstruktur mit zwei Textangaben und einer Zeilenummer
    //! \param in_pMessage1 erste Fehlermeldung
    //! \param in_pMessage2 zweite Fehlermeldung
    //! \param in_iLineNr Zeilennummer
    CFemDataError(const char* in_pMessage1, const char* in_pMessage2, const int in_iLineNr):m_iLineNr(in_iLineNr)
    {
        assert(in_pMessage1 != NULL);
        assert(in_pMessage2 != NULL);
        m_sMessage=in_pMessage1;
        m_sMessage.append(in_pMessage2);
    }

    //! Liefert die Zeilennummer in der Fehler auftratt.<br>
    //! Ein R&uuml;ckgabewert von -1 bedeutet, dass keine Zeileninformation vorhanden ist.
    //! \return Zeilenummer
    int         GetLineNr () const { return m_iLineNr;          };

    //! Liefert die Fehlermeldung.
    //! \return Ein mit Null terminierter String.
    const char* GetErrorMessage() const
    {
        static char sEmpty='\0';
        if (m_sMessage.empty())
            return &sEmpty;
        return m_sMessage.c_str();
    };
};

#endif //_DVA_FEM_DATA_ERROR_H__