#include <float.h>

#include "windows.h"		// void OutputDebugString( char *);

#include "FemProcessor.h"
#include "VectorNArray.h"		// CVectorNArray
#include "MatrixNMHull.h"		// CMatrixNMHull
#include "MatrixNMList.h"		// CMatrixNMList
#include "Algorithm.h"
#include "PerfTimer.h"
#include "FemEWHelper.h"


#include <algorithm>
#include <iomanip>


//! Defaultkonstruktor
CFemProcessor::CFemProcessor():
	m_pTimer(NULL)
{
}

	
CFemProcessor::CFemProcessor(CPerfTimer* in_pTimer):
    m_pTimer(in_pTimer)
{}

//! Destruktor, gibt den Speicher der Kantenliste frei
CFemProcessor::~CFemProcessor()
{
    FreeData();
}

//! Gibt dynamisch allokierten Speicher frei
void CFemProcessor::FreeData()
{
    m_FemData.FreeData();
    FreeGeoList();
}


//! &Ouml;ffnet eine Textdatei im FEM-Format und wertet deren Inhalt aus
//! \param in_pFilename Dateiname mit relativen oder absoluten Pfad
void CFemProcessor::LoadFemIn(const char* in_pFileName, const bool in_bLoadWithFemOut, CVectorNArray<FP_TYPE>* out_pEW, CMatrixNMArray<FP_TYPE>* out_pEV)
{
    assert(in_pFileName != NULL);

    try
    {
        m_FemData.LoadFemIn(in_pFileName, in_bLoadWithFemOut, out_pEW, out_pEV);

        //-- Fr jede Fem-Datei wird eine dazugehrige Log-Datei erstellt
        //
        if (m_ofLogFile.is_open())
            m_ofLogFile.close();

        string strLogFile=in_pFileName;
        strLogFile=strLogFile.substr(0, strLogFile.rfind("."));
        strLogFile+=".log";

        m_ofLogFile.open(strLogFile.c_str());
    }
    catch(CFemDataError &e)
    {
        throw e;
    }
}





//! Ermittelt die maximale und minimale Ausdehnung des FEM-Gebietes.
//! \param out_fMinX Minimaler x-Wert
//! \param out_fMaxX Maximaler x-Wert
//! \param out_fMinY Minimaler y-Wert
//! \param out_fMaxY Maximaler y-Wert
//! \return true bei Erfolg, false sonst
bool CFemProcessor::GetMinMaxCoord(float &out_fMinX, float &out_fMaxX, float &out_fMinY, float &out_fMaxY)
{
	out_fMinY=out_fMinX=FLT_MAX;
	out_fMaxY=out_fMaxX=-FLT_MAX;

	for (int i=0; i<m_FemData.GetNodeCount(); i++)
	{
		float x,y;
		m_FemData.GetNodeCoord(i, x, y);
		if (x>out_fMaxX)
			out_fMaxX=x;
		if (x<out_fMinX)
			out_fMinX=x;
		if (y>out_fMaxY)
			out_fMaxY=y;
		if (y<out_fMinY)
			out_fMinY=y;
	}
	return (i != 0);
}


