//-------------------------------------------------------------------+
// CFemProcessor.h                                                   |
//-------------------------------------------------------------------+
// Definiton of CFemProcessor-class.                                 |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 08.02.2001
#ifndef _DVA_FEM_PROCESSOR_H__
#define _DVA_FEM_PROCESSOR_H__

#include "FemData.h"
#include "FemGeoEdge.h"
#include "MatrixNMArray.h"
#include "PerfTimer.h"

#include <list>
#include <fstream>
typedef list<CFemGeoEdge*> GeoEdgeLIST;
typedef unsigned long DWORD;

/*!
    \brief Kapselt die Berechnung von Finiten-Elementen.
    \author Tobias Lang
    \version 0.5
    \date    2001
*/
class CFemProcessor
{
private:
    CFemData    m_FemData;  //!< Daten auf der die Berechnungen stattfinden

    GeoEdgeLIST m_GeoEdge;  //!< Liste mit den Kanten des FEM-Gebiets

    CPerfTimer* m_pTimer;       //!< Zeiger auf Performance-Timer
    ofstream    m_ofLogFile;    //!< Textdatei mit Statusausgaben im Zusammenhang mit dem Timer

public:
    //! Defaultkonstruktor
    CFemProcessor();

    //! Konstruktor
    CFemProcessor(CPerfTimer* in_pTimer);

    //! Destruktor, gibt den Speicher der Kantenliste frei
    ~CFemProcessor();

    //! &Ouml;ffnet eine Textdatei im FEM-Format und wertet deren Inhalt aus
    //! \param in_pFilename Dateiname mit relativen oder absoluten Pfad
    void LoadFemIn(const char* in_pFileName, const bool in_bLoadWithFemOut=false, CVectorNArray<FP_TYPE>* out_pEW=NULL, CMatrixNMArray<FP_TYPE>* out_pEV=NULL);


    //! Ermglich den Zugriff auf die FEM-Daten
    //! \return CFemData-Klasse
    CFemData& GetFemData() { return m_FemData; }

    //! Ermittelt die maximale und minimale Ausdehnung des FEM-Gebietes.
    //! \param out_fMinX Minimaler x-Wert
    //! \param out_fMaxX Maximaler x-Wert
    //! \param out_fMinY Minimaler y-Wert
    //! \param out_fMaxY Maximaler y-Wert
    //! \return true bei Erfolg, false sonst
	bool GetMinMaxCoord(float &out_fMinX, float &out_fMaxX, float &out_fMinY, float &out_fMaxY);



private:
    //! Gibt dynamisch allokierten Speicher frei
    void FreeData();

    //! Gibt den Speicher der GeoList frei
    void FreeGeoList()
    {
        for (GeoEdgeLIST::iterator it=m_GeoEdge.begin(); it != m_GeoEdge.end(); ++it)
            delete (*it);
        m_GeoEdge.clear();
    }


};

#endif //_DVA_FEM_PROCESSOR_H__