// MatrixNM.h: Schnittstelle fr die Klasse CMatrixNM.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MATRIXNM_H__FDE176C5_833B_474F_8026_B52C712FEB5E__INCLUDED_)
#define AFX_MATRIXNM_H__FDE176C5_833B_474F_8026_B52C712FEB5E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "VectorN.h"

// Mutter aller Matrixklassen
// von dieser sollen sich die anderen Matrixklassen ableiten

// Fehlerbehandlung mit exceptions oder assert

// Zuweisungen mit verschiedenem NxM bewirken einen Angleich

/*!
    \brief konkrete Implementierung: Speicherarray
    \author Martin Mitting
    \version 0.1
    \date    31.01.2001
*/
template <class T>
class CMatrixNM  
{
public:

	//! Defaultkonstruktor
								CMatrixNM		( void );


	//! Destruktor
	virtual						~CMatrixNM		( void );


	//! fr lesenden elementweisen Zugriff 
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
    //! \return Element in der Matrix
	const T						P				( int inZeile, int inSpalte ) const;		


	//! fr schreibenden elementweisen Zugriff 
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
	//! \param inValue Element in der Matrix
	void						P				( int inZeile, int inSpalte, T inValue );	

/*
	//! fr alle Elemente aufrufen. Vor der eigentlichen Kompilation ausfhren (fr FEM)
	//! (wird fr die Ermittlung der Matrixstruktur gebraucht, z.B. fr Hllenorierntierte Speicherung)
	//! \param inOrder Knotennummern des Elementes
	void						PreKompilation	( CVectorN<int> &inOrder  );	
*/

	//! Einarbeitung der Elementsteifigkeitsmatrizen in die Gesamtsteifigkeitsmatrix (fr FEM)
	//! vorher PreKompilation ausfhren
	//! \param inElementMat Elementsteifigkeitsmatrix
	//! \param inOrder Knotennummern in der Elementsteifigkeitsmatrix
	void						Kompilation		( CMatrixNM<T> &inElementMat, CVectorN<int> &inOrder );


	//! Randbedingungen einarbeiten (fr FEM)
	//! \param inoutRSide rechte Seite
	//! \param inNodeNumber Knotennummer
	//! \param inValue Wert, der an dem Knoten herrscht
	void						RBEinarbeiten	( CVectorN<T> &inoutRSide, int inNodeNumber, T inValue );


	//! Daten freigeben
	void						FreeData		( void );


	//! Anzahl der Zeilen zurckliefern
	int							GetN			( void );


	//! Anzahl der Spalten zurckliefern
	int							GetM			( void );

	int							CalcUsedMemory	( void );


	//! Ausgabe des Inhaltes
	void						Debug			( void );	// -> cout
};


template <class T>
CMatrixNM<T>::CMatrixNM( void )
{

}

template <class T>
CMatrixNM<T>::~CMatrixNM( void )
{

}


#endif // !defined(AFX_MATRIXNM_H__FDE176C5_833B_474F_8026_B52C712FEB5E__INCLUDED_)
