// MatrixNMArray.h: Schnittstelle fr die Klasse CMatrixNMArray.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MATRIXNMARRAY_H__BA874AA8_C588_4448_BCBA_0A555BCCDC10__INCLUDED_)
#define AFX_MATRIXNMARRAY_H__BA874AA8_C588_4448_BCBA_0A555BCCDC10__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "basis.h"			// choly	(Cholesky Zerlegung aus Engeln Mllges)
#include "u_proto.h"		// choly	(Cholesky Zerlegung aus Engeln Mllges)

#include <iostream>		    // cout
using namespace std;

#include "MatrixNM.h"
#include "VectorNArray.h"


// Fehlerbehandlung mit exceptions oder assert


// Zuweisungen mit verschiedenem N bewirken einen Angleich

/*!
    \brief konkrete Implementierung: Speicherarray
    \author Martin Mitting
    \version 0.1
    \date    31.01.2001
*/
template <class T>
class CMatrixNMArray :public CMatrixNM<T>
{
public:
	
	//! Defaultkonstruktor (es werden keine Daten angelegt)
								CMatrixNMArray		( void );


	//! Konstruktor (es wird eine NxM Matrix angelegt), alle Elemente werden auf 0 gesetzt
	//! \param inZeilen Anzahl der Zeilen der Matrix
	//! \param inSpalten Anzahl der Spalten der Matrix
								CMatrixNMArray		( int inZeilen, int inSpalten );


	//! Copy-Konstruktor
	//! \param inOrig Original, dessen Daten kopierten werden sollen
								CMatrixNMArray		( CMatrixNMArray<T> &inOrig );


	//! Konstruktor, Initialisieren mit einem Array (Gre mu NxM sein)
	//! \param inZeilen Anzahl der Zeilen der Matrix
	//! \param inSpalten Anzahl der Spalten der Matrix
	//! \param inpArray Zeiger auf das Array mit den Elementen
								CMatrixNMArray		( int inZeilen, int inSpalten, const T *inpArray );

	//! Destruktor
	virtual 					~CMatrixNMArray		( void );

	void						SetDiagonalValues	( T inValue );

	//! Gr&ouml;&szlig;e nachtr&auml;glich &auml;ndern, Elemente werden umkopiert, optimierbar
	//! \param inZeilen Anzahl der Zeilen der Matrix
	//! \param inSpalten Anzahl der Spalten der Matrix
	void						Resize				( int inZeilen, int inSpalten );

	//! Gr&ouml;&szlig;e nachtr&auml;glich &auml;ndern, Elemente werden immer gelscht
	//! \param inZeilen Anzahl der Zeilen der Matrix
	//! \param inSpalten Anzahl der Spalten der Matrix
	void						ResizeAndClear		( int inZeilen, int inSpalten );

	//! fr lesenden elementweisen Zugriff 
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
    //! \return Element in der Matrix
	const T						P					( int inZeile, int inSpalte ) const;		


	//! fr schreibenden elementweisen Zugriff 
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
	//! \param inValue Element in der Matrix
	void						P					( int inZeile, int inSpalte, const T inValue );	


	//! berechnet das Dyadische Produkt
	//! \param inZeilenV erster Parameter Zeilenvektor
	//! \param inSpaltenV zweiter Paramater Spaltenvektor
	void						DyadischesProd		( CVectorNArray<T> &inZeilenV, CVectorNArray<T> &inSpaltenV );


	//! Inplace transponieren
	void						Transpose			( void );

	//! Zuweisungsoperator mit gleichem Typ
	//! \param inOrig Original, das kopiert werden soll
	CMatrixNMArray<T> &			operator=			( CMatrixNMArray<T> &inOrig );

	//! Multipliktion mit gleichem Typ
	//! hiermit lt sich auch dass dyadische Produkt und das Skalarprodukt berechnen
	//! \param inP, Paramter mit dem multipliziert werden soll (von rechts)
	CMatrixNMArray<T> 			operator*			( CMatrixNMArray<T> &inP );

	//! Multipliktion mit einem Vector
	//! \param inP, Paramter mit dem multipliziert werden soll (von rechts)
	CVectorNArray<T> 			operator*			( CVectorNArray<T> &inP );


	//! Multipliktion mit einem Skalar
	//! \param inSkalar, Parameter mit dem multipliziert werden soll
	CMatrixNMArray<T> 			operator*			( T inSkalar );


	//! Addition mit gleichem Typ
	//! \param inP, Parameter mit dem addiert werden soll
	CMatrixNMArray<T>			operator+			( CMatrixNMArray<T> &inP );


	//! Subtraktion mit gleichem Typ
	//! \param inP, Parameter mit dem subtrahiert werden soll
	CMatrixNMArray<T>			operator-			( CMatrixNMArray<T> &inP );

	//! Addition mit gleichem Typ
	//! \param inP, Parameter mit dem addiert werden soll
	CMatrixNMArray<T>			operator+=			( CMatrixNMArray<T> &inP );

	//! Daten freigeben
	void						FreeData			( void );


	//! Anzahl der Zeilen zurckliefern
	int							GetN				( void ) const;


	//! Anzahl der Spalten zurckliefern
	int							GetM				( void ) const;

	//! Extrahiert einen best. Spaltenvektor
	void						GetVecNo			( int inSpalte, CVectorNArray<T> &inP );

	//! berechnet die belegte Speichermenge
	int							CalcUsedMemory		( void ) const;

	void						Combine				( const CMatrixNMArray<T> &a11, const CMatrixNMArray<T> &a12,
													  const CMatrixNMArray<T> &a21, const CMatrixNMArray<T> &a22 );



	//! nur frs Debugging
	void						Debug				( void );	// -> cout

protected:

	T *		m_Data;				//!< Zeiger auf den mit new erzeugten Speicherblock
	int		m_N;				//!< Anzahl der Zeilen des Vectors
	int		m_M;				//!< Anzahl der Spalten des Vectors
};



// Konstruktor
template <class T>
CMatrixNMArray<T>::CMatrixNMArray( void )
{
	m_Data=0;
	m_N=0;
	m_M=0;
}





// Konstruktor
template <class T>
CMatrixNMArray<T>::CMatrixNMArray( int inZeilen, int inSpalten )
{
	m_Data=new T[inZeilen*inSpalten];
	m_N=inZeilen;
	m_M=inSpalten;

	for(int i=0;i<m_N*m_M;i++)
		m_Data[i]=0;										// Inhalte lschen
}


// Copy-Konstruktor
template <class T>
CMatrixNMArray<T>::CMatrixNMArray( CMatrixNMArray<T> &inOrig )
{
	if(inOrig.m_N==0 || inOrig.m_N==0)throw "Matrix: Matrix hat keine Elemente";

	m_Data=new T[inOrig.m_N*inOrig.m_M];
	m_N=inOrig.m_N;
	m_M=inOrig.m_M;

	for(int i=0;i<m_N*m_M;i++)
		m_Data[i]=inOrig.m_Data[i];							// Inhalte kopieren
}



// Destruktor
template <class T>
CMatrixNMArray<T>::~CMatrixNMArray()
{
	FreeData();
}




// Daten freigeben
template <class T>
void CMatrixNMArray<T>::FreeData( void )
{
	delete [] m_Data;m_Data=0;
	m_N=0;m_M=0;
}



// Zuweisungoperator mit gleichem Typ
template <class T>
CMatrixNMArray<T> & CMatrixNMArray<T>::operator=( CMatrixNMArray<T> &inOrig )
{
	if(inOrig.m_N!=m_N || inOrig.m_M!=m_M)
	{
		FreeData();											// Daten freigeben

		m_Data=new T[inOrig.m_N*inOrig.m_M];
		m_N=inOrig.m_N;
		m_M=inOrig.m_M;
	}

	for(int i=0;i<m_N*m_M;i++)
		m_Data[i]=inOrig.m_Data[i];							// Inhalte kopieren

	return(*this);
}





// Gre nachtrglich ndern, Elemente werden umkopiert, optimierbar
template <class T>
void CMatrixNMArray<T>::Resize( int inZeilen, int inSpalten )
{
	if(inZeilen!=m_N || inSpalten!=m_M)						// Arbeit ntig?
	{
		if(inZeilen==0)
		{
			if(inSpalten==0)
				{ FreeData();return; }

			throw "Matrix: fehlerhafte Matrixgre bei Resize (inZeilen=0)";
		}
		if(inSpalten==0)
			throw "Matrix: fehlerhafte Matrixgre bei Resize (inSpalten=0)";

		
		T *Data=new T[inZeilen*inSpalten];					// neuen Speicher allocieren

		int sz=(inZeilen<m_N)?inZeilen:m_N;					// den kleineren ermitteln
		int ss=(inSpalten<m_N)?inSpalten:m_M;				// den kleineren ermitteln

		for(int y=0;y<sz;y++)
		{
			for(int x=0;x<ss;x++)
			{
				Data[y*inSpalten+x]=m_Data[y*m_M+x];		// Inhalte kopieren
			}
			for(;x<inSpalten;x++)
				Data[y*inSpalten+x]=0;						// briges lschen
		}

		for(;y<inZeilen;y++)
		for(int x=0;x<inSpalten;x++)
			Data[y*inSpalten+x]=0;							// briges lschen

		delete [] m_Data;
		m_Data=Data;
		m_N=inZeilen;
		m_M=inSpalten;
	}
}

// Gre nachtrglich ndern, Elemente werden alle gelscht
template <class T>
void CMatrixNMArray<T>::ResizeAndClear( int inZeilen, int inSpalten )
{
	if(inZeilen!=m_N || inSpalten!=m_M)						// Arbeit ntig?
	{
		if(inZeilen==0)
		{
			if(inSpalten==0)
				{ FreeData();return; }

			throw "Matrix: fehlerhafte Matrixgre bei ResizeAndClear (inZeilen=0)";
		}
		if(inSpalten==0)
			throw "Matrix: fehlerhafte Matrixgre bei ResizeAndClear (inSpalten=0)";

		
		T *Data=new T[inZeilen*inSpalten];					// neuen Speicher allocieren

		delete [] m_Data;
		m_Data=Data;
		m_N=inZeilen;
		m_M=inSpalten;
	}

	for(int i=0;i<m_N*m_M;i++)
		m_Data[i]=0;										// Inhalte lschen
}


// Multipliktion mit gleichem Typ
// hiermit lt sich auch das dyadische Produkt und das Skalarprodukt berechnen
template <class T>
CMatrixNMArray<T> CMatrixNMArray<T>::operator*( CMatrixNMArray<T> &inP )
{
	int n=GetN();
	int m=GetM();

	if(inP.GetN()!=m)throw "Matrix: falsche Matrixgren bei Multiplikation";
	if(inP.GetM()!=n)throw "Matrix: falsche Matrixgren bei Multiplikation";

	CMatrixNMArray<T> ret(n,n);								// Ausgabe ist eine NxN Matrix

	for(int x=0;x<n;x++)
	for(int y=0;y<n;y++)
	{
		T value=0;

		for(int i=0;i<m;i++)
			value+=P(y,i)*inP.P(i,x);

		ret.P(y,x,value);
	}

	return(ret);
}


template <class T>
void CMatrixNMArray<T>::DyadischesProd( CVectorNArray<T> &inZeilenV, CVectorNArray<T> &inSpaltenV )
{
	int n=inZeilenV.GetN(),m=inSpaltenV.GetN();

	if(GetM()!=m)throw "Matrix: falsche Matrixgre bei dyadischem Produkt";
	if(GetN()!=n)throw "Matrix: falsche Matrixgre bei dyadischem Produkt";

	for(int y=0;y<n;y++)
	for(int x=0;x<m;x++)
	{
		P(y,x,inZeilenV.P(y)*inSpaltenV.P(x));
	}
}




// Multipliktion mit einem Vector
template <class T>
CVectorNArray<T> CMatrixNMArray<T>::operator*( CVectorNArray<T> &inP )
{
	int n=GetN();
	int m=GetM();

	if(inP.GetN()!=m)throw "Matrix: falsche Vector/Matrixgren bei Multiplikation mit Vector";

	CVectorNArray<T> ret(n);								// Ausgabe ist ein N Vector

	for(int x=0;x<n;x++)
	{
		T value=0;

		for(int i=0;i<m;i++)
			value+=P(x,i)*inP.P(i);

		ret.P(x,value);
	}


	return(ret);
}




// Addition mit gleichem Typ
template <class T>
CMatrixNMArray<T> CMatrixNMArray<T>::operator+( CMatrixNMArray<T> &inP )
{
	int n=GetN();
	int m=GetM();

	if(inP.GetN()!=n)throw "Matrix: falsche Matrixgren bei Addition";
	if(inP.GetM()!=m)throw "Matrix: falsche Matrixgren bei Addition";

	CMatrixNMArray<T> ret(n,m);								// Ausgabe ist eine NxM Matrix

	for(int y=0;y<n;y++)
	for(int x=0;x<m;x++)
		ret.P(y,x, P(y,x)+inP.P(y,x) );						// Elementweise addieren

	return(ret);
}

// Subtraktion mit gleichem Typ
template <class T>
CMatrixNMArray<T> CMatrixNMArray<T>::operator-( CMatrixNMArray<T> &inP )
{
	int n=GetN();
	int m=GetM();

	if(inP.GetN()!=n)throw "Matrix: falsche Matrixgren bei Addition";
	if(inP.GetM()!=m)throw "Matrix: falsche Matrixgren bei Addition";

	CMatrixNMArray<T> ret(n,m);								// Ausgabe ist eine NxM Matrix

	for(int y=0;y<n;y++)
	for(int x=0;x<m;x++)
		ret.P(y,x, P(y,x)-inP.P(y,x) );						// Elementweise subtrahieren

	return(ret);
}


// Addition mit gleichem Typ
template <class T>
CMatrixNMArray<T> CMatrixNMArray<T>::operator+=( CMatrixNMArray<T> &inP )
{
	int n=GetN();
	int m=GetM();

	if(inP.GetN()!=n)throw "Matrix: falsche Matrixgren bei Addition";
	if(inP.GetM()!=m)throw "Matrix: falsche Matrixgren bei Addition";

	for(int y=0;y<n;y++)
	for(int x=0;x<m;x++)
		P(y,x, P(y,x)+inP.P(y,x) );						// Elementweise addieren

	return(*this);
}

// Multipliktion mit einem Skalar
template <class T>
CMatrixNMArray<T> CMatrixNMArray<T>::operator*( T inSkalar )
{
	int n=GetN();
	int m=GetM();

	CMatrixNMArray<T> ret(n,m);								// Ausgabe ist eine NxM Matrix

	for(int y=0;y<n;y++)
	for(int x=0;x<m;x++)
	{
		ret.P(y,x, P(y,x)*inSkalar );						// Elementweise multiplizieren
	}

	return(ret);
}


// lesender Zugriff (von 0 an gezhlt)
template <class T>
const T CMatrixNMArray<T>::P( int inZeile, int inSpalte ) const
{
	if(!m_Data)throw "Matrix: lesender Elementzugriff, m_Data";
	if(((unsigned long)inZeile)>=m_N)throw "Matrix: lesender Elementzugriff, inZeile grer als Zeilengre oder inZeile negativ";
	if(((unsigned long)inSpalte)>=m_M)throw "Matrix: lesender Elementzugriff, inSpalte grer als Spaltengre oder inSpalte negativ";

	return(m_Data[inZeile*m_M+inSpalte]);
}


// schreibender Zugriff (von 0 an gezhlt)
template <class T>
void CMatrixNMArray<T>::P( int inZeile, int inSpalte, const T inValue )
{
	if(!m_Data)throw "Matrix: schreibender Elementzugriff, m_Data";
	if(((unsigned long)inZeile)>=m_N)throw "Matrix: lesender Elementzugriff, inZeile grer als Zeilengre oder inZeile negativ";
	if(((unsigned long)inSpalte)>=m_M)throw "Matrix: lesender Elementzugriff, inSpalte grer als Spaltengre oder inSpalte negativ";

	m_Data[inZeile*m_M+inSpalte]=inValue;
}








// Anzahl der Zeilen zurckliefern
template <class T>
int CMatrixNMArray<T>::GetN( void ) const
{
	return(m_N);
}

// Anzahl der Spalten zurckliefern
template <class T>
int	CMatrixNMArray<T>::GetM( void ) const
{
	return(m_M);
}




//! Initialisieren mit einem Array (Gre mu NxM sein)
//! \param inpArray Zeiger auf das Array mit den Elementen
template <class T>
CMatrixNMArray<T>::CMatrixNMArray( int inZeilen, int inSpalten, const T *inpArray )
{
	assert(inpArray);
	// hier noch bessere Fehlerabfragen machen

	m_Data=m_Data=new T[inZeilen*inSpalten];
	m_N=inZeilen;
	m_M=inSpalten;

	for(int x=0;x<m_M*m_N;x++)
		m_Data[x]=*inpArray++;
}


template <class T>
void CMatrixNMArray<T>::Debug( void )	// -> cout
{
	if(m_Data)
	{
		for(int y=0;y<GetN();y++)
		{
			for(int x=0;x<GetM();x++)
			{
				cout.width(6);
				cout.precision(3);
				cout << P(y,x) << " ";
			}
			cout << endl;
		}
	}
	else
	{
		cout << "CMatrixNMArray is empty";
	}
	cout << endl;
}






template <class T>
int	CMatrixNMArray<T>::CalcUsedMemory( void ) const
{
	return(n_M*n_N+sizeof(T));			
}


// optimierbar
template <class T>
void CMatrixNMArray<T>::SetDiagonalValues( T inValue )
{
	int end=min(GetM(),GetN());

	for(int i=0;i<end;i++)
		P(i,i,inValue);
}

//! Inplace transponieren
template <class T>
void CMatrixNMArray<T>::Transpose( void )
{
	int n=GetN();
	
	if(n!=GetM())throw "Transpose: Matrix ist nicht quadratisch";

	for(int y=1;y<n;y++)
	for(int x=0;x<y;x++)
	{
		T swap=P(y,x);

		P(y,x, P(x,y));
		P(x,y, swap);
	}
}




#endif // !defined(AFX_MATRIXNMARRAY_H__BA874AA8_C588_4448_BCBA_0A555BCCDC10__INCLUDED_)
