// MatrixNMArray.h: Schnittstelle fr die Klasse CMatrixNMHull.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MATRIXNMHULL_H__BA874AA8_C588_4448_BCBA_0A555BCCDC10__INCLUDED_)
#define AFX_MATRIXNMHULL_H__BA874AA8_C588_4448_BCBA_0A555BCCDC10__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "limits.h"
#include "basis.h"			// choly	(Cholesky Zerlegung aus Engeln Mllges)
#include "u_proto.h"		// choly	(Cholesky Zerlegung aus Engeln Mllges)

#include <iostream>		    // cout
using namespace std;

#include "MatrixNM.h"
#include "MatrixNMArray.h"
#include "VectorNArray.h"

#include "rqm.h"			// vorkonditionierte Rayleigh-Quoteint-Minimierung mittels konjugierter Gradienten der allg. EWA  A * x = lambda * B * x


// Fehlerbehandlung mit exceptions oder assert


// Zuweisungen mit verschiedenem N bewirken einen Angleich

/*!
    \brief konkrete Implementierung: Hllenorientierte Speicherung, nur fr symetrische Mat, untere Dreickesmatrix wird gespeichert
    \author Martin Mitting
    \version 0.1
    \date    31.01.2001
*/
template <class T>
class CMatrixNMHull :public CMatrixNM<T>
{
public:
	
	//! Defaultkonstruktor fhrt zu Zustand a)
								CMatrixNMHull	( void );


	//! Konstruktor fhrt zu Zustand b)
	//! \param inZeilen Anzahl der Zeilen der Matrix
								CMatrixNMHull	( int inZeilen );
	

	//! Copy-Konstruktor
	//! \param inOrig Original, dessen Daten kopierten werden sollen
								CMatrixNMHull	( CMatrixNMHull<T> &inOrig );


	//! Destruktor
	virtual 					~CMatrixNMHull	( void );


	//! fr Tests, ob eine Element in der Hlle ist, nur fr Zustand c) oder d)
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
	bool						exist			( int inZeile, int inSpalte ) const;		


	//! fr lesenden elementweisen Zugriff, nur fr Zustand d)
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
    //! \return Element in der Matrix
	const T						P				( int inZeile, int inSpalte ) const;		

	//! fr lesenden elementweisen Zugriff, nur fr Zustand d)
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
    //! \return Element in der Matrix
	const T						safeP			( int inZeile, int inSpalte ) const;		


	//! fr schreibenden elementweisen Zugriff, nur fr Zustand d)
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inSpalte Spalte in der Matrix (von 0 an gezhlt)
	//! \param inValue Element in der Matrix
	void						P				( int inZeile, int inSpalte, T inValue );	

	DWORD						CountElementsInLine( int inZeile ) const ;

	DWORD						CalcNeededMemory( void ) const;
	DWORD						CalcElementCount( void ) const;

	//! Gr&ouml;&szlig;e nachtr&auml;glich &auml;ndern, Elemente werden immer gelscht
	//! \param inZeilen Anzahl der Zeilen der Matrix
//	void						ResizeAndClear		( int inZeilen );



	//! Zweisungsoperator
	//! \param inOrig Original, das kopiert werden soll
	CMatrixNMHull<T> &			operator=		( CMatrixNMHull<T> &inOrig );

/*
	//! Multipliktion mit gleichem Typ
	//! hiermit lt sich auch dass dyadische Produkt und das Skalarprodukt berechnen
	//! \param inP, Paramter mit dem multipliziert werden soll (von rechts)
	CMatrixNMHull<T> 			operator*		( CMatrixNMHull<T> &inP );


	//! Multipliktion mit einem Skalar
	//! \param inSkalar, Paramter mit dem multipliziert werden soll (von rechts)
	CMatrixNMHull<T> 			operator*		( T inSkalar );


	//! Addition mit gleichem Typ
	//! \param inP, Paramter mit dem addiert werden soll
	CMatrixNMHull<T>			operator+		( CMatrixNMHull<T> &inP );
*/

	//! Daten freigeben fhrt zu Zustand a)
	void						FreeData		( void );


	//! Anzahl der Zeilen zurckliefern
	int							GetN			( void ) const;


	//! Ausgabe des Inhaltes
	void						Debug			( void ) const;	// -> cout


protected:

	T **	m_LineData;			//!< Zeiger auf die mit new erzeugten Speicherblcke mit den Daten
	int		*m_LineStart;		//!< Index auf den Anfang der Daten einer Zeile
	int		m_N;				//!< Anzahl der Zeilen des Vectors (m_Lines[0..n_N])

};


// mgliche Zustnde:
//
// a) nach Defaultkonstruktor (nichts allociert, keine Zeilenzahl festgelegt)
// b) nach Konstruktor(Zeilen) (m_LineStart wird mit MAX_INT initialisiert, keine Daten)
// c) nach PreKompilation (m_LineStart mit ntiger Abmessung initialisiert, keine Daten)
// d) nach und in Kompilation (m_LineStart initialisiert, mit Daten befllt)




// Konstruktor fhrt zu Zustand a)
template <class T>
CMatrixNMHull<T>::CMatrixNMHull( void )
{
	m_LineData=0;
	m_LineStart=0;
	m_N=0;
}





// Konstruktor fhrt zu Zustand b)
template <class T>
CMatrixNMHull<T>::CMatrixNMHull( int inZeilen )
{
	m_N=inZeilen;
	m_LineData=new T*[inZeilen];
	m_LineStart=new int[inZeilen];

	for(int i=0;i<m_N;i++)
	{
		m_LineStart[i]=INT_MAX;				// Inhalte lschen
		m_LineData[i]=0;
	}
}


/*
// optimierbar, fhrt zu Zustand b)
template <class T>
void CMatrixNMHull<T>::ResizeAndClear( int inZeilen )
{
	FreeData();

	m_N=inZeilen;
	m_LineData=new T*[inZeilen];
	m_LineStart=new int[inZeilen];

	for(int i=0;i<m_N;i++)
	{
		m_LineStart[i]=INT_MAX;				// Inhalte lschen
		m_LineData[i]=0;
	}
}
*/



// Copy-Konstruktor
template <class T>
CMatrixNMHull<T>::CMatrixNMHull( CMatrixNMHull<T> &inOrig )
{
	if(inOrig.GetN()==0)throw "Matrix: Matrix hat keine Elemente";

	m_N=inOrig.m_N;
	m_LineData=new T*[m_N];
	m_LineStart=new int[m_N];

	for(int i=0;i<m_N;i++)
	{
		m_LineStart[i]=inOrig.m_LineStart[i];				// Inhalte kopieren

		if(inOrig.m_LineData[i])
		{
			m_LineData[i]=new T[i+1-m_LineStart[i]];

			for(int e=0;e<i+1-m_LineStart[i];e++)
				m_LineData[i][e]=inOrig.m_LineData[i][e];
		}
		else m_LineData[i]=0;
	}
}



// Destruktor
template <class T>
CMatrixNMHull<T>::~CMatrixNMHull()
{
	FreeData();
}



// Daten freigeben, fhrt wieder zum Zustand a)
template <class T>
void CMatrixNMHull<T>::FreeData( void )
{
	if(m_LineData)
	{
		for(int i=0;i<m_N;i++)
			delete [] m_LineData[i];
	}
	delete [] m_LineData;m_LineData=0;
	delete [] m_LineStart;m_LineStart=0;
	m_N=0;
}


template <class T>
DWORD CMatrixNMHull<T>::CalcNeededMemory( void ) const
{
	return(CalcElementCount()*sizeof(T));
}



template <class T>
DWORD CMatrixNMHull<T>::CalcElementCount( void ) const
{
	DWORD ret=0;

	for(int i=0;i<m_N;i++)
	{
		ret+=i+1-m_LineStart[i];
	}

	return(ret);
}



// Zuweisungoperator
template <class T>
CMatrixNMHull<T> & CMatrixNMHull<T>::operator=( CMatrixNMHull<T> &inOrig )
{
	FreeData();

	m_N=inOrig.m_N;
	m_LineData=new T*[m_N];
	m_LineStart=new int[m_N];
	
	for(int i=0;i<m_N;i++)
	{
		m_LineStart[i]=inOrig.m_LineStart[i];				// Inhalte kopieren
		
		if(inOrig.m_LineData[i])
		{
			m_LineData[i]=new T[i+1-m_LineStart[i]];

			for(int e=0;e<i+1-m_LineStart[i];e++)
				m_LineData[i][e]=inOrig.m_LineData[i][e];
		}
		else m_LineData[i]=0;
	}

	return(*this);
}




/*

// Multipliktion mit gleichem Typ
// hiermit lt sich auch das dyadische Produkt und das Skalarprodukt berechnen
template <class T>
CMatrixNMHull<T> CMatrixNMHull<T>::operator*( CMatrixNMHull<T> &inP )
{
	int n=GetN();
	int m=GetM();

	if(inP.GetN()!=m)throw "Matrix: falsche Matrixgren bei Multiplikation";
	if(inP.GetM()!=n)throw "Matrix: falsche Matrixgren bei Multiplikation";

	CMatrixNMHull<T> ret(n,n);		// Ausgabe ist eine NxN Matrix

	for(int y=0;y<n;y++)
	for(int x=0;x<n;x++)
	{
		T value=0;

		for(int i=0;i<m;i++)
			value+=P(i,y)*inP.P(x,i);

		ret.P(x,y,value);
	}

	return(ret);
}




// Addition mit gleichem Typ
template <class T>
CMatrixNMHull<T> CMatrixNMHull<T>::operator+( CMatrixNMHull<T> &inP )
{
	int n=GetN();
	int m=GetM();

	if(inP.GetN()!=n)throw "Matrix: falsche Matrixgren bei Addition";
	if(inP.GetM()!=m)throw "Matrix: falsche Matrixgren bei Addition";

	CMatrixNMHull<T> ret(n,m);				// Ausgabe ist eine NxM Matrix

	for(int y=0;y<n;y++)
	for(int x=0;x<m;x++)
	{
		ret.P(x,y, P(x,y)+inP.P(x,y) );		// Elementweise addieren
	}

	return(ret);
}



// Multipliktion mit einem Skalar
template <class T>
CMatrixNMHull<T> CMatrixNMHull<T>::operator*( T inSkalar )
{
	int n=GetN();
	int m=GetM();

	CMatrixNMHull<T> ret(n,m);				// Ausgabe ist eine NxM Matrix

	for(int y=0;y<n;y++)
	for(int x=0;x<m;x++)
	{
		ret.P(x,y, P(x,y)*inSkalar );		// Elementweise multiplizieren
	}

	return(ret);
}

*/


	//! fr Tests, ob eine Element in der Hlle ist, nur fr Zustand c) oder d)
template <class T>
bool CMatrixNMHull<T>::exist( int inZeile, int inSpalte ) const
{
	if(!m_LineData)throw "Matrix: exist, m_LineData";

	if(inZeile<0)return(false);
	if(inZeile>=m_N)return(false);

	int start=m_LineStart[inZeile];
	int end=inZeile+1;

	// eigentlich keine exception, aber Verwendung macht nur innerhalb der Hlle Sinn.
	if(inSpalte<start)return(false);
	if(inSpalte>=end)return(false);

	return(true);
}


template <class T>
const T CMatrixNMHull<T>::safeP( int inZeile, int inSpalte ) const
{
	if(inSpalte<=inZeile)
	{
		if(!exist(inZeile,inSpalte))return(0);

		return(P(inZeile,inSpalte));
	}
	else
	{
		if(!exist(inSpalte,inZeile))return(0);

		return(P(inSpalte,inZeile));
	}
}


// lesender Zugriff (von 0 an gezhlt), nur fr Zustand d)
template <class T>
const T CMatrixNMHull<T>::P( int inZeile, int inSpalte ) const
{
	if(!m_LineData)throw "Matrix: lesender Elementzugriff, m_LineData";
	if(inZeile<0)throw "Matrix: lesender Elementzugriff, inZeile negativ";
	if(inZeile>=m_N)throw "Matrix: lesender Elementzugriff, inZeile grer als Zeilengre";

	int start=m_LineStart[inZeile];
	int end=inZeile+1;

	// eigentlich keine exception, aber Verwendung macht nur innerhalb der Hlle Sinn.
	if(inSpalte<start)
		throw "Matrix: lesender Elementzugriff, inSpalte links der Hlle";
	if(inSpalte>=end)
		throw "Matrix: lesender Elementzugriff, inSpalte rechts der Hlle";

	assert(m_LineData[inZeile]);
	return(m_LineData[inZeile][inSpalte-start]);
}

template <class T>
DWORD CMatrixNMHull<T>::CountElementsInLine( int inZeile ) const
{
	if(!m_LineData)throw "Matrix: lesender Elementzugriff, m_LineData";
	if(inZeile<0)throw "Matrix: lesender Elementzugriff, inZeile negativ";
	if(inZeile>=m_N)throw "Matrix: lesender Elementzugriff, inZeile grer als Zeilengre";

	int start=m_LineStart[inZeile];
	int end=inZeile+1;

	return(end-start);
}


// schreibender Zugriff (von 0 an gezhlt), nur fr Zustand d)
template <class T>
void CMatrixNMHull<T>::P( int inZeile, int inSpalte, T inValue )
{
	if(!m_LineData)throw "Matrix: schreibender Elementzugriff, m_LineData";
	if(inZeile<0)throw "Matrix: schreibender Elementzugriff, inZeile negativ";
	if(inZeile>=m_N)throw "Matrix: schreibender Elementzugriff, inZeile grer als Zeilengre";

	int start=m_LineStart[inZeile];
	int end=inZeile+1;

	// eigentlich keine exception, aber Verwendung macht nur innerhalb der Hlle Sinn.
	if(inSpalte<start)
 		throw "Matrix: schreibender Elementzugriff, inSpalte links der Hlle";
	if(inSpalte>=end)
		throw "Matrix: schreibender Elementzugriff, inSpalte rechts der Hlle";

	assert(m_LineData[inZeile]);
	m_LineData[inZeile][inSpalte-start]=inValue;
}








// Anzahl der Zeilen zurckliefern
template <class T>
int CMatrixNMHull<T>::GetN( void ) const
{
	return(m_N);
}



template <class T>
void CMatrixNMHull<T>::Debug( void ) const	// -> cout
{
	if(m_LineData)
	{
		for(int y=0;y<GetN();y++)
		{
			cout << "[" << m_LineStart[y] << ".." << y << "]= ";
			
			if(m_LineData[y]!=0)
			{
				for(int x=0;x<GetN();x++)
				{
					cout.width(5);
					cout.precision(2);

					cout << safeP(y,x) << " ";
				}
			}
			else cout << "not allocated";

			cout << endl;
		}
	}
	else
	{
		cout << "CMatrixNMHull is empty";
	}
	cout << endl;
}




#endif // !defined(AFX_MATRIXNMHULL_H__BA874AA8_C588_4448_BCBA_0A555BCCDC10__INCLUDED_)
