//-------------------------------------------------------------------+
// PerfTimer.h                                                       |
//-------------------------------------------------------------------+
// Kapselt den Zugriff auf den PerformanceCounter von Windows        |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 04.03.2001
#ifndef _PERF_TIMER_H__
#define _PERF_TIMER_H__

#include <windows.h>

/*!
    \brief Kapselt den Zugriff auf den PerformanceCounter von Windows.
    \author Tobias Lang
    \version 1.0
    \date    2001
*/
class CPerfTimer
{
private:
    __int64     m_iStartCounter;    //!< Erster Z&auml;hlwert, der als Bezugspunkt benutzt wird.
    double      m_dCounterRate;     //!< Zeit, die f&uuml;r einen Takt ben&ouml;ntigt wird.

    __int64     m_iBeginStopCounter;    //!< Zeitpunkt, ab dem die Zeitmessung begann.
public:
    //! Defaultkonstruktor setzt alle Attribute auf Null.
    CPerfTimer():
      m_iStartCounter(0),
      m_dCounterRate(0),
      m_iBeginStopCounter(0)
    {}

    //! Initialisert den Timer
    //! \return true bei Erfolg, sonst false
    bool Init()
    {
        __int64 iCounterFrequency;
        if (!QueryPerformanceFrequency((LARGE_INTEGER*)&iCounterFrequency))
            return false;

        if (iCounterFrequency==0)
            return false;

        m_dCounterRate=1.0/iCounterFrequency;

        if (!QueryPerformanceCounter((LARGE_INTEGER*)&m_iStartCounter))
            return false;

        return true;
    }


    //! Liefert die Zeit in msec, die seit dem Init-Aufruf verstrichen ist.
    //! \return Zeitstempel in ms.
    double GetTimeStamp() const
    {
        __int64 iEndCounter;
        QueryPerformanceCounter((LARGE_INTEGER*)&iEndCounter);
        return ((iEndCounter-m_iStartCounter)*m_dCounterRate)*1000;
    }

    //! Initiert eine neue Zeitmessung.
    void BeginStop()
    {
        QueryPerformanceCounter((LARGE_INTEGER*)&m_iBeginStopCounter);
    }

    //! Beendet eine Zeitmessung und liefert die verstrichene Zeit sein dem BeginStop-Aufruf.
    //! \return Zeit in ms.
    double EndStop() const
    {
        __int64 iEndCounter;
        QueryPerformanceCounter((LARGE_INTEGER*)&iEndCounter);
        return ((iEndCounter-m_iBeginStopCounter)*m_dCounterRate)*1000;
    }
};


#endif//_PERF_TIMER_H__