//-------------------------------------------------------------------+
// ScrewDriver.h                                                     |
//-------------------------------------------------------------------+
// Definiton of CScrewDriver-class.                                  |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 12.07.2001
#ifndef _DVA_SCREW_DRIVER_H__
#define _DVA_SCREW_DRIVER_H__

#include <cstdlib>

struct NodeForce
{
    int     iNodeNr;
    float   fXDirection;
    float   fYDirection;
};

class CScrewDriver
{
private:
    int         m_iForceCount;  //!< Anzahl der angreifenden Krfte
    NodeForce*  m_pForce;       //!< Vektor mit den Krften

public:
    double      m_dEModul;      //!< Elastizittsmodul
    float       m_fPoisson;     //!< Poissonzahl bzw. Querkontraktionszahl
    float       m_fThickness;   //!< Materialstrke

public:
    CScrewDriver()
        :m_pForce(NULL),
         m_iForceCount(0),
         m_fThickness(0),
         m_fPoisson(0),
         m_dEModul(0)
    {}

    ~CScrewDriver()
    {
        delete [] m_pForce;
    }


    void SetForceCount(const int in_iForceCount)
    {
        m_iForceCount=in_iForceCount;
        delete [] m_pForce;
        m_pForce=new NodeForce[m_iForceCount];
    }

    int GetForceCount() const { return m_iForceCount; }

    //! Setzt einen Eintrag in der Krfte-Tabelle
    //! \param in_iIndex Nullindizierter Index des Tabelleneintrags.
    //! \param in_iNodeNr Nummer des Knotenpunktes, an dem die Kraft angreift.
    //! \param in_fXDirection Anteil der Kraft in X-Richtung
    //! \param in_fYDirection Anteil der Kraft in Y-Richtung
    void SetForce(const int in_iIndex, const int in_iNodeNr, const float in_fXDirection, const float in_fYDirection)
    {
        if (m_pForce)
        if ((in_iIndex >= 0) && (in_iIndex < m_iForceCount))
        {
            m_pForce[in_iIndex].iNodeNr     =in_iNodeNr;
            m_pForce[in_iIndex].fXDirection =in_fXDirection;
            m_pForce[in_iIndex].fYDirection =in_fYDirection;
        }
    }

    //! Zur Abfrage eines Eintrags in der Krfte-Tabelle
    //! \param in_iIndex Nullindizierter Index des Tabelleneintrags.
    //! \param out_iNodeNr Nummer des Knotenpunktes, an dem die Kraft angreift.
    //! \param out_fXDirection Anteil der Kraft in X-Richtung
    //! \param out_fYDirection Anteil der Kraft in Y-Richtung
    void GetForce(const int in_iIndex, int& out_iNodeNr, float& out_fXDirection, float& out_fYDirection) const
    {
        if (m_pForce)
        if ((in_iIndex >= 0) && (in_iIndex < m_iForceCount))
        {
            out_iNodeNr     =m_pForce[in_iIndex].iNodeNr;
            out_fXDirection =m_pForce[in_iIndex].fXDirection;
            out_fYDirection =m_pForce[in_iIndex].fYDirection;
        }
    }

    //! Liefert einen Eintrag aus der Krfte-Tabelle
    //! \param in_iIndex Nullindizierter Index des Tabelleneintrags.
    //! \return Eintrag
    const NodeForce& GetForce(const int in_iIndex) const
    {
        assert(m_pForce != NULL);
        return m_pForce[in_iIndex];
    }
};


#endif//_DVA_SCREW_DRIVER_H__
