// VectorNArray.h: Schnittstelle fr die Klasse CVectorNArray.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VECTORNARRAY_H__C4EB09D0_A0E1_4D43_9EC5_BC40407F2A7C__INCLUDED_)
#define AFX_VECTORNARRAY_H__C4EB09D0_A0E1_4D43_9EC5_BC40407F2A7C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "VectorN.h"

// Fehlerbehandlung mit exceptions oder assert


// Zuweisungen mit verschiedenem N bewirken einen Angleich

/*!
    \brief konkrete Implementierung: Speicherarray
    \author Martin Mitting
    \version 0.2
    \date    31.01.2001
*/
template <class T>
class CVectorNArray : public CVectorN<class T>  
{
public:

	//! Defaultkonstruktor
								CVectorNArray	( void );

	//! Konstruktor
	//! \param inZeilen Anzahl der Zeilen des Vectors
								CVectorNArray	( int inZeilen );

	//! Konstruktor, Initialisieren mit einem Array (Gre mu N sein)
	//! \param inZeilen Anzahl der Zeilen des Vectors
	//! \param inpArray Zeiger auf das Array mit den Elementen
								CVectorNArray	( int inZeilen, const T *inpArray );

	//! Copy-Konstruktor
	//! \param inOrig Original, dessen Daten kopierten werden sollen
								CVectorNArray	( CVectorNArray<T> &inOrig );

	//! Destruktor
	virtual						~CVectorNArray	( void );

	//! Zuweisungoperator
	//! \param inOrig Original, das kopiert werden soll
	CVectorNArray<T> &			operator=		( CVectorNArray<T> &inOrig );

	//! Multipliktion mit einem Skalar
	//! \param inSkalar, Parameter mit dem multipliziert werden soll
	CVectorNArray<T> 			operator*		( T inSkalar );

	//! Multipliktion mit gleichem Typ (Skalarprodukt)
	//! \param inSkalar, Parameter mit dem multipliziert werden soll
	T				 			operator*		( CVectorNArray<T> &inP );

	//! Addition mit gleichem Typ
	//! \param inP, Parameter mit dem addiert werden soll
	CVectorNArray<T>			operator+		( CVectorNArray<T> &inP );

	//! Subtraktion mit gleichem Typ
	//! \param inP, Parameter mit dem subtrahiert werden soll
	CVectorNArray<T>			operator-		( CVectorNArray<T> &inP );

	//! Gr&ouml;&szlig;e nachtr&auml;glich &auml;ndern, Elemente werden umkopiert, evtl. optimierbar
	//! \param inZeilen Anzahl der Zeilen des Vectors
	void						Resize			( int inZeilen );

	//! fr lesenden elementweisen Zugriff 
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
    //! \return Element in der Matrix
	const T						P				( int inZeile ) const;		

	//! fr schreibenden elementweisen Zugriff 
	//! \param inZeile Zeile in der Matrix (von 0 an gezhlt)
	//! \param inValue Element in der Matrix
	void						P				( int inZeile, T inValue );	

	//! Euklidische Distanz
	T							Length			( void );

	//! Anzahl der Zeilen zurckliefern
	int							GetN			( void );

	//! Daten freigeben
	void						FreeData		( void );

	T *							GetMemoryAccess	( void );

	void						Debug			( void );	// -> cout

protected:

	T *		m_Data;				//!< Zeiger auf den mit new erzeugten Speicherblock
	int		m_N;				//!< Anzahl der Knoten auf dem Rand
};


// Konstruktor
template <class T>
CVectorNArray<T>::CVectorNArray()
{
	m_Data=0;
	m_N=0;
}

// Konstruktor
template <class T>
CVectorNArray<T>::CVectorNArray( int inZeilen )
{
	m_Data=new T[inZeilen];
	m_N=inZeilen;

	for(int i=0;i<m_N;i++)
		m_Data[i]=0;								// Inhalte lschen
}


// Initialisieren mit einem Array (Gre mu N sein)
template <class T>
CVectorNArray<T>::CVectorNArray( int inZeilen, const T *inpArray )
{
	assert(inpArray);
	// hier noch bessere Fehlerabfragen machen

	m_Data=m_Data=new T[inZeilen];
	m_N=inZeilen;

	for(int i=0;i<m_N;i++)
		m_Data[i]=inpArray[i];

}


// Copy-Konstruktor
template <class T>
CVectorNArray<T>::CVectorNArray( CVectorNArray<T> &inOrig )
{
	if(inOrig.m_N!=0)
	{
		m_Data=new T[inOrig.m_N];
		m_N=inOrig.m_N;

		for(int i=0;i<m_N;i++)
			m_Data[i]=inOrig.m_Data[i];				// Inhalte kopieren
	}
}


// Destruktor
template <class T>
CVectorNArray<T>::~CVectorNArray()
{
	FreeData();
}



// Zuweisungoperator
template <class T>
CVectorNArray<T> & CVectorNArray<T>::operator=( CVectorNArray<T> &inOrig )
{
	if(inOrig.m_N!=m_N)
	{
		FreeData();										// Daten freigeben

		if(inOrig.m_N!=0)
		{
			m_Data=new T[inOrig.m_N];
			m_N=inOrig.m_N;
		}
	}

	for(int i=0;i<m_N;i++)
		m_Data[i]=inOrig.m_Data[i];						// Inhalte kopieren

	return(*this);
}



// Multipliktion mit einem Skalar
template <class T>
CVectorNArray<T> CVectorNArray<T>::operator*( T inSkalar )
{
	int n=GetN();

	CVectorNArray<T> ret(n);							// Ausgabe ist ein N Vector

	for(int y=0;y<n;y++)
		ret.P(y, P(y)*inSkalar );						// Elementweise multiplizieren

	return(ret);
}



// Multipliktion mit gleichem Typ (Skalarprodukt)
template <class T>
T CVectorNArray<T>::operator*( CVectorNArray<T> &inP )
{
	int n=GetN();

	if(inP.GetN()!=n)throw "Vector: falsche Vectorgre bei Multipliktion";

	T ret=0;											// Ausgabe ist ein Skalar

	for(int y=0;y<n;y++)
		ret+=P(y)*inP.P(y);

	return(ret);
}



// Addition mit gleichem Typ
template <class T>
CVectorNArray<T> CVectorNArray<T>::operator+( CVectorNArray<T> &inP )
{
	int n=GetN();

	if(inP.GetN()!=n)throw "Vector: falsche Vectorgre bei Addition";

	CVectorNArray<T> ret(n);							// Ausgabe ist ein N Vector

	for(int y=0;y<n;y++)
	{
		ret.P(y, P(y)+inP.P(y) );						// Elementweise addieren
	}

	return(ret);
}



// Subtraktion mit gleichem Typ
template <class T>
CVectorNArray<T> CVectorNArray<T>::operator-( CVectorNArray<T> &inP )
{
	int n=GetN();

	if(inP.GetN()!=n)throw "Vactor: falsche Vectorgre bei Subtraktion";

	CVectorNArray<T> ret(n);							// Ausgabe ist ein N Vector

	for(int y=0;y<n;y++)
	{
		ret.P(y, P(y)-inP.P(y) );						// Elementweise addieren
	}

	return(ret);
}


// Gre nachtrglich ndern, Elemente werden umkopiert
template <class T>
void CVectorNArray<T>::Resize( int inZeilen )
{
	if(inZeilen!=m_N)								// Arbeit ntig?
	{
		if(inZeilen!=0)
		{
			T *Data=new T[inZeilen];

			int s=(inZeilen<m_N)?inZeilen:m_N;		// den kleineren ermitteln
			
			for(int i=0;i<s;i++)
				Data[i]=m_Data[i];					// Inhalte kopieren

			for(;i<inZeilen;i++)
				Data[i]=0;							// briges lschen

			delete [] m_Data;
			m_Data=Data;
			m_N=inZeilen;
		}
		else FreeData();
	}
}

// lesender Zugriff (von 0 an gezhlt)
template <class T>
const T	CVectorNArray<T>::P( int inZeile ) const
{
	if(!m_Data)throw "CVectorNArray::P(x) m_Data==0";
	if(((unsigned long)inZeile)>=m_N)throw "CVectorNArray::P(x) inZeile>=m_N oder inZeile<0";

	return(m_Data[inZeile]);
}


// schreibender Zugriff (von 0 an gezhlt)
template <class T>
void CVectorNArray<T>::P( int inZeile, T inValue )
{
	if(!m_Data)throw "CVectorNArray::P(x,y) m_Data==0";
	if(((unsigned long)inZeile)>=m_N)throw "CVectorNArray::P(x,y) inZeile>=m_N oder inZeile<0";

	m_Data[inZeile]=inValue;
}


// Daten freigeben
template <class T>
void CVectorNArray<T>::FreeData( void )
{
	delete [] m_Data;m_Data=0;
	m_N=0;
}


template <class T>
T *CVectorNArray<T>::GetMemoryAccess( void )
{
	return(m_Data);
}



// Anzahl der Zeilen zurckliefern
template <class T>
int CVectorNArray<T>::GetN( void )
{
	return(m_N);
}


template <class T>
void CVectorNArray<T>::Debug( void )	// -> cout
{
	cout.width(5);
	cout.precision(2);

	if(m_Data)
	{
		for(int y=0;y<GetN();y++)
		{
			cout << P(y);
			cout << endl;
		}
	}
	else
	{
		cout << "CVectorNArray is empty";
	}
	cout << endl;
}

//! Euklidische Distanz
template <class T>
T CVectorNArray<T>::Length( void )
{
	T sum=0;

	for(int y=0;y<GetN();y++)
	{
		T d=m_Data[y];

		sum+=d*d;
	}

	return(sqrt(sum));
}



#endif // !defined(AFX_VECTORNARRAY_H__C4EB09D0_A0E1_4D43_9EC5_BC40407F2A7C__INCLUDED_)
