//-------------------------------------------------------------------+
// CRGBColor: Definiert Operationen auf einen RGB-Farbwert.          |
//-------------------------------------------------------------------+
// (c) Copyright 1999 by Tobias Lang                                 |
//     - 9.2.2000 Martin Mittring erweitert
//-------------------------------------------------------------------+ t.l. 30.03.99
#ifndef _ME_COLOR_H__
#define _ME_COLOR_H__

typedef unsigned char UBYTE;
typedef unsigned long ULONG;

class CColor
{
private:

	ULONG m_iRGBA;

public:
	// Konstruktoren
			    CColor			()												{}
				CColor			( ULONG inRGBA )								{ m_iRGBA=inRGBA; }
				CColor			( UBYTE inR, UBYTE inG, UBYTE inB )				{ m_iRGBA=((ULONG)inR<<16)+((ULONG)inG<<8)+(ULONG)inB; }
  				CColor			( UBYTE inA, UBYTE inR, UBYTE inG, UBYTE inB )	{ m_iRGBA=((ULONG)inA<<24)+((ULONG)inR<<16)+((ULONG)inG<<8)+(ULONG)inB; }
	// Copy-Konstruktor
				CColor			( const CColor& a)								{ m_iRGBA=a.m_iRGBA; };

    // Attribut-Zugriff
    ULONG		GetRGBA			() const;					// ist schneller, weil Maskieren entfllt
    ULONG		GetRGB			() const;					// ist wegen Maskieren langsamer

    void		SetRGBA			( const ULONG inColor );	// ist schneller, weil Maskieren entfllt
    void		SetRGB			( const ULONG inColor );	// ist wegen Maskieren langsamer

    // lesender Zugriff auf Komponenten
    UBYTE		GetR			() const;
    UBYTE		GetG			() const;
    UBYTE		GetB			() const;
    UBYTE		GetA			() const;

	// schreibender Zugriff auf Komponenten
    void		SetR			( const UBYTE inColor );
    void		SetG			( const UBYTE inColor );
    void		SetB			( const UBYTE inColor );
    void		SetA			( const UBYTE inColor );

    void		SetGreyRGB		( const UBYTE inColor );	// R,G,B   sind gleich gewichtet
    void		SetGreyRGBA		( const UBYTE inColor );	// R,G,B,A sind gleich gewichtet

    // Operatoren   
				operator ULONG	() const;

    CColor &	operator+=		( const CColor &rhs );
    CColor &	operator-=		( const CColor &rhs );
    CColor &	operator*=		( const UBYTE &rhs );
    CColor &	operator*=		( const CColor &rhs );

    CColor		operator+		( const CColor &rhs );
	CColor		operator-		( const UBYTE &rhs );
	CColor		operator-		( const CColor &rhs );
    CColor		operator*		( const CColor &rhs );
    CColor		operator*		( const UBYTE &rhs );
	CColor		operator>>		( int inShift );

// Zuweisungsoperator
    CColor		operator=		( const CColor &rhs );

	// Hilfsmethoden
	void		MixColor		( CColor colora, CColor colorb, UBYTE part );	// Lerp  dest=colora*(1-part)+colorb*part
};


//----------------------------------------+
// Attribut-Zugriff                       |
//----------------------------------------+
__forceinline ULONG CColor::GetRGBA() const 
{ 
	return(m_iRGBA);
}

__forceinline ULONG CColor::GetRGB() const 
{ 
	return(m_iRGBA & 0x00ffffff); 
}

__forceinline void CColor::SetRGBA(const ULONG inColor) 
{ 
	m_iRGBA=inColor; 
}

__forceinline void CColor::SetRGB(const ULONG inColor)  
{ 
	m_iRGBA= inColor & 0x00ffffff; 
}


// Zugriff auf Farbkomponenten
__forceinline UBYTE CColor::GetA() const 
{ 
	return (UBYTE)(m_iRGBA>>24); 
}

__forceinline UBYTE CColor::GetR() const
{ 
	return (UBYTE)(m_iRGBA>>16); 
}

__forceinline UBYTE CColor::GetG() const
{ 
	return (UBYTE)(m_iRGBA>>8); 
}

__forceinline UBYTE CColor::GetB() const 
{ 
	return (UBYTE)(m_iRGBA); 
}


__forceinline void CColor::SetR(const UBYTE inColor) 
{ 
	m_iRGBA=(m_iRGBA&0xff00ffff) | ((ULONG)inColor<<16); 
}

__forceinline void CColor::SetG(const UBYTE inColor) 
{ 
	m_iRGBA=(m_iRGBA&0xffff00ff) | ((ULONG)inColor<< 8); 
}

__forceinline void CColor::SetB(const UBYTE inColor) 
{ 
	m_iRGBA=(m_iRGBA&0xffffff00) | ((ULONG)inColor); 
}

__forceinline void CColor::SetA(const UBYTE inColor) 
{
	m_iRGBA=(m_iRGBA&0x00ffffff) | ((ULONG)inColor<<24); 
}

__forceinline void CColor::SetGreyRGBA( const UBYTE inColor )
{
    m_iRGBA=((ULONG)inColor<<24)|((ULONG)inColor<<16)|((ULONG)inColor<<8)|(ULONG)inColor;
}

__forceinline void CColor::SetGreyRGB( const UBYTE inColor )
{
    m_iRGBA=(inColor<<16) | (inColor<<8) | (inColor);
}


__forceinline void CColor::MixColor( CColor colora, CColor colorb, UBYTE part )
{
	// ganz ohne MixPixel 114/85

	
/*	// funktionier noch nicht, Optimierung aus 6Mul mach 3
	int p=(short)part;

	char b=(char)(colora)-(char)(colorb);
	char g=(char)(colora>>8)-(char)(colorb>>8);
	char r=(char)(colora>>16)-(char)(colorb>>16);

	m_iRGBA=(int)colora - ((((int)r*part)>>8)<<16) - ((((int)g*part)>>8)<<8) - (((int)b*part)>>8);
*/

	// PSetAA: 135/101
/*	ULONG p=(ULONG)part;
	ULONG q=255-(ULONG)p;

	ULONG b=((colora&    0xff)*q+(colorb&    0xff)*p)>>8;
	ULONG g=((colora&  0xff00)*q+(colorb&  0xff00)*p)>>16;
	ULONG r=((colora&0xff0000)*q+(colorb&0xff0000)*p)>>24;

	m_iRGBA=((r<<16)|(g<<8)|b);
*/
/*
	ULONG p=(ULONG)part;
	ULONG q=255-(ULONG)p;

	ULONG b=((colora&    0xff)*q+(colorb&    0xff)*p)>>8;
	ULONG g=((colora&  0xff00)*q+(colorb&  0xff00)*p)>>16;
	ULONG r=((colora&0xff0000)*q+(colorb&0xff0000)*p)>>24;

	m_iRGBA=((r<<16)|(g<<8)|b);
*/
/*
	// PSetAA: 135/101
	ULONG p=(ULONG)part;
	ULONG q=255-(ULONG)p;

	ULONG g=((colora&  0xff00)*q+(colorb&  0xff00)*p)>>16;
	ULONG br=(((colora&0xff00ff)*q+(colorb&0xff00ff)*p)>>8)&0xff00ff;

	m_iRGBA=(br|(g<<8));
*/

	// PSetAA: 130/97 fastest (favorite)
	ULONG p=(ULONG)part;

	ULONG mask=0xff00;

	ULONG g=((colora&  mask)*(255-p)+  (colorb&  mask)*p   )>>16;

	mask=0xff00ff;
	
	m_iRGBA = ((((colora&mask)*(255-p)+(colorb&mask)*p)>>8) & mask ) | (g<<8);

/*
	// PSetAA: 130/97 fastest
	ULONG p=(ULONG)part;
	ULONG p2=255-p;

	ULONG mask=0xff00ff;

	ULONG br=(((colora&mask)*p2+(colorb&mask)*p)>>8)&mask;

	mask>>=8;
	
	ULONG g=((colora&  mask)*p2+  (colorb&  mask)*p   )>>16;

	m_iRGBA=(br|(g<<8));
*/}


/*
// 120/89 vorher (C++)
// 132/100 Assembler vom Visual bersetzt
__forceinline void CColor::MixColor( CColor colora, CColor colorb, UBYTE part )
{
_asm
{
push esi
push edi
push ebx
push ebp


//	ULONG p=(ULONG)part;

//	ULONG g=((colora&  0xff00)*(255-p)+  (colorb&  0xff00)*p   )>>16;
//	ULONG br=(((colora&0xff00ff)*(255-p)+(colorb&0xff00ff)*p)>>8)&0xff00ff;

//	m_iRGBA=(br|(g<<8));

	mov	dl, part

	mov	eax, colora
	mov	esi, colorb
	and	edx, 255				; 000000ffH   p

	mov	edi, 255				; 000000ffH
	mov	ebx, esi
	mov	ebp, eax
	sub	edi, edx				; 255-p

	and	ebp, 65280				; 0000ff00H color a
	and	ebx, 65280				; 0000ff00H color b
	imul	ebx, edx
	imul	ebp, edi
	add	ebx, ebp

	and	eax, 16711935				; 00ff00ffH  color a
	and	esi, 16711935				; 00ff00ffH  color b
	imul	eax, edi
	imul	esi, edx
	add	eax, esi

	and	bh, 0
	and	eax, -16711681				; ff00ffffH
	or	ebx, eax
	shr	ebx, 8
	mov	DWORD PTR [ecx], ebx


pop	ebp
pop	ebx
pop edi
pop esi
}
}
*/








// Operatoren
/*
inline ULONG CColor::operator+(const CColor &rhs)
{
    ULONG r=( m_iRGBA>>16            )+( rhs.m_iRGBA>>16            );
    ULONG g=((m_iRGBA>> 8)&0x000000ff)+((rhs.m_iRGBA>> 8)&0x000000ff);
    ULONG b=( m_iRGBA     &0x000000ff)+( rhs.m_iRGBA     &0x000000ff);
    if (r>255) r=255;
    if (g>255) g=255;
    if (b>255) b=255;

    return (r<<16)|(g<<8)|b;
}
*/

__forceinline CColor &CColor::operator+=( const CColor &rhs )
{
	(*this)=(*this)+rhs;

	return(*this);
}

__forceinline CColor &CColor::operator*=( const UBYTE &rhs )
{
	(*this)=(*this)*rhs;

	return(*this);
}
__forceinline CColor &CColor::operator*=( const CColor &rhs )
{
	(*this)=(*this)*rhs;

	return(*this);
}

__forceinline CColor &CColor::operator-=( const CColor &rhs )
{
	(*this)=(*this)-rhs;

	return(*this);
}


__forceinline CColor CColor::operator+( const CColor &rhs )
{
    int r=GetR() + rhs.GetR();
    if (r>255) r=255;

    int g=GetG() + rhs.GetG();
    if (g>255) g=255;

    int b=GetB() + rhs.GetB();
    if (b>255) b=255;

    return (r<<16)|(g<<8)|b;

/*    ULONG r=(m_iRGBA&0x00ff0000) + (rhs.m_iRGBA&0x00ff0000);
    r|=(r&0x01000000)-0x00010000;   // Sttigung Rot
    ULONG g=(m_iRGBA&0x0000ff00) + (rhs.m_iRGBA&0x0000ff00);
    g|=(g&0x00010000)-0x00000100;   // Sttigung Grn
    ULONG b=(m_iRGBA&0x000000ff) + (rhs.m_iRGBA&0x000000ff);
    b|=(b&0x00000100)-0x00000001;   // Sttigung Blau

    return (r|g|b);
*/
}

__forceinline CColor CColor::operator-( const CColor &rhs )
{
    int r=GetR()-rhs.GetR(); if (r<0) r=0;

    int g=GetG()-rhs.GetG(); if (g<0) g=0;

    int b=GetB()-rhs.GetB(); if (b<0) b=0;

    return (r<<16)|(g<<8)|b;
}




__forceinline CColor CColor::operator-( const UBYTE &rhs )
{
    int r=GetR()-rhs; if (r<0) r=0;

    int g=GetG()-rhs; if (g<0) g=0;

    int b=GetB()-rhs; if (b<0) b=0;

    return((r<<16)|(g<<8)|b);
}

__forceinline CColor CColor::operator*( const CColor &rhs )
{
    ULONG r=(GetR() * rhs.GetR())>>8;

    ULONG g=(GetG() * rhs.GetG())>>8;

    ULONG b=(GetB() * rhs.GetB())>>8;

    return((r<<16)|(g<<8)|b);
}


__forceinline CColor CColor::operator*( const UBYTE &rhs )
{
	ULONG p=(ULONG)rhs;

	ULONG g=(  (m_iRGBA&  0xff00)*p   )>>16;
	ULONG br=(((m_iRGBA&0xff00ff)*p)>>8)&0xff00ff;

	return(br|(g<<8));
}

__forceinline CColor CColor::operator>>( int inShift )
{
	ULONG r=GetR()>>inShift;
    ULONG g=GetG()>>inShift;
    ULONG b=GetB()>>inShift;

    return((r<<16)|(g<<8)|b);
}


__forceinline CColor::operator ULONG() const
{
    return m_iRGBA;
}


// Zuweisungsoperator
__forceinline CColor CColor::operator=( const CColor &rhs )
{
	return(m_iRGBA=rhs.m_iRGBA);
}


#endif //_ME_COLOR_H__