//----------------------------------------------------------------------------------+
// _CColor: Definiert Operationen auf einen RGB-Farbwert. in T (float oder double)  |
//          (ohne Alpha, ohne Clamping, auch negative Farbwerte)                    |
//                                                                                  |
// CColorF      float,float,float                                                   |  
// CColorD      double,double,double                                                |
//----------------------------------------------------------------------------------+

#ifndef _ME_COLORF_H__
#define _ME_COLORF_H__

class CColor;

template <class T>
class _CColorRGB
{
private:

	T					p[3];

public:
	// Konstruktoren
					    _CColorRGB		() {}						// aus Effizienzgrnden keine Initialisierung
						_CColorRGB		( CColor inRGBA );			
						_CColorRGB		( T inR, T inG, T inB );

	// Copy-Konstruktor
						_CColorRGB		( const _CColorRGB<T>& a );
		
	// Attribut-Zugriff
    ULONG				GetRGB			() const;					// ist wegen Maskieren langsamer

//    void				SetRGBA			( const ULONG inColor );	// ist schneller, weil Maskieren entfllt
//    void				SetRGB			( const ULONG inColor );	// ist wegen Maskieren langsamer

    // lesender Zugriff auf Komponenten
    T					GetR			() const;
    T					GetG			() const;
    T					GetB			() const;


	// schreibender Zugriff auf Komponenten
/*    void				SetR			( const T inColor );
    void				SetG			( const T inColor );
    void				SetB			( const T inColor );
    void				SetA			( const T inColor );
*/
    void				SetGrey			( const T inValue );	// R,G,B   sind gleich gewichtet

    // Operatoren   
//				operator ULONG	() const;

    _CColorRGB<T> &		operator+=		( const _CColorRGB<T> &rhs );
    _CColorRGB<T> &		operator-=		( const _CColorRGB<T> &rhs );
    _CColorRGB<T> &		operator*=		( const _CColorRGB<T> &rhs );
    _CColorRGB<T> &		operator*=		( const T inValue );

    _CColorRGB<T>		operator+		( const _CColorRGB<T> &rhs );
	_CColorRGB<T>		operator-		( const _CColorRGB<T> &rhs );
    _CColorRGB<T>		operator*		( const _CColorRGB<T> &rhs );
    _CColorRGB<T>		operator*		( const T inValue );

	void				Clamp			( void );		// auf max 1 begrenzen
	float				Distance		( _CColorRGB<T> b );	// 0..1

// Zuweisungsoperator
    _CColorRGB<T>		operator=		( const _CColorRGB<T> &rhs );


	// (lineare Interpolation)
};
template <class T> _CColorRGB<T> Lerp( _CColorRGB<T> colora, _CColorRGB<T> colorb, T part );	// Lerp  dest=colora*(1-part)+colorb*part




// ******************************************************************************************
// ******************************************************************************************
// ******************************************************************************************
// ******************************************************************************************
// ******************************************************************************************

// Copy-Konstruktor
template <class T>
_CColorRGB<T>::_CColorRGB( const _CColorRGB<T>& a )
{
	p[0]=a.p[0];
	p[1]=a.p[1];
	p[2]=a.p[2];
}


// Konstruktor
template <class T>
_CColorRGB<T>::_CColorRGB( CColor inRGB )
{
	p[0]=inRGB.GetR()*(1.0/255.0f);
	p[1]=inRGB.GetG()*(1.0/255.0f);
	p[2]=inRGB.GetB()*(1.0/255.0f);
}

// Konstruktor
template <class T>
_CColorRGB<T>::_CColorRGB( T inR, T inG, T inB )
{
	p[0]=inR;p[1]=inG;p[2]=inB;
}


template <class T>
T _CColorRGB<T>::GetR( void ) const 
{
	return(p[0]);
}

template <class T>
T _CColorRGB<T>::GetG( void ) const 
{
	return(p[1]);
}

template <class T>
T _CColorRGB<T>::GetB( void ) const 
{
	return(p[2]);
}



template <class T>
void _CColorRGB<T>::SetGrey( const T inValue )
{
	p[0]=p[1]=p[2]=inValue;
}


// Lerp (lineare Interpolation) oder MixColor (part=[0..1])
template <class T>
void Lerp( _CColorRGB<T> colora, _CColorRGB<T> colorb, T part )
{
	assert(part>=0.0f);
	assert(part<=1.0f);

	return( colora*(1.0f-part) + colorb*part );
}



// ohne Clamping
template <class T>
ULONG _CColorRGB<T>::GetRGB( void ) const
{
	UBYTE R=(UBYTE)(p[0]*255.0f);
	UBYTE G=(UBYTE)(p[1]*255.0f);
	UBYTE B=(UBYTE)(p[2]*255.0f);

	return(
			(((ULONG)R)<<16) |
			(((ULONG)G)<<8) |
			((ULONG)B)
		  );
}





// wird auf operator+ abgebildet
template <class T> _CColorRGB<T> &_CColorRGB<T>::operator+=( const _CColorRGB<T> &rhs )
	{ (*this)=(*this)+rhs;return(*this); }

// wird auf operator- abgebildet
template <class T> _CColorRGB<T> &_CColorRGB<T>::operator-=( const _CColorRGB<T> &rhs )
	{ (*this)=(*this)-rhs;return(*this); }

// wird auf operator* abgebildet
template <class T> _CColorRGB<T> &_CColorRGB<T>::operator*=( const _CColorRGB<T> &rhs )
	{ (*this)=(*this)*rhs;return(*this); }

// wird auf operator* abgebildet
template <class T> _CColorRGB<T> &_CColorRGB<T>::operator*=( const T inValue )
	{ (*this)=(*this)*inValue;return(*this); }


// ohne Clamping
template <class T>
_CColorRGB<T> _CColorRGB<T>::operator+( const _CColorRGB<T> &rhs )
{
	return( _CColorRGB<T>(p[0]+rhs.p[0],p[1]+rhs.p[1],p[2]+rhs.p[2]) );
}

// ohne Clamping
template <class T>
_CColorRGB<T> _CColorRGB<T>::operator-( const _CColorRGB<T> &rhs )
{
	return( _CColorRGB<T>(p[0]-rhs.p[0],p[1]+-rhs.p[1],p[2]-rhs.p[2]) );
}

template <class T>
_CColorRGB<T> _CColorRGB<T>::operator*( const _CColorRGB<T> &rhs )
{
	return( _CColorRGB<T>(p[0]*rhs.p[0],p[1]*rhs.p[1],p[2]*rhs.p[2]) );
}

template <class T>
_CColorRGB<T> _CColorRGB<T>::operator*( const T inValue )
{
	return( _CColorRGB<T>(p[0]*inValue,p[1]*inValue,p[2]*inValue) );
}


// Zuweisungsoperator
template <class T>
_CColorRGB<T> _CColorRGB<T>::operator=( const _CColorRGB<T> &rhs )
{
	p[0]=rhs.p[0];
	p[1]=rhs.p[1];
	p[2]=rhs.p[2];

	return(*this);
}

template <class T>
void _CColorRGB<T>::Clamp( void )
{
	if(p[0]>1.0f)p[0]=1.0f;
	if(p[1]>1.0f)p[1]=1.0f;
	if(p[2]>1.0f)p[2]=1.0f;
}


// nicht effizient
template <class T>
float _CColorRGB<T>::Distance( _CColorRGB<T> b )
{
	_CColorRGB<T> a=*this;

	a.Clamp();b.Clamp();

	return(max(max(fabs(a.p[0]-b.p[0]),fabs(a.p[1]-b.p[1])),fabs(a.p[2]-b.p[2])));
}


typedef _CColorRGB<float> CColorF;
typedef _CColorRGB<double> CColorD;


#endif //_ME_COLOR_H__