#include <assert.h>				// assert
#include "Color.h"				// CColor
#include <math.h>				// sqrt

#ifndef _DGRAPHLIB_H
#define _DGRAPHLIB_H


class CPoint2I
{
public:
	int x,y;

	CPoint2I( )	// uninit
	{
	}

	CPoint2I( int a, int b )
	{
		x=a;y=b;
	}

	CPoint2I &operator+=( const CPoint2I &a )
	{
		x+=a.x;y+=a.y;
		return(*this);
	}

	CPoint2I &operator-=( const CPoint2I &a )
	{
		x-=a.x;y-=a.y;
		return(*this);
	}

	CPoint2I operator-( const CPoint2I &a ) const
	{
		return(CPoint2I(x-a.x,y-a.y));
	}

	CPoint2I operator+( const CPoint2I &a ) const
	{
		return(CPoint2I(x+a.x,y+a.y));
	}

	CPoint2I operator/( const int a ) const
	{
		return(CPoint2I(x/a,y/a));
	}

	CPoint2I operator*( const int a ) const
	{
		return(CPoint2I(x*a,y*a));
	}

	CPoint2I operator>>( const int a ) const
	{
		return(CPoint2I(x>>a,y>>a));
	}

	CPoint2I operator<<( const int a ) const
	{
		return(CPoint2I(x<<a,y<<a));
	}

#ifdef AFX_DATA				// MFC
	CPoint2I( CPoint a )
	{
		x=a.x;y=a.y;
	}

	CPoint2I &operator=( CPoint a )
	{
		x=a.x;y=a.y;
		return(*this);
	}
#endif
};


class CPoint2F
{
public:
	float x,y;

	CPoint2F( )	// uninit
	{
	}

	// Konstruktor
	CPoint2F( float a, float b )
	{
		x=a;y=b;
	}

	CPoint2F( const CPoint2I &inP )
	{
		x=(float)inP.x;y=(float)inP.y;
	}

	void Set( float a, float b )
	{
		x=a;y=b;
	}

	CPoint2F &operator+=( const CPoint2F &a )
	{
		x+=a.x;y+=a.y;
		return(*this);
	}

	CPoint2F &operator-=( const CPoint2F &a )
	{
		x-=a.x;y-=a.y;
		return(*this);
	}

	CPoint2F operator-( const CPoint2F &a ) const
	{
		return(CPoint2F(x-a.x,y-a.y));
	}

	CPoint2F operator+( const CPoint2F &a ) const
	{
		return(CPoint2F(x+a.x,y+a.y));
	}

	CPoint2F operator/( const float a ) const
	{
		return(CPoint2F(x/a,y/a));
	}

	CPoint2F operator*( const float a ) const
	{
		return(CPoint2F(x*a,y*a));
	}

	float Length( void )			// Abstand vom Nullpunkt (oder als Vektor interpretiert)
	{
		return((float)sqrt(x*x+y*y));
	}

	// Interpolate vertex information  (needed for clipping)
	void vLerpX( float alpha, CPoint2F *Va, CPoint2F *Vb )		// alles auer X (clipping)
	{
		if(alpha<=0){ alpha=-alpha;CPoint2F *s=Va; Va=Vb;Vb=s; }

		y=Va->y*alpha+Vb->y*(1.0f-alpha);
	}

	// Interpolate vertex information  (needed for clipping)
	void vLerpY( float alpha, CPoint2F *Va, CPoint2F *Vb )		// alles auer Y (clipping)
	{
		if(alpha<=0){ alpha=-alpha;CPoint2F *s=Va; Va=Vb;Vb=s; }

		x=Va->x*alpha+Vb->x*(1.0f-alpha);
	}

};


// ****************************************************************************



class CRect2I
{
public:
	int	top,left,right,bottom;

	CRect2I()
	{
	}

	CRect2I( const CPoint2I min, const CPoint2I max )
	{
		left=min.x;right=max.x;
		top=min.y;bottom=max.y;
	}

	CRect2I( const int _left, const int _top, const int _right, const int _bottom )
	{
		left=_left;right=_right;
		top=_top;bottom=_bottom;
	}

	bool PtInRect( CPoint2I xy ) const
	{
		return(xy.x>=left && xy.y>=top && xy.x<right && xy.y<bottom);
	}

	CPoint2I GetMidPoint( void ) const
	{
		return(CPoint2I((left+right)/2,(top+bottom)/2));
	}

	int Height( void ) const
	{
		return(bottom-top);
	}

	int Width( void ) const
	{
		return(right-left);
	}

	void InflateRect( int r )	// make the rectangle bigger, negative values make smaller
	{
		left-=r;top-=r;
		right+=r;bottom+=r;
	}

	bool IsNormalized( void ) const
	{
		return(Width()>=0 && Height()>=0);
	}

	void NormalizeRect( void )
	{
		if(left>right){ int s=left; left=right;right=s; }
		if(top>bottom){ int s=top; top=bottom;bottom=s; }
	}

	bool IntersectRect( CRect2I const &a, CRect2I const &b )
	{
		assert(a.IsNormalized());
		assert(b.IsNormalized());

		left=	a.left	>b.left		? a.left	: b.left;
		top=	a.top	>b.top		? a.top		: b.top;
		right=	a.right	>b.right	? b.right	: a.right;
		bottom= a.bottom>b.bottom	? b.bottom	: a.bottom;

		return(Height()>0 && Width()>0);
	}

	void UnionRect( CRect2I const &a, CRect2I const &b )
	{
		assert(a.IsNormalized());
		assert(b.IsNormalized());

		left=	a.left	>b.left		? b.left	: a.left;
		top=	a.top	>b.top		? b.top		: a.top;
		right=	a.right	>b.right	? a.right	: b.right;
		bottom= a.bottom>b.bottom	? a.bottom	: b.bottom;
	}

	void UnionPoint( CRect2I const &a, CPoint2I const &b )
	{
		left=	a.left	>b.x		? b.x		: a.left;
		top=	a.top	>b.y		? b.y		: a.top;
		right=	a.right	>b.x		? a.right	: b.x;
		bottom= a.bottom>b.y		? a.bottom	: b.y;
	}

	CRect2I &operator-=( CPoint2I &c )
	{
		left=left-c.x;		right=right-c.x;	
		top=top-c.y;		bottom=bottom-c.y;

		return(*this);
	}

	CRect2I &operator+=( CPoint2I &c )
	{
		left=left+c.x;		right=right+c.x;	
		top=top+c.y;		bottom=bottom+c.y;

		return(*this);
	}

	CRect2I operator-( CPoint2I &c ) const
	{
		CRect2I ret;

		ret.left=left-c.x;		ret.right=right-c.x;	
		ret.top=top-c.y;		ret.bottom=bottom-c.y;

		return(ret);
	}

	CRect2I operator+( CPoint2I &c ) const
	{
		CRect2I ret;

		ret.left=left+c.x;		ret.right=right+c.x;	
		ret.top=top+c.y;		ret.bottom=bottom+c.y;

		return(ret);
	}

	CRect2I operator>>( int i ) const
	{
		CRect2I ret;

		ret.left=left>>i;		ret.right=right>>i;	
		ret.top=top>>i;			ret.bottom=bottom>>i;
	
		return(ret);
	}


#ifdef AFX_DATA				// MFC
	CRect2I &operator=( const CRect &a )
	{
		left=a.left;right=a.right;top=a.top;bottom=a.bottom;
		return(*this);
	}

	CRect2I( const CRect &a )
	{
		left=a.left;right=a.right;top=a.top;bottom=a.bottom;
	}
#endif
};


//class CPolVertex;


#endif
