#ifndef _GRAPHLIB_H
#define _GRAPHLIB_H

#include "..\..\shared\Datatypes.h"				//
#include "..\..\shared\Interfaces.h"			// IUnknown	
#include "DGraphLib.h"							//
#include "ColorF.h"								// CColorF

class I2DMemoryDCSurface;
class CMC;



// Elemente mit diesem Typ werden aus einem Element mit dem Typ I2DMemoryCSurface erstellt
// Alle Operationen (auer wenn explizit das Gegenteil behauptet wird) clippen gegen die
// im CMC definierten Rechtecksgrenzen 
// (hnlich einem Windows DC )
class I2DMemoryDC :public IUnknown 
{ 
public:
	// Interface Methods
	virtual void			SetData				( I2DMemoryDCSurface *_surf )=0;	// fr Init
	virtual void			SetClip				( CRect2I rec )=0;					// fr Init
	virtual const			CRect2I& GetClip	( void ) const =0;
	
	// Fllt ein Rechteck mit einer Farbe, sehr Hardwarenah
	// Untergrund wird einfach berschrieben
	// rechter und unterer Rand gehrt nicht zum Bereich
	// mit Clipping
	virtual void			FillRect			( CRect2I rec,
												  CColor color,
												  UBYTE density=255 )=0;

	// mit Clipping
	virtual void			PSet				( int _xa, int _ya, 
												  CColor color )=0;

	// mit Clipping (auerhalb ist Farbe 0xffffffff)
	virtual CColor			GetPixel			( int _xa, int _ya ) const =0;

	// mit Clipping
	virtual void			MixPixel			( int _xa, int _ya,
												  CColor color,
												  UBYTE part )=0;
	
	virtual bool			BlitFromSurface		( I2DMemoryDCSurface *_src, 
												  CRect2I _destRect,
												  int _srcx, int _srcy )=0;			// kann fehlschlagen, wenn Surface kein 32Bit Zugriff zulsst
	
	virtual bool			StretchBlitFromSurface( I2DMemoryDCSurface *_src,		// kann fehlschlagen, wenn Surface kein 32Bit Zugriff zulsst 
												  float _srcx, float _srcy,
												  float _srcw, float _srch,			// Flippen kann man durch negieren dieser Parameter
												  float _dstx, float _dsty,
												  float _dstw, float _dsth,			// sollen immer positiv sein
												  CColor mask=0xffffffff,			// mask=0xffffffff heit ohne Maske
												  ULONG flags=0 )=0;		

	// weniger Hardwarenah

	virtual void			PSetAA				( float _xa, float _ya,								  
												  CColor color,						// Mit AntiAliasing 
												  UBYTE density)=0;					// (density 0..255)
	virtual void			Rect				( CRect2I rec, int border,			// ungeflltes Rechteck, ohne Antialising
												  CColor color, 
												  DWORD flags=0 )=0;				// Bit0: ShadeBorder
	virtual void			CircleAA			( float _xa, float _ya,				// Mit Antialiasing
												  float rx, float ry,
												  CColor color,
												  UBYTE density )=0;				// (density 0..255)
	// Subpixelgenau, ohne Gammakorrektur (linear)
	virtual void			LineAA				( float _xa, float _ya,				// Mit Antialiasing 
												  float _xe, float _ye,
												  CColor color, 
												  UBYTE density )=0;				// (density 0..255)
	virtual void			PolyAA_noclip		( CPoint2F poly[], 
												  ULONG VertexCount,
												  CColor color )=0;
	virtual void			PolyAA				( CPoint2F poly[], 
												  ULONG VertexCount,
												  CColor color )=0;
	virtual void			DiscAA				( float _xa, float _ya,
												  float rx, float ry,
												  CColor color )=0;
	virtual bool			TextAA				( float _xa, float _ya, 
												  const char *txt, HFONT hfnt,
												  float magx, float magy,	  
												  CColor color )=0;					// Mu Truetype-font sein, return: true bei Erfolg
	virtual void			TriAAGouraud		( CPoint2F poly[3], 
												  CColorF color[3] )=0;
	virtual void			TriAASpectrum		( CPoint2F poly[3], 
												  float value[3] )=0; // 1 Wert wird interpoliert value=[0..1]

	// Maximum_Alpha Antialiased
	virtual void			Alpha_LineAA		( float _xa, float _ya,				// Mit Antialiasing 
												  float _xe, float _ye,
												  CColor color, 
												  UBYTE density )=0;				// (density 0..255)

	virtual HDC				GetHDC				( void )=0;							// fr Windows-Befehle, mit ReleaseHDC wieder freigeben
	virtual void			ReleaseHDC			( HDC s )=0;						// fr Windows-Befehle, vorher mu GetHDC aufgerufen worden sein 
};
// {58C7D746-B0B8-46f3-B8A0-9789899ED598}
DEFINE_GUID(IID_I2DMemoryDC, 0x58c7d746, 0xb0b8, 0x46f3, 0xb8, 0xa0, 0x97, 0x89, 0x89, 0x9e, 0xd5, 0x98);


// verwaltet eine 32Bit-Surface (00RRGGBB oder AARRGGBB)
class I2DMemoryDCSurface :public IUnknown 
{ 
public:
	// Interface Methods
	virtual bool			Create				( LPDIRECTDRAW7 p, 
												  DWORD br, DWORD ho, 
												  I2DMemoryDC **cmc )=0;	// erzeugt die Surface
	virtual bool			CreateFromFile		( LPDIRECTDRAW7 p,
												  char *inPathName,
												  I2DMemoryDC **cmc )=0;	// erzeugt die Surface, derzeit nur TGA, ber Plugins spter erweiterbar
	virtual bool			SaveToFile			( char *inPathName,
												  bool inWithAlpha )=0;		// speichtert das Bild, derzeit nur TGA
	virtual CColor *		GetMemPtr			( void )=0;					// 0=failed, fr Zeichenbefehle (macht new, sollte selten verwendet werden), kein Freigen ntig
	virtual HDC				GetHDC				( void )=0;					// 0=failed, ReleaseHDC nicht vegessen, bis Release kein Speicherzugriff mglich
	virtual	void			ReleaseHDC			( HDC dc )=0;				// fr GetHDC
	virtual void			Free				( void )=0;					// Gibt die Resource wieder frei
	virtual void			GetSize				( DWORD &Width, 
												  DWORD &Height )=0;		// liefert Breite und Hher der Surface zurck
	virtual void			Blit				( HDC cdc, CPoint2I p )=0;	// Kopiert die Surface in den HDC
	virtual void			Blit				( HDC inCdc, 
												  CRect2I inSrcRec, 
												  CPoint2I inP )=0;			// Kopiert die Surface in den HDC
};



// {F4FE25BE-C961-4984-9497-49F1879FFC35}
DEFINE_GUID(IID_I2DMemoryDCSurface, 0xf4fe25be, 0xc961, 0x4984, 0x94, 0x97, 0x49, 0xf1, 0x87, 0x9f, 0xfc, 0x35);

#endif
