// DocObject.cpp: Implementierung der Klasse CDocObject.
//
//////////////////////////////////////////////////////////////////////

//#include "stdafx.h"
#include "DocObject.h"
#include "..\..\shared\CInstanceFactory.h"



#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern class IEditorApplication	*g_App;
#define ERR if(g_App->GetErrorHandler())(g_App->GetErrorHandler())		// fr Fehlerausgabe

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDocObject::CDocObject() : CComponentBase()
{
	m_rect.left=0;
	m_rect.right=0;
	m_rect.top=0;
	m_rect.bottom=0;
}

CDocObject::~CDocObject()
{

}

// Interface I2DDrawObject

CRect2I	CDocObject::GetRect( void )
{
	return(m_rect);
}

void CDocObject::SetRect( const CRect2I &r )
{
	m_rect=r;
}


void CDocObject::ReleaseSubObjects( void )					// Unterobjekte freigeben
{
	ISceneGraphObject *p=GetIn();

	for(;p!=0;p=p->GetNext())
		p->ReleaseSubObjects();

	m_hIn=0;
}

bool CDocObject::InsertObject( ISceneGraphObject *p )		// Unterobjekt einfgen
{
	p->SetParent(this);										// Kontainerobjekt

	p->SetNext(m_hIn);m_hIn=p;								// 1fach verkettet

	return(true);
}





CPoint2I CDocObject::GetWorldPosition( void )				// arbeitet rekursiv
{
	ISceneGraphObject *m=GetParent();
	
	if(m==0)return(CPoint2I(m_rect.left,m_rect.top));
	else
	{
		I2DDraw *p;
		m->QueryInterface(IID_I2DDraw,(LPVOID*)&p);
		assert(p);											// mu gehen

		CPoint2I pnt = p->GetWorldPosition()+CPoint2I(m_rect.left,m_rect.top);

		p->Release();

		return pnt;
	}
}









bool CDocObject::IsObjectAt( CPoint2I xy )
{
	return(false);	// Objekt ist nicht anklickbar
}


// Interface I2DDrawObject
void CDocObject::Draw( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify ) 
{
	ISceneGraphObject *p=GetIn();

	// Connectors zeichnen
	for(;p!=0;p=p->GetNext())
	{
		CHandle<IConnector> c=QUERYINTERFACE(IConnector,p);
		if(!c)continue;

		CHandle<I2DDraw> m=QUERYINTERFACE(I2DDraw,p);
		if(m)m->Draw(pMC,pos,inMagnify);
		else
		{
			CHandle<I2DDraw> n=QUERYINTERFACE(I2DDraw,p);
			assert(n);

			n->Draw(pMC,pos,inMagnify);	
		}
	}

	p=GetIn();

	// non Connectors zeichnen
	for(;p!=0;p=p->GetNext())
	{
		CHandle<IConnector> c=QUERYINTERFACE(IConnector,p);
		if(c)continue;

		CHandle<I2DDraw> m=QUERYINTERFACE(I2DDraw,p);

		if(m)m->Draw(pMC,pos,inMagnify);
		else
		{
			CHandle<I2DDraw> n=QUERYINTERFACE(I2DDraw,p);
			assert(n);

			n->Draw(pMC,pos,inMagnify);
		}
	}
}



void CDocObject::MarkedDraw( I2DMemoryDC *pMC, int r, CPoint2I pos, int inMagnify )
{
}

ULONG CDocObject::IsAnchorPointAt( int r, CPoint2I xy )	// nur fr markierte Objekte, 0, wenn dort kein Punkt, sonst ist dort ein Angriffspunkt (Skalieren, ...)
{
	return(0);
}

void CDocObject::MoveObject( CPoint2I rel, ULONG &AnchorPoint )
{
	m_rect.left+=rel.x;
	m_rect.right+=rel.x;
	m_rect.top+=rel.y;
	m_rect.bottom+=rel.y;
}

void CDocObject::ScaleObject( CPoint2I rel )
{
	m_rect.left-=rel.x;
	m_rect.right+=rel.x;
	m_rect.top-=rel.y;
	m_rect.bottom+=rel.y;

	int b=m_rect.right-m_rect.left;
	int h=m_rect.bottom-m_rect.top;

	if(b<6)
	{
		int mid2=(m_rect.right+m_rect.left);

		m_rect.left=(mid2-6)/2;
		m_rect.right=(mid2+6)/2;
	}

	if(h<6)
	{
		int mid2=(m_rect.bottom+m_rect.top);

		m_rect.top=(mid2-6)/2;
		m_rect.bottom=(mid2+6)/2;
	}
}






I2DDraw *CDocObject::GetTopObjectAt( CPoint2I xy )
{
	ISceneGraphObject *p=GetIn();
	I2DDraw *top=0;

	for(;p!=0;p=p->GetNext())
	{
		CHandle<I2DDraw> m=QUERYINTERFACE(I2DDraw,p);
		if(m)
		{
			I2DDraw *here=m->GetTopObjectAt(xy);

			if(here)top=here;
		}
	}

	return(top);
}


I2DDraw *CDocObject::GetTopAnchorPointAt( int r, CPoint2I xy, ULONG &anchorid )	// id von 1 bis ...
{
	ISceneGraphObject *p=GetIn();
	I2DDraw *top=0;

	for(;p!=0;p=p->GetNext())
	{
		I2DDraw *m;
		p->QueryInterface(IID_I2DDraw,(LPVOID*)&m);

		I2DDraw *here=m->GetTopAnchorPointAt(r,xy,anchorid);

		if(here)top=here;

		m->Release();
	}

	return(top);
}


IPopUpMenu *CDocObject::MakeCreateMenu( IPopUpMenu *k, CPoint2I xy )
{
// Alle Objekte in den Factories auslesen
	IPopUpMenu *p=k->Create();

	xy-=CPoint2I(m_rect.left,m_rect.top);

	{
		CInstanceFactory::iterator i(*(g_App->GetFactory()));
		ULONG id=0;

		for(;i!=g_App->GetFactory()->end();i++)
		{
			CInstanceFactory *fac	= g_App->GetFactory();

			IUnknown *ip = fac->CreateInstance(*(*i).m_pCLSID);

			if(!ip)
				continue;

			IUnknown *tp;

			if(FAILED(ip->QueryInterface(IID_I2DDraw,(LPVOID*)&tp)))
			{
				ip->Release();
				continue;
			}
			else
				tp->Release();

			if(SUCCEEDED(ip->QueryInterface(IID_IConnector,(LPVOID*)&tp)))
			{
				ip->Release();
				tp->Release();
				continue;
			}

			ip->Release();

//			if(!i.HasInterface(IID_I2DDraw))continue;		// nur eI2DDraw untersttzende Objekte
//			if(i.HasInterface(IID_IConnector))continue;	// keine Connectoren
//			if((*i).name=="CDORoot")continue;

			char idstr[80];

			sprintf(idstr,"Create(%s,%p,%p,%d,%d)",(*i).m_szProgID_VI,(ISceneGraphObject *)this,i.GetFactoryEntry(),xy.x,xy.y);

			p->AppendItem((char *)(*i).m_szProgID_VI,idstr);
		}
	}
	return(p);
}






void CDocObject::ReleaseObject( void )
{
	if(GetParent())
		GetParent()->OnChildReleased(this);

	ReleaseSubObjects();

	if(GetParent())
	{
		ISceneGraphObject *p=GetParent()->GetIn(),*old=0;

		while(p)
		{
			if(p==this)
			{
				if(old==0){ GetParent()->SetIn(p->GetNext());return; }
					else { old->SetNext(p->GetNext());return; }
			}

			old=p;
			p=p->GetNext();
		}

		assert(0);	// interner Fehler 
	}
}


CHandle<ISceneGraphObject> CDocObject::GetTopmostParent( void )
{
	CHandle<ISceneGraphObject> akt=this,old;

	while(akt)
	{
		old=akt;akt=akt->GetParent();
	}

	return(old);
}


void CDocObject::OnChildReleased( ISceneGraphObject *p )
{
	assert(GetParent());

	GetParent()->OnChildReleased(this);
}


void CDocObject::SerializeRecursive( CSerializeArchivePointer &ar )
{
	// erst das Objekt (somit wird das erste Objekt das RootObject)

	if(ar.BeforeSerialize(this))	// immer vor jedem Serialize()
	{
		Serialize(ar);
		ar.AfterSerialize(this);	// immer nach jedem Serialize()
	}

	// dann alle in ihm enthaltenden Objekte

	CHandle<ISceneGraphObject> n=GetIn();

	while(n)			
	{
		ISerialize *ret;

		n->QueryInterface(IID_ISerialize,(void **)&ret);
		assert(ret);

		ret->SerializeRecursive(ar);		// enthaltenen Baum abgehen

		n=n->GetNext();
	}
}




void write_long( FILE *out, ULONG data );

// hier wird nur das zu serialisierende Objekt betrachtet,
// nicht Objekte auf die das Objekt werweist oder beinhaltet.
void CDocObject::Serialize( CSerializeArchivePointer &ar )
{
	// if(ar.GetMode==CSAM_saveADEF) ....

	AR_HANDLE(ISceneGraphObject,m_hIn,"in");
	AR_HANDLE(ISceneGraphObject,m_hNext,"next");
//	ar.DHandle(IID_ISceneGraphObject,m_hIn,"in");			// enthaltene Objekte
//	ar.DHandle(IID_ISceneGraphObject,m_hNext,"next");		// nachfolgende Objekte (einfach verkettet)
//	AR_HANDLE(ISceneGraphObject,m_hParent,"parent");	// Redundant, mu nicht serialisiert werden

	ar.Dint(m_rect.left,"left");
	ar.Dint(m_rect.top,"top");
	ar.Dint(m_rect.bottom,"bottom");
	ar.Dint(m_rect.right,"right");
}

void CDocObject::OnAfterLoad( void )
{
	if(m_hIn)		// wenn untergeordnete Objekte existieren, mssen deren Parent Zeiger gesetzt werden
	{
		ISceneGraphObject *p=GetIn();

		for(;p!=0;p=p->GetNext())
		{
			p->SetParent(this);
		}
	}
}


bool CDocObject::InsertObjectAtEnd( ISceneGraphObject *p )			// Unterobjekt einfgen
{
	p->SetParent(this);						// Kontainerobjekt

	ISceneGraphObject *last=0,*l=GetIn();

	if(!l)
	{
		p->SetNext(m_hIn);m_hIn=p;		// 1fach verkettet
		return(true);
	}

	for(;l!=0;l=l->GetNext())
		last=l;

	last->SetNext(p);				// 1fach verkettet

	return(true);
}

