// CClassFactoryBase.cpp: implementation of the CClassFactoryBase class.
//
//////////////////////////////////////////////////////////////////////

//#ifdef _AFXDLL				// MFC
//#include "stdafx.h"
//#endif


#include "CClassFactoryBase.h"
#include <stdio.h>

//#include "..\shared\Datatypes.h"
//#include "..\shared\Interfaces.h"

LONG	CClassFactoryBase::m_cServerLocks	= 0;
HMODULE	CClassFactoryBase::m_hModule		= 0L;
const int CLSID_STRING_SIZE = 39 ;

extern CClassFactoryData	g_ClassFactoryData[];


CClassFactoryBase::CClassFactoryBase(const CClassFactoryData *pFactoryData) : m_RefCount(1)
{
	m_pFactoryData	= pFactoryData;
}

HRESULT __stdcall CClassFactoryBase::QueryInterface(REFIID iid,LPVOID *ppv)
{
	IUnknown	*pUnknown;

	if( (iid == IID_IUnknown) || (iid == IID_IClassFactory))
	{
		pUnknown	= this;
	}
	else
	{
		*ppv	= 0L;
		return	E_NOINTERFACE;
	}

	pUnknown->AddRef();
	*ppv	= pUnknown;

	return S_OK;
}

ULONG __stdcall CClassFactoryBase::AddRef()
{
	return ::InterlockedIncrement(&m_RefCount);
}

ULONG __stdcall CClassFactoryBase::Release()
{
	if(! ::InterlockedDecrement(&m_RefCount))
	{
		delete this;
		return 0;
	}

	return m_RefCount;
}

// diese Methode wird nur fr COM-Kompatibilitt benutzt ????????
HRESULT __stdcall CClassFactoryBase::CreateInstance(IUnknown *pUnknownOuter, const IID &iid,LPVOID *ppv)
{
	pUnknownOuter=pUnknownOuter;
//	if( (pUnknownOuter != 0L) && (iid != IID_IUnknown) )
//		return CLASS_E_NOAGGREGATION;


	CComponentBase	*pNewComponent;			// Create the component

	HRESULT	hr	= m_pFactoryData->CreateComponent(/*pUnknownOuter*/0, &pNewComponent);

	if(FAILED(hr))
		return hr;


	hr	= pNewComponent->Init();			// Initialize the component

	if(FAILED(hr))
	{
		pNewComponent->Release_ND();
		return hr;
	}

	hr	= pNewComponent->QueryInterface_ND(iid,ppv);
	pNewComponent->Release_ND();

	return hr;
}

HRESULT __stdcall CClassFactoryBase::LockServer(BOOL bLock)
{
	if(bLock)
		::InterlockedIncrement(&m_cServerLocks);
	else
		::InterlockedDecrement(&m_cServerLocks);

	return S_OK;
}

HRESULT CClassFactoryBase::GetClassObject(const CLSID &clsID,const IID &iid,LPVOID *ppv)
{
	if( (iid != IID_IUnknown) && (iid != IID_IClassFactory))
		return E_NOINTERFACE;

	for(int i = 0;; i++)
	{
		const CClassFactoryData	*pData	= &g_ClassFactoryData[i];

		if(pData->m_pCLSID==0)break;		// Endemarker

		if(pData->IsClassID(clsID))
		{
			*ppv	= (IUnknown *) new CClassFactoryBase(pData);

			if(!*ppv)
				return E_OUTOFMEMORY;
			
			return NOERROR;
		}
	}

	return CLASS_E_CLASSNOTAVAILABLE;
}

HRESULT CClassFactoryBase::Register()
{
	for(int i = 0;; i++)
	{
		const CClassFactoryData	*pData	= &g_ClassFactoryData[i];

		if(pData->m_pCLSID==0)break;		// Endemarker

		char	Buf[_MAX_PATH];
		sprintf(Buf,"%s.%d",pData->m_szProgID_VI,*pData->m_pVersion);

		CClassFactoryBase::RegisterServer(m_hModule,*(pData->m_pCLSID),pData->m_szProgID_VI,pData->m_szProgID_VI,Buf);
	}

	return S_OK;
}

HRESULT CClassFactoryBase::UnRegister()
{
	for(int i = 0;; i++)
	{
		const CClassFactoryData	*pData	= &g_ClassFactoryData[i];

		if(pData->m_pCLSID==0)break;		// Endemarker

		char	Buf[_MAX_PATH];
		sprintf(Buf,"%s.%d",pData->m_szProgID_VI,*pData->m_pVersion);

		CClassFactoryBase::UnregisterServer(*(pData->m_pCLSID),pData->m_szProgID_VI,Buf);
	}

	return S_OK;
}

HRESULT CClassFactoryBase::CanUnloadNow()
{
	if( CComponentBase::GetActiveComponents() || IsLocked() )
		return S_FALSE;
	else
		return S_OK;
}

HRESULT	CClassFactoryBase::RegisterServer(HMODULE hModule,const CLSID &clsID,const char *szFriendlyName,const char *szProgID_VI,const char *szProgID)
{
	char szModule[512] ;

	DWORD dwResult = ::GetModuleFileName(hModule,szModule,sizeof(szModule)/sizeof(char));

	if(!dwResult)
		return E_FAIL;

	char szCLSID[CLSID_STRING_SIZE] ;
	CClassFactoryBase::CLSIDtoChar(clsID, szCLSID, sizeof(szCLSID)) ;

	// Build the key CLSID\\{...}
	char szKey[64] ;
	strcpy(szKey, "CLSID\\") ;
	strcat(szKey, szCLSID) ;

	// Add the CLSID to the registry.
	CClassFactoryBase::SetKeyAndValue(szKey, NULL, szFriendlyName) ;

	// Add the server filename subkey under the CLSID key.
	CClassFactoryBase::SetKeyAndValue(szKey, "InprocServer32", szModule) ;

	// Add the ProgID subkey under the CLSID key.
	CClassFactoryBase::SetKeyAndValue(szKey, "ProgID", szProgID) ;

	// Add the version-independent ProgID subkey under CLSID key.
	CClassFactoryBase::SetKeyAndValue(szKey, "VersionIndependentProgID",szProgID_VI) ;

	// Add the version-independent ProgID subkey under HKEY_CLASSES_ROOT.
	CClassFactoryBase::SetKeyAndValue(szProgID_VI, NULL, szFriendlyName) ; 
	CClassFactoryBase::SetKeyAndValue(szProgID_VI, "CLSID", szCLSID) ;
	CClassFactoryBase::SetKeyAndValue(szProgID_VI, "CurVer", szProgID) ;

	// Add the versioned ProgID subkey under HKEY_CLASSES_ROOT.
	CClassFactoryBase::SetKeyAndValue(szProgID, NULL, szFriendlyName) ; 
	CClassFactoryBase::SetKeyAndValue(szProgID, "CLSID", szCLSID) ;

	return S_OK ;
}

// Convert a CLSID to a char string.
void CClassFactoryBase::CLSIDtoChar(const CLSID& clsid,char* szCLSID,int length)
{
	if(length>=CLSID_STRING_SIZE)
		return;

	// Get CLSID
	LPOLESTR wszCLSID = NULL ;
	HRESULT hr = StringFromCLSID(clsid, &wszCLSID) ;
	
	if(FAILED(hr))
		return;

	// Covert from wide characters to non-wide.
	wcstombs(szCLSID, wszCLSID, length) ;

	// Free memory.
	CoTaskMemFree(wszCLSID) ;
}

BOOL CClassFactoryBase::SetKeyAndValue(const char* szKey,const char* szSubkey,const char* szValue)
{
	HKEY hKey;

	char szKeyBuf[1024] ;
	strcpy(szKeyBuf, szKey) ;

	if (szSubkey)
	{
		strcat(szKeyBuf, "\\") ;
		strcat(szKeyBuf, szSubkey ) ;
	}

	long lResult = RegCreateKeyEx(HKEY_CLASSES_ROOT ,szKeyBuf, 0, 0L, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS, 0L, &hKey, 0L) ;

	if (lResult != ERROR_SUCCESS)
		return FALSE ;

	if (szValue)
		RegSetValueEx(hKey, NULL, 0, REG_SZ, (BYTE *)szValue, strlen(szValue)+1) ;

	RegCloseKey(hKey) ;
	return TRUE ;
}

LONG CClassFactoryBase::UnregisterServer(const CLSID& clsID,const char* szProgID_VI,const char* szProgID)
{
	// Convert the CLSID into a char.
	char szCLSID[CLSID_STRING_SIZE] ;
	CClassFactoryBase::CLSIDtoChar(clsID, szCLSID, sizeof(szCLSID)) ;

	// Build the key CLSID\\{...}
	char szKey[64] ;
	strcpy(szKey, "CLSID\\") ;
	strcat(szKey, szCLSID) ;

	// Delete the CLSID Key - CLSID\{...}
	LONG lResult = CClassFactoryBase::RecursiveDeleteKey(HKEY_CLASSES_ROOT, szKey) ;

	// Delete the version-independent ProgID Key.
	lResult = CClassFactoryBase::RecursiveDeleteKey(HKEY_CLASSES_ROOT, szProgID_VI) ;

	// Delete the ProgID key.
	lResult = CClassFactoryBase::RecursiveDeleteKey(HKEY_CLASSES_ROOT, szProgID) ;

	return S_OK ;
}

LONG CClassFactoryBase::RecursiveDeleteKey(HKEY hKeyParent,const char* lpszKeyChild)
{
	// Open the child.
	HKEY hKeyChild ;

	LONG lRes = RegOpenKeyEx(hKeyParent, lpszKeyChild, 0, KEY_ALL_ACCESS, &hKeyChild) ;

	if (lRes != ERROR_SUCCESS)
		return lRes ;

	// Enumerate all of the decendents of this child.
	FILETIME time;
	char szBuffer[256] ;
	DWORD dwSize = 256 ;
	while (RegEnumKeyEx(hKeyChild, 0, szBuffer, &dwSize, NULL, NULL, NULL, &time) == S_OK)
	{
		// Delete the decendents of this child.
		lRes = CClassFactoryBase::RecursiveDeleteKey(hKeyChild, szBuffer) ;
		if (lRes != ERROR_SUCCESS)
		{
			// Cleanup before exiting.
			RegCloseKey(hKeyChild) ;
			return lRes;
		}
		dwSize = 256 ;
	}

	// Close the child.
	RegCloseKey(hKeyChild) ;

	// Delete this child.
	return RegDeleteKey(hKeyParent, lpszKeyChild) ;
}
