// CClassFactoryBase.h: interface for the CClassFactoryBase class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CCLASSFACTORYBASE_H__566F6406_E111_11D2_A179_444553540000__INCLUDED_)
#define AFX_CCLASSFACTORYBASE_H__566F6406_E111_11D2_A179_444553540000__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif

#include <objbase.h>
#include "CComponentBase.h"

typedef HRESULT (*FPCREATECOMPONENT)(IUnknown*, CComponentBase**);
#define END_FACTORY {0L,"",0,0L },

class CClassFactoryData
{
	public:
		const	CLSID		*m_pCLSID;
//		const	char		*m_szRegName;
		const	char		*m_szProgID_VI;
		const	int			*m_pVersion;

		FPCREATECOMPONENT	 CreateComponent;

		BOOL	IsClassID(const CLSID &clsID) const
				{	return (*m_pCLSID == clsID);	}
};

class CClassFactoryBase : public IClassFactory
{
	public:
		CClassFactoryBase(const CClassFactoryData *pFactoryData);
		virtual ~CClassFactoryBase() { }


		// IUnknown Interface

		virtual	HRESULT	__stdcall	QueryInterface(const IID &iid,LPVOID *ppv);
		virtual ULONG	__stdcall	AddRef();
		virtual	ULONG	__stdcall	Release();


		// IClassFactory interface

		// diese Methode wird nur fr COM-Kompatibilitt benutzt ?????
		virtual HRESULT	__stdcall	CreateInstance(IUnknown *pUnknownOuter,const IID &iid,LPVOID *ppv);

		virtual	HRESULT	__stdcall	LockServer(BOOL bLock);


		// Support functions

		static	HRESULT	GetClassObject(const CLSID &clsID,const IID &iid,LPVOID *ppv);
		static	BOOL	IsLocked()	{	return m_cServerLocks>0;	}

		static	HRESULT	Register();
		static	HRESULT	UnRegister();

		static	HRESULT	CanUnloadNow();

		static	HMODULE	m_hModule;

	private:
		long	m_RefCount;

		static	LONG	m_cServerLocks;

		const	CClassFactoryData	*m_pFactoryData;

		static	HRESULT	RegisterServer(HMODULE hModule,const CLSID &clsID,const char *szFriendlyName,const char *szProgID_VI,const char *szProgID);
		static	LONG	UnregisterServer(const CLSID& clsID,const char* szProgID_VI,const char* szProgID);

		static	void	CLSIDtoChar(const CLSID& clsid,char* szCLSID,int length);
		static	BOOL	SetKeyAndValue(const char* szKey,const char* szSubkey,const char* szValue);

		static	LONG	RecursiveDeleteKey(HKEY hKeyParent,const char* lpszKeyChild);
};

#endif  
