// CInstanceFactory.cpp: implementation of the CInstanceFactory class.
//
//////////////////////////////////////////////////////////////////////

//#ifndef _USRDLL
//	#include "stdafx.h"
//	#include "visualprogrammer3.h"
//#endif

#include "CInstanceFactory.h"

#include <io.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInstanceFactory::CInstanceFactory()
{

}

CInstanceFactory::~CInstanceFactory()
{

}

// Returnwert: true - alles Ok, false - Fehler aufgetreten
bool CInstanceFactory::Init( char *pPathName, IUnknown *app )
{
    struct _finddata_t c_file;    
	long hFile;

	{
		char extpath[256];

		sprintf(extpath,"%s\\*.dll",pPathName);

	    if( (hFile = _findfirst(extpath, &c_file )) == -1L )return(false);
	}

    do    
	{
		if(strcmp(c_file.name,".")==0)continue;
		if(strcmp(c_file.name,"..")==0)continue;

		if(c_file.attrib & _A_SUBDIR)	// Recursive Decend
		{
			char path2[256];

			sprintf(path2,"%s\\%s",pPathName,c_file.name);

			if(!Init(path2,app))
				return(false);
		}

		char str[256];

		sprintf(str,"%s\\%s",pPathName,c_file.name);

		HINSTANCE hComp=::LoadLibrary(str);
		if(hComp)
		{
			GETCLASSFACTORIES GetFactories = (GETCLASSFACTORIES)::GetProcAddress(hComp,"GetClassFactories");
			
			if(!GetFactories)
			{
				FreeLibrary(hComp);
				continue;
			}

			int	iCount=0;
			CClassFactoryData	*arr	= GetFactories(iCount);

			if(!iCount || !arr)
			{
				FreeLibrary(hComp);
				continue;
			}


			CREATEINSTANCE CreateFunc = (CREATEINSTANCE) ::GetProcAddress(hComp,"CreateInstance");

			if(!CreateFunc)
			{
				FreeLibrary(hComp);
				continue;
			}

			InstanceInfo	info;
			info.iiData		= arr;
			info.iiCount	= iCount;
			info.CreateFunc	= CreateFunc;

			m_lstInstanceInfos.push_back(info);

			INITDLL	InitDLL = (INITDLL) ::GetProcAddress(hComp,"InitDLL");

			if(InitDLL)
			{
				InitDLL(app);
			}
		}

    } while ( _findnext( hFile, &c_file ) == 0 );     
	
	_findclose( hFile );   

	return true;
}


// erzeugt ein Objekt anhand der ClassID (GUID)
IUnknown *CInstanceFactory::CreateInstance(const CLSID cid)
{
	for(vector<InstanceInfo>::iterator v = m_lstInstanceInfos.begin(); v != m_lstInstanceInfos.end(); ++v)
	{
		for(int i=0;i<(*v).iiCount;++i)
		{
			if(*((*v).iiData[i].m_pCLSID) == cid)
			{
				IUnknown	*pUnknown	= 0L;
				if(SUCCEEDED((*v).CreateFunc(cid,(LPVOID *)&pUnknown)))
				{
					return pUnknown;
				}
			}
		}
	}

	return 0L;
}


// erzeugt ein Objekt anhand des Klassennamens (wie in Component.cpp definiert)
IUnknown *CInstanceFactory::CreateInstance( const char *pName )
{
	for( iterator i=begin();i!=end();i++ )
	{
		const char *pRName=(*i).m_szProgID_VI;

		if(strcmp(pRName,pName)==0)
		{
			IUnknown	*pUnknown	= 0L;

			// ziemlich Ineffizient hier, erst wird richtiges COM Obj ermittelt,
			// dann ID erfragt und wieder danach gesucht (besser machbar)
			if(SUCCEEDED((  i.GetFactoryEntry()->CreateFunc)(*((*i).m_pCLSID),(LPVOID *)&pUnknown)))
			{
				return pUnknown;
			}
		}
	}

	return 0L;
}
