// ASCIIFile.cpp: Implementierung der Klasse CASCIIFile.
//
// letzte nderung: 20.1.2000
//
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <malloc.h>		// malloc,free
#include <stdio.h>		// FILE
#include <fcntl.h>      // filesize
#include <io.h>         // filesize
#include <string.h>     // strlen
#include <assert.h>     // assert

#include "ASCIIFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CASCIIFile::CASCIIFile()
{
	m_Size=0;
	m_Data=0;
}

CASCIIFile::~CASCIIFile()
{
	free(m_Data);
}

CASCIIFile::CASCIIFile( CASCIIFile &a )								// Copy Konstruktor
{
	char *mem=(char *)malloc(a.m_Size);

	if(mem)
	{
		m_Data=mem;
		m_Size=a.m_Size;
	}
	else throw "low memory";
}


const CASCIIFile CASCIIFile::operator=( const CASCIIFile &a )		// Zuweisungsoperator
{
	char *mem=(char *)malloc(a.m_Size);

	if(mem)
	{
		ReleaseData();
		m_Data=mem;
		m_Size=a.m_Size;
	}
	else throw "low memory";

	return(*this);
}





void CASCIIFile::ReleaseData( void )
{
	free(m_Data);m_Data=0;
	m_Size=0;
}

bool CASCIIFile::IO_GetAvailability( const char *pathname )		// Hilffkt, ob Datei vorhanden ist
{
	int handle;

	handle=open(pathname,O_RDONLY);

	if(handle==-1)return(false);

	close(handle);

	return(true);
}

ULONG CASCIIFile::IO_GetFileSize( const char *Name )
{
    int handle,size;

	handle=open(Name,O_RDONLY);

	if(handle==-1)return(0);

    size=filelength(handle);
    close(handle);

	return(size);
}



bool CASCIIFile::IO_SaveASCIIFile( const char *pathname )	 
{
	if(m_Data==0)return(false);
	
	FILE *datei;

    if((datei=fopen(pathname,"wb"))==NULL){ return(false); }
    else
    {
        fwrite(m_Data,m_Size-1,1,datei);
        fclose(datei);
		return(true);
    }
	return(false);
}


bool CASCIIFile::IO_LoadASCIIFile( const char *pathname )		// allociert 1+Filesize, liest Daten und 0 terminiert, free nicht vergessen !!
{
	ULONG size=IO_GetFileSize(pathname);
	char *ret=0;

	ret=(char *)malloc(size+1);

	if(ret==0)return(false);	// no Memory
	
	FILE *datei;

    if((datei=fopen(pathname,"rb"))==NULL){ free(ret);return(false); }
    else
    {
        fread(ret,size,1,datei);
        ret[size]=0;				// 0 terminieren

		m_Size=size+1;
		m_Data=ret;

        fclose(datei);
		return(true);
    }
	return(false);
}

ULONG CASCIIFile::GetDataSize( void )
{
	if(m_Data)return(m_Size-1);
		else return(0);
}

char *CASCIIFile::GetDataPtr( void )						// Speicherposition
{
	return(m_Data);
}

void CASCIIFile::CoverThisData( char *ptr, ULONG size )	// altes freigeben, neues bernehmen, keine Kopie machen
{
	ReleaseData();
	m_Data=ptr;m_Size=size+1;
}



// ist sicher nicht perfekt, case sensitiv oder nicht,
// mu [] am anfang der Zeile stehen, was darf danach stehen, ...
bool CASCIIFile::FindStartINIGroup( const char *ingrpname, ULONG &outGrpStart, ULONG &outGrpEnd )
{
	assert(strlen(ingrpname)<1024);

	char buf[1040];

	sprintf(buf,"[%s]",ingrpname);

	char *ptr=strstr(m_Data,buf);

	if(ptr)
	{
		outGrpStart=((ULONG)ptr)-((ULONG)m_Data);

		sprintf(buf,"%c[",10);

		char *ptr2=strstr(ptr+1,buf);

		if(ptr2){ outGrpEnd=(ULONG)ptr2-(ULONG)m_Data; }
			else outGrpEnd=m_Size-1;

		return(true); 
	}

	return(false);
}



// es u ein Return vor dem String stehen, also am besten immer in einer Gruppe rein
bool CASCIIFile::GetINIString( ULONG inGrpStart, ULONG inGrpEnd, const char *inname,
							   char *_outString, ULONG inMaxChar, ULONG *outGetPos )
{
	assert(strlen(inname)<1024);

	char *outString=_outString;

	char buf[1040];

	sprintf(buf,"%c%s=",(char)10,inname);

	char *ptr=strstr(&m_Data[inGrpStart],buf);

	if(ptr)
	{
		if(((ULONG)ptr)-((ULONG)m_Data)>inGrpEnd)return(false);

		ptr+=strlen(buf);
	
		if(outGetPos)*outGetPos=(ULONG)ptr-(ULONG)m_Data;

		for(ULONG i=0;;i++)
		{
			if(i>=inMaxChar)return(false);

			if(*ptr==10 || *ptr==13 || *ptr==0
			|| *ptr==';')								// Kommentare
			{ 
				*outString=0;
				outString=_outString;					// abschlieende Leerzeichen und Tabs abschneiden
				while(*_outString!=0)
				{
					if(*_outString!=' ' && *_outString!=9)outString=_outString+1;
					_outString++;
				}
				*outString=0;
				return(true); 
			}

			*outString++=*ptr++;
		}
	}

	return(false);
}



