// BucketContainer.h: interface for the CBucketContainer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(BUCKETCONTAINER_H)
#define BUCKETCONTAINER_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// whrend der Operationen wird kein malloc/new/delete/free gemacht

#define INT_MAX 2147483647

template <class TObj, class TPos, int Dim, int MaxBucketsPerLine> class CBucketLine;


// Testfall: * mehr als MaxBucketCount Objekte die perfekt aufeinader liegen
//           * mit leerem Container ProcessContacts aufrufen
//
// m_MaxBucketSize ist nur eine Empfehlung und wird bei schnelchen Splits nicht immer eingehalten

// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************


template <class TObj>
class CBucketObject
{
public:

	// **********************************************************************************************
	CBucketObject()
	{
	}

	// **********************************************************************************************
	~CBucketObject()
	{
	}
	
	// **********************************************************************************************
	void SetData( TObj *inData )
	{
		m_Data=inData;
	}

	// **********************************************************************************************
	TObj *GetData( void )
	{
		return(m_Data);
	}

	// **********************************************************************************************
	CBucketObject<TObj> *GetNext( void )
	{
		return(m_Next);
	}

	// **********************************************************************************************
	void SetNext( CBucketObject<TObj> *inNext )
	{
		m_Next=inNext;
	}


private:

	TObj *					m_Data;
	CBucketObject<TObj>	*	m_Next;
};


// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************


template <class TObj, class TPos, int Dim, int MaxBucketsPerLine>
class CBucket
{
public:

	// **********************************************************************************************
	// constructor (no initialisation for performace reasons)
	CBucket()
	{
	}

	// **********************************************************************************************
	// destructor
					~CBucket()
	{
	}

	// **********************************************************************************************
	// initialisation
	void			Init()
	{
		m_First=0;m_Count=0;
	}

	// **********************************************************************************************
	// insert
	bool			Insert( TObj *inObj, CBucketObject<TObj> *&inObjects, int &inoutFreeObjects, int inMaxBucketSize )
	{
		assert(m_Count!=-1);		// 

		if(inoutFreeObjects>0)
		{
			// correct bounding box
//			if(m_First)inObj->UnionMinMax(m_Min,m_Max);
//				else inObj->GetMinMax(m_Min,m_Max);

			// insert in list
			inObjects->SetNext(m_First);
			m_First=inObjects;

			// set data
			inObjects->SetData(inObj);

			// now we have one element less
			inObjects++;inoutFreeObjects--;
			m_Count++;

			if(m_Count>inMaxBucketSize)			// bucket is full
				return(false);					// split the bucket and reinsert
		}
		else throw "CBucket::Insert failed, MaxObjectCount too low";

		return(true);		// split is not neccesary
	}

	// **********************************************************************************************
	// get count
	int				GetCount( void )
	{
		return(m_Count);
	}

	// **********************************************************************************************
/*	int				CalcLongestDimension( TPos &outMin, TPos &outMax )
	{
		TPos BestSize=m_Max[0]-m_Min[0];
		int ret=0;

		for(int i=1;i<Dim;i++)
		{
			if(m_Max[i]-m_Min[i]>BestSize){ BestSize=m_Max[i]-m_Min[i];ret=i; }
		}

		outMin=m_Min[ret];
		outMax=m_Max[ret];

		return(ret);
	}
*/
	// **********************************************************************************************
	CBucketObject<TObj>	*GetFirst( void )
	{
		assert(m_Count!=-1);
		return(m_First);
	}

	// **********************************************************************************************
	void SetChild( CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine> *inBucketLine )
	{
		m_Count=-1;m_Child=inBucketLine;
	}

	// **********************************************************************************************
	CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine> *GetChild( void )
	{
		assert(m_Count==-1);
		return(m_Child);
	}

private:

	union
	{
		CBucketObject<TObj>	*							m_First;	// m_Count!=-1 pointer to the objects in the bucket (might be 0)
		CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine> *	m_Child;	// m_Count==-1
	};

public: // fuck, nur kurz frs debugging
//	TPos					m_Min[Dim];			// min. position in the dimension (invalid if m_first=0)
//	TPos					m_Max[Dim];			// max. position in the dimension (invalid if m_first=0)
	int						m_Count;			// 0..N: number of childs (m_First is valid), -1: object has children (m_Child is valid)
};


// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************


template <class TObj, class TPos, int Dim, int MaxBucketsPerLine>
class CBucketLine
{
public:

	// **********************************************************************************************
	// constructor
				CBucketLine()
	{
	}

	// **********************************************************************************************
	// destructor
	virtual ~	CBucketLine()
	{
	}

	// **********************************************************************************************
	// initialisation
	void		Init( int inBucketDimension, const TPos &inLowBound, const TPos &inHighBound )
	{
		for(int i=0;i<MaxBucketsPerLine;i++)
			m_Bucket[i].Init();

		m_BucketDimension=inBucketDimension;
		m_LowBound=inLowBound;
		m_HighBound=inHighBound;
	}


	// **********************************************************************************************
	void HelperCalcMinMaxBuck( TObj *inObj, int &outMinBuck, int &outMaxBuck )
	{
		TPos min,max;

		inObj->GetMinMaxDimension(m_BucketDimension,min,max);

		// !!!!!!! optimizable !!!!!!!
		outMinBuck=(int)((min-m_LowBound)/(m_HighBound-m_LowBound)*MaxBucketsPerLine);
		outMaxBuck=(int)((max-m_LowBound)/(m_HighBound-m_LowBound)*MaxBucketsPerLine);

		// limit 
		if(outMinBuck<0)outMinBuck=0;
		if(outMaxBuck<0)outMaxBuck=0;
		if(outMinBuck>MaxBucketsPerLine-1)outMinBuck=MaxBucketsPerLine-1;
		if(outMaxBuck>MaxBucketsPerLine-1)outMaxBuck=MaxBucketsPerLine-1;

		// Debug
//		printf("    dim(%d) Position [%.2f..%.2f] insert in [%d..%d]\n",m_BucketDimension,min,max,outMinBuck,outMaxBuck);
	}

	// **********************************************************************************************
	// insert only one object, so the pointer in inObj is not used as input
	void		InsertOneObject( TObj *inObj, 
								 CBucketObject<TObj> *&inObjects, 
								 int &inoutFreeObjects,
								 CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine> *&inBucketNodes, 
								 int &inoutFreeBuckets,
								 int inMaxBucketSize, TPos inMinMax[Dim*2] )
	{
		int minbuck,maxbuck;

		// Calculate the affected Bucket Numbers
		HelperCalcMinMaxBuck(inObj,minbuck,maxbuck);

		TPos bucketsize=(m_HighBound-m_LowBound)/MaxBucketsPerLine;

		for(int i=minbuck;i<maxbuck+1;i++)
		{
			inMinMax[m_BucketDimension*2]=bucketsize*i+m_LowBound;
			inMinMax[m_BucketDimension*2+1]=bucketsize*(i+1)+m_LowBound;

			if(m_Bucket[i].GetCount()==-1)		// Node is already a subtree, object falls down
			{
				CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine> *line=m_Bucket[i].GetChild();

				line->InsertOneObject(inObj,inObjects,inoutFreeObjects,inBucketNodes,inoutFreeBuckets,inMaxBucketSize,inMinMax);
			}
			else
			if(!m_Bucket[i].Insert(inObj,inObjects,inoutFreeObjects,inMaxBucketSize))
			{
				// I have to split the Bucket
				TPos min=inMinMax[0],max=inMinMax[1];
				int longdim=0;

				for(int d=1;d<Dim;d++)
				{
					if(inMinMax[d*2+1]-inMinMax[d*2]>max-min)
					{
						longdim=d;
						min=inMinMax[d*2];
						max=inMinMax[d*2+1];
					}
				}


				if(inoutFreeBuckets>0)
				{
					(*inBucketNodes).Init(longdim,min,max);

					CBucketObject<TObj>	*p=m_Bucket[i].GetFirst();

					while(p)
					{
						(*inBucketNodes).MoveOneObject(p,inObjects,inoutFreeObjects);

						p=p->GetNext();
					}

					m_Bucket[i].SetChild(inBucketNodes);		// set m_Count to -1 

					inBucketNodes++;inoutFreeBuckets--;
				}
//				else throw "CBucketLine::Insert failed, MaxBucketCount too low";	// Fehlerausgabe msste nicht sein
			}
		}

		inMinMax[m_BucketDimension*2]=m_LowBound;
		inMinMax[m_BucketDimension*2+1]=m_HighBound;
	}

	// **********************************************************************************************
	// die CBucket werden gleich verwendet
	// return, Menge konnte erfolgreich aufgeteilt werden
	void		MoveOneObject( CBucketObject<TObj> *inObj,
							   CBucketObject<TObj> *&inObjects, 
							   int &inoutFreeObjects )
	{
		int minbuck,maxbuck;


		HelperCalcMinMaxBuck(inObj->GetData(),minbuck,maxbuck);

//		printf("    moveoneobject: %p",inObj);

		for(int i=minbuck;i<maxbuck+1;i++)
		{
			m_Bucket[i].Insert(inObj->GetData(),inObjects,inoutFreeObjects,INT_MAX);
		}
//		Debug();
	}


	// **********************************************************************************************
	// process all objects that overlap in all dimensions (oriented bounding boxes)
	void		ProcessContacts( I2DMemoryDC *dc )
	{
		for(int i=0;i<MaxBucketsPerLine;i++)
		{
			int cnt=m_Bucket[i].GetCount();

			if(cnt>=0)										// First
			{
				CBucketObject<TObj>	*p=m_Bucket[i].GetFirst();

				while(p)
				{
					CBucketObject<TObj>	*p2=p->GetNext();

					while(p2)
					{
						p->GetData()->DebugContact(dc,p2->GetData());
						p2=p2->GetNext();
					}
					p=p->GetNext();
				}
			}
			else											// Child
			  m_Bucket[i].GetChild()->ProcessContacts(dc);
		}
	}


	// **********************************************************************************************
	void		Debug( I2DMemoryDC *dc, int inMaxlevel, TPos inMinMax[Dim*2] )
	{
		if(inMaxlevel>=1)
			TObj::DebugSplit(dc,m_BucketDimension,m_LowBound,m_HighBound,MaxBucketsPerLine);

		TPos bucketsize=(m_HighBound-m_LowBound)/MaxBucketsPerLine;

		for(int i=0;i<MaxBucketsPerLine;i++)
		{
			int cnt=m_Bucket[i].GetCount();

			inMinMax[m_BucketDimension*2]=bucketsize*i+m_LowBound;
			inMinMax[m_BucketDimension*2+1]=bucketsize*(i+1)+m_LowBound;

			if(inMaxlevel>=1)
			{
				TObj::DebugContainer(dc, inMinMax[0],inMinMax[2],
										 inMinMax[1],inMinMax[3]);

			}

			if(cnt>=0)												// 
			{
				CBucketObject<TObj>	*p=m_Bucket[i].GetFirst();

				while(p)
				{
					p->GetData()->DebugObject(dc);
					p=p->GetNext();
				}
			}
			else													// 
			{
				CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine> *line=m_Bucket[i].GetChild();

				int dim=line->m_BucketDimension;

				inMinMax[dim*2]=line->m_LowBound;
				inMinMax[dim*2+1]=line->m_HighBound;

				m_Bucket[i].GetChild()->Debug(dc,inMaxlevel-1,inMinMax);
			}
		}

		inMinMax[m_BucketDimension*2]=m_LowBound;
	}

	// **********************************************************************************************

private:

	CBucket<TObj,TPos,Dim,MaxBucketsPerLine>	m_Bucket[MaxBucketsPerLine];
	TPos										m_LowBound;					// low bound of the bucket line
	TPos										m_HighBound;				// high bound of the bucket line
	int											m_BucketDimension;			// dimension of the bucket line (0=x, 1=y, ...
};

// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************
// *****************************************************************************************************

// z.B. C3DObject,float,3,7,10,32,9000  (in 3d space,
//										coordinates in float,
//										the objects are referenced via "C3DObject *",
//										10 buckets are the maximum (for preallocation),
//										max.. 9000 objectes parts (for preallocation), 
//										32 buckets per node (best value depends on situation)
//
// if MaxObjectCount is reached, errors will occur
//
template <class TObj, class TPos, int Dim, int MaxBucketsPerLine,
		  int MaxBucketCount, int MaxObjectCount >
class CBucketContainer  
{
public:
	
	// **********************************************************************************************
	// constructor
				CBucketContainer()
	{
		m_FreeBuckets=MaxBucketCount;
	}
	
	// **********************************************************************************************
	// destructor
	virtual		~CBucketContainer()
	{
	}

	// **********************************************************************************************
	// initialize
	void		Init( int inMaxBucketSize, const int inUseDimension=0 )
	{
		assert(inUseDimension<Dim);
		m_MaxBucketSize=inMaxBucketSize;
		m_FreeObjects=MaxObjectCount;m_FreeBuckets=MaxBucketCount-1;

		m_BucketNodes[0].Init(inUseDimension,m_MinMax[inUseDimension*2],m_MinMax[inUseDimension*2+1]);
	}

	// **********************************************************************************************
	void		SetMinMax( int inDim, TPos inMin, TPos inMax )
	{
		assert(inDim<Dim);
		m_MinMax[inDim*2]=inMin;
		m_MinMax[inDim*2+1]=inMax;
	}

	// **********************************************************************************************
	// insert one object
	void		InsertObject( TObj *inObj )
	{
		CBucketObject<TObj> *a=&(m_Objects[GetObjectCount()]);
		CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine> *b=&(m_BucketNodes[GetBucketCount()]);

		m_BucketNodes[0].InsertOneObject(inObj,
			a,m_FreeObjects,
			b,m_FreeBuckets,
			m_MaxBucketSize,m_MinMax);
	}

	// **********************************************************************************************
	// process all objects that overlap in all dimensions (oriented bounding boxes)
	void		ProcessContacts( I2DMemoryDC *dc )
	{
		m_BucketNodes[0].ProcessContacts(dc);
	}

	// **********************************************************************************************
	int			GetObjectCount( void )
	{
		return(MaxObjectCount-m_FreeObjects);
	}

	// **********************************************************************************************
	int			GetBucketCount( void )
	{
		return(MaxBucketCount-m_FreeBuckets);
	}

	// **********************************************************************************************
	void		Debug( I2DMemoryDC *dc, int inMaxlevel )
	{
		m_BucketNodes[0].Debug(dc,inMaxlevel,m_MinMax);
	}

	// **********************************************************************************************

private:

	CBucketObject<TObj>								m_Objects[MaxObjectCount];		// shared pool for all nodes
	int												m_FreeObjects;					// 0..MaxObjectCount
	CBucketLine<TObj,TPos,Dim,MaxBucketsPerLine>	m_BucketNodes[MaxBucketCount];	// m_BucketNodes[0] is always the root
	int												m_FreeBuckets;					// 0..MaxBucketCount-1
	int												m_MaxBucketSize;				// max Anzahl, sonst Split
	TPos											m_MinMax[Dim*2];				// BoundingBox
};

#endif // !defined(BUCKETCONTAINER_H)
