#ifndef MATH_H
#define MATH_H

#include <math.h>
#include <stdlib.h>		// rand()
#include <assert.h>		// assert()
/*#include <stdio.h>
#include <string.h>
#include "types.hpp"
*/

#define PI	3.1415926535897932384626433832795
#define FLOATEPSILON	0.000001				// Werte nur erraten
#define DOUBLEEPSILON	0.000000000001			// Werte nur erraten


template <class T> T quad(T a) { return(a*a); }


class View3dC;
template <class T, class T2> class _Matrix34C;



template < class T > class _Vector3dC
{
public:
    T p[3];

	// Konstruktoren
inline						_Vector3dC			();
inline						_Vector3dC			( T x, T y, T z );
inline						_Vector3dC			( const _Vector3dC<T> &x );
inline						_Vector3dC			( const T &x );

inline    void				Set					( T x, T y, T z );

inline    void				Max					( const _Vector3dC<T> &c );							// Komponentenweise wird jeweils das max genommen
inline    void				Min					( const _Vector3dC<T> &c );							// Komponentenweise wird jeweils das min genommen

inline    void				GetOtherBaseVec		( _Vector3dC<T> &a, _Vector3dC<T> &b ) const;

inline	  _Vector3dC<T>		operator-			( const _Vector3dC<T> &other ) const;				// Subtraktion
inline	  _Vector3dC<T>		operator-=			( const _Vector3dC<T> &other );						// Subtraktion
inline	  _Vector3dC<T>		operator+			( const _Vector3dC<T> &other ) const;				// Addition
inline	  _Vector3dC<T>		operator+=			( const _Vector3dC<T> &other );						// Addition
inline    T					operator*			( const _Vector3dC<T> &other ) const;				// Skalarprodukt
inline	  _Vector3dC<T>		operator*			( const T fak ) const;								// Skalierung
inline	  _Vector3dC<T>		operator*=			( const T fak );									// Skalierung

inline    T					Length				( void ) const;										// Lnge des Vektors
inline    double			LengthQuad			( void ) const;										// (Lnge des Vektors)^2
inline    T					LengthNear			( void ) const;										// ungefhre Lnge des Vektors (sollte schneller sein)
inline    void				CrossProd			( const _Vector3dC<T> &a, const _Vector3dC<T> &b );	// Kreuzprodukt
inline    void				Normalize			( void );											// Lnge des Vectors auf 1 skalieren (normieren)

inline    void				CalcVector			( const _Vector3dC<T> &what, const _Matrix34C<T,float> &mat );
inline    void				CalcVectorInv		( const _Vector3dC<T> &what, const _Matrix34C<T,float> &mat );
inline    void				CalcVector			( const _Vector3dC<T> &what, const _Matrix34C<T,double> &mat );
inline    void				CalcVectorInv		( const _Vector3dC<T> &what, const _Matrix34C<T,double> &mat );

const inline float			_LightVector2		( const _Vector3dC<T> &n );	// wie oben, nur pos=0,0,0 und beide Vektoren sind 1 lang

    friend class View3dC;
};






// eingentlich richtige Representation:
//
// m11 m12 m13 p1 - als double
// m21 m22 m23 p2 - als double
// m31 m32 m33 p3 - als double
//  0   0   0  1 
//
//
// wirkliche interne Representation:
//                                  (Vorteil: p123 kann als Vektor angesprochen werden)
// m11 m12 m13    - als double
// m21 m22 m23    - als double
// m31 m32 m33    - als double
// p1  p2  p3     - als float
//
template <class T, class T2> class _Matrix34C
{
public:
    T2		m[3][3];			// evtl. knnte m[12] effizienter sein
    T		p[3];

	// Konstruktoren
inline						_Matrix34C			();
inline						_Matrix34C			( const _Matrix34C<T,T2> &x );

inline    void				EinheitsMat			( double eins=1.0 );
inline    void				SetWxyz				( double wx, double wy, double wz );
inline    void				SetVec				( float x, float y, float z );
inline    void				SetVec				( const _Vector3dC<T> w );					
inline    void				SetBase				( _Vector3dC<T> &a,_Vector3dC<T> &b );		// nur Kreuzprodukt
inline    bool				SetNormBase			( const _Vector3dC<T> &a, const _Vector3dC<T> &b );	// senkrecht, normieren, Kreuzprodukt, false wenn keine Basis
inline    bool				SetOriented			( _Vector3dC<T> &a );						// vorne ist Vectorrichtung, Drehung darum ist nicht gegeben

inline _Vector3dC<T> &		GetVec				( void ) const;					// Translation
inline _Vector3dC<T>		GetWxVec			( void ) const;					// X-Achse (horizontal)
inline _Vector3dC<T>		GetWyVec			( void ) const;					// Z-Achse (Tiefe)
inline _Vector3dC<T>		GetWzVec			( void ) const;					// Y-Achse (vertikal)

inline    void				ApplyWxyz			( double wx, double wy, double wz );
inline    void				MixWith_onlyWxyz	( double fak, const _Matrix34C<T,T2> &a );
inline    void				ApplyVec			( float x, float y, float z );
inline    void				ApplyVec			( const _Vector3dC<T> &w );
inline    void				ApplyScale_onlyWxyz	( double m );
inline    void				ApplyMat			( const _Matrix34C<T,T2> &x );
  
inline    void				Copy_onlyWxyz		( const _Matrix34C<T,T2> &x );
inline    void				Inverse				( const _Matrix34C<T,T2> &a );	// Bei Orthogonalmatrizen reicht Transponieren
inline    void				Inverse_onlyWxyz	( const _Matrix34C<T,T2> &a );	// Bei Orthogonalmatrizen reicht Transponieren
inline    void				Transp_onlyWxyz		( void );

inline    void				Mul					( const _Matrix34C<T,T2> &a, const _Matrix34C<T,T2> &b );
inline    void				Mul_onlyWxyz		( const _Matrix34C<T,T2> &a, const _Matrix34C<T,T2> &b );
inline    void				Pot32				( const _Matrix34C<T,T2> &x );
inline    void				Pot32_onlyWxyz		( const _Matrix34C<T,T2> &x );
inline    void				MulPotX				( const _Matrix34C<T,T2> &x, int pot );
inline    void				GramSchmitt			( void );
inline    void				Normieren			( void );
inline    void				Daempfung			( double fakxyz, double fak );
inline	  void				Save				( FILE *out ); 
inline	  void				Load				( FILE *in ); 

const _Matrix34C<T,T2>		operator=			( const _Matrix34C<T,T2> &mat );
//const _Matrix34C<float,float>	operator=			( const _Matrix34C<float,float> &mat );
		void				Convert				( _Matrix34C<float,double> &mat );

    friend class _Vector3dC<T>;
};


// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************

template <class T> inline
_Vector3dC<T>::_Vector3dC()
{}

template <class T> inline
_Vector3dC<T>::_Vector3dC( T x, T y, T z )
{
    p[0]=x;p[1]=y;p[2]=z;
}

template <class T> inline
_Vector3dC<T>::_Vector3dC( const T &x )
{
	ASSERT(x==0);		// sonst macht das glaub ich keinen Sinn
    p[0]=x;p[1]=x;p[2]=x;
}

template <class T> inline
_Vector3dC<T>::_Vector3dC( const _Vector3dC<T> &x )
{
    p[0]=x.p[0];p[1]=x.p[1];p[2]=x.p[2];
}

template <class T> inline
void _Vector3dC<T>::Set( T x, T y, T z )
{
    p[0]=x;p[1]=y;p[2]=z;
}



// wie oben, nur pos=0,0,0 und beide Vektoren sind 1 lang
template <class T> inline
const float _Vector3dC<T>::_LightVector2(const _Vector3dC<T> &n )
{
//	 const _Vector3dC<T> &light ist der Vector selbst
    float h=n*(*this);

    return(h>0.0f?h:0.0f);
}

template <class T> inline
void _Vector3dC<T>::Max( const _Vector3dC<T> &c )
{
    p[0]=(p[0]>c.p[0])?p[0]:c.p[0];
    p[1]=(p[1]>c.p[1])?p[1]:c.p[1];
    p[2]=(p[2]>c.p[2])?p[2]:c.p[2];
}


template <class T> inline
void _Vector3dC<T>::Min( const _Vector3dC<T> &c )
{
    p[0]=(p[0]<c.p[0])?p[0]:c.p[0];
    p[1]=(p[1]<c.p[1])?p[1]:c.p[1];
    p[2]=(p[2]<c.p[2])?p[2]:c.p[2];
}


template <class T> inline
_Vector3dC<T> _Vector3dC<T>::operator-( const _Vector3dC<T> &b ) const
{
	return _Vector3dC<T>(p[0]-b.p[0],p[1]-b.p[1],p[2]-b.p[2] );
};

template <class T> inline
_Vector3dC<T> _Vector3dC<T>::operator-=( const _Vector3dC<T> &b )
{
	p[0]-=b.p[0];p[1]-=b.p[1];p[2]-=b.p[2];

	return *this;
}

template <class T> inline
_Vector3dC<T> _Vector3dC<T>::operator+( const _Vector3dC<T> &b ) const
{
	return _Vector3dC<T>(p[0]+b.p[0],p[1]+b.p[1],p[2]+b.p[2] );
}

template <class T> inline
_Vector3dC<T> _Vector3dC<T>::operator+=( const _Vector3dC<T> &b )
{
	p[0]+=b.p[0];p[1]+=b.p[1];p[2]+=b.p[2];

	return *this;
}


template <class T> inline
_Vector3dC<T> _Vector3dC<T>::operator*( const T f ) const
{
	return _Vector3dC<T>(p[0]*f,p[1]*f,p[2]*f );
}

template <class T> inline
T _Vector3dC<T>::operator*( const _Vector3dC<T> &b ) const
{
	return(p[0]*b.p[0]+p[1]*b.p[1]+p[2]*b.p[2]);
}


template <class T> inline
_Vector3dC<T> _Vector3dC<T>::operator*=( const T f )
{
	p[0]*=f;p[1]*=f;p[2]*=f;

	return *this;
}






template <class T> inline
void _Vector3dC<T>::CrossProd( const _Vector3dC<T> &a, const _Vector3dC<T> &b )
{
    p[0]=a.p[1]*b.p[2] - a.p[2]*b.p[1];
    p[1]=a.p[2]*b.p[0] - a.p[0]*b.p[2];
    p[2]=a.p[0]*b.p[1] - a.p[1]*b.p[0];
}


// Rckgabewert:
//    ziel
template <class T> inline
void _Vector3dC<T>::CalcVector( const _Vector3dC<T> &what, const _Matrix34C<T,double> &mat )
{
    double xa=(double)(what.p[0]),
           ya=(double)(what.p[1]),
           za=(double)(what.p[2]);

    p[0]=(float)(mat.m[0][0]*xa + mat.m[0][1]*ya + mat.m[0][2]*za + mat.p[0]);
    p[1]=(float)(mat.m[1][0]*xa + mat.m[1][1]*ya + mat.m[1][2]*za + mat.p[1]);
    p[2]=(float)(mat.m[2][0]*xa + mat.m[2][1]*ya + mat.m[2][2]*za + mat.p[2]);
}

template <class T> inline
void _Vector3dC<T>::CalcVector( const _Vector3dC<T> &what, const _Matrix34C<T,float> &mat )
{
    float  xa=what.p[0],
           ya=what.p[1],
           za=what.p[2];

    p[0]= mat.m[0][0]*xa + mat.m[0][1]*ya + mat.m[0][2]*za + mat.p[0];
    p[1]= mat.m[1][0]*xa + mat.m[1][1]*ya + mat.m[1][2]*za + mat.p[1];
    p[2]= mat.m[2][0]*xa + mat.m[2][1]*ya + mat.m[2][2]*za + mat.p[2];
}


// Rckgabewert:  (Mat mu orthogonal sein)
//    ziel
template <class T> inline
void _Vector3dC<T>::CalcVectorInv( const _Vector3dC<T> &what, const _Matrix34C<T,float> &mat )
{
    double xa=(double)(what.p[0]-mat.p[0]),
           ya=(double)(what.p[1]-mat.p[1]),
           za=(double)(what.p[2]-mat.p[2]);

    p[0]=(float)(mat.m[0][0]*xa + mat.m[1][0]*ya + mat.m[2][0]*za);
    p[1]=(float)(mat.m[0][1]*xa + mat.m[1][1]*ya + mat.m[2][1]*za);
    p[2]=(float)(mat.m[0][2]*xa + mat.m[1][2]*ya + mat.m[2][2]*za);
}

template <class T> inline
void _Vector3dC<T>::CalcVectorInv( const _Vector3dC<T> &what, const _Matrix34C<T,double> &mat )
{
    double xa=(double)(what.p[0]-mat.p[0]),
           ya=(double)(what.p[1]-mat.p[1]),
           za=(double)(what.p[2]-mat.p[2]);

    p[0]=(float)(mat.m[0][0]*xa + mat.m[1][0]*ya + mat.m[2][0]*za);
    p[1]=(float)(mat.m[0][1]*xa + mat.m[1][1]*ya + mat.m[2][1]*za);
    p[2]=(float)(mat.m[0][2]*xa + mat.m[1][2]*ya + mat.m[2][2]*za);
}










template <class T> inline
void _Vector3dC<T>::Normalize( void )
{
    T len=1.0f/Length();

    p[0]*=len;p[1]*=len;p[2]*=len;
}


template <class T> inline
T _Vector3dC<T>::Length( void ) const
{
    return( (float)sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]) );
}


template <class T> inline
double _Vector3dC<T>::LengthQuad( void ) const
{
    return( p[0]*p[0]+p[1]*p[1]+p[2]*p[2] );
}

template <class T> inline
T _Vector3dC<T>::LengthNear( void ) const
{
    _Vector3dC<T> b;

    b.p[0]=ABS(p[0]);b.p[1]=ABS(p[1]);b.p[2]=ABS(p[2]);

    if(b.p[0]>b.p[1])
    {
        if(b.p[0]>b.p[2])
        {
            return((float)(b.p[0] + ((b.p[1]>b.p[2])?( (b.p[1]*0.5)+(b.p[2]*0.25) ):( (b.p[2]*0.5)+(b.p[1]*0.25) ))));
        }
        else
        {
            return((float)(b.p[2] + (b.p[0]*0.5)+(b.p[2]*0.25)));
        }
    }
    else
    {
        if(b.p[1]>b.p[2])
        {
            return((float)(b.p[1] + ((b.p[0]>b.p[2])?( (b.p[0]*0.5)+(b.p[2]*0.25) ):( (b.p[2]*0.5)+(b.p[0]*0.25) ))));
        }
        else
        {
            return((float)(b.p[2] + (b.p[1]*0.5)+(b.p[2]*0.25)));
        }
    }
}






// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************





template <class T, class T2> inline		// Konstruktor
_Matrix34C<T,T2>::_Matrix34C()
{}

template <class T, class T2> inline		// Copy Konstruktor
_Matrix34C<T,T2>::_Matrix34C( const _Matrix34C<T,T2> &x )
{
	for(int a=0;a<3;a++)
	{
		p[a]=x.p[a];
		for(int b=0;b<3;b++)m[a][b]=x.m[a][b];
	}
}


template <class T, class T2> inline
void _Matrix34C<T,T2>::EinheitsMat( double eins )
{
	m[0][1]=m[0][2]=m[1][0]=m[1][2]=m[2][0]=m[2][1]=p[0]=p[1]=p[2]=0.0;
    m[0][0]=m[1][1]=m[2][2]=eins;
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::Mul( const _Matrix34C<T,T2> &a, const _Matrix34C<T,T2> &b )
{
	assert(&a!=this);
	assert(&b!=this);

    int i,e;

    for(i=0;i<3;i++)
        for(e=0;e<3;e++)
            m[i][e]= a.m[i][0]*b.m[0][e] + a.m[i][1]*b.m[1][e] + a.m[i][2]*b.m[2][e];

    for(i=0;i<3;i++)
        p[i]=(float)(a.m[i][0]*b.p[0] + a.m[i][1]*b.p[1] + a.m[i][2]*b.p[2] + a.p[i]);
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::Mul_onlyWxyz( const _Matrix34C<T,T2> &a, const _Matrix34C<T,T2> &b )
{
	assert(&a!=this);
	assert(&b!=this);

    for(int i=0;i<3;i++)
        for(int e=0;e<3;e++)
            m[i][e]= a.m[i][0]*b.m[0][e] + a.m[i][1]*b.m[1][e] + a.m[i][2]*b.m[2][e];
}


template <class T, class T2> inline
void _Matrix34C<T,T2>::Copy_onlyWxyz( const _Matrix34C<T,T2> &x )
{
	for(int a=0;a<3;a++)
		for(int b=0;b<3;b++)m[a][b]=x.m[a][b];
}


template <class T, class T2> inline
void _Matrix34C<T,T2>::Pot32( const _Matrix34C<T,T2> &x )
{
    _Matrix34C<T,T2> a;

    Mul(x,x);a.Mul(*this,*this);      // *4
    Mul(a,a);a.Mul(*this,*this);    // *4
    Mul(a,a);						// *2
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::Pot32_onlyWxyz( const _Matrix34C<T,T2> &x )
{
    _Matrix34C<T,T2> a;

    Mul_onlyWxyz(x,x);a.Mul_onlyWxyz(*this,*this);      // *4
    Mul_onlyWxyz(a,a);a.Mul_onlyWxyz(*this,*this);      // *4
    Mul_onlyWxyz(a,a);                   // *2
}


 



// Wesentlich besserer Alg. mglich !!
template <class T, class T2> inline
void _Matrix34C<T,T2>::MulPotX( const _Matrix34C<T,T2> &x, int pot )
{
    for(int p=0;p<pot;p++)
    {
        _Matrix34C<T,T2> cpy;

        cpy=*this;
        Mul(x,cpy);
    }
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::SetWxyz( double wx, double wy, double wz )
{
    double swx=sin(wx), swy=sin(wy), swz=sin(wz),
           cwx=cos(wx), cwy=cos(wy), cwz=cos(wz);
    
    double a=swx*swz, b=cwx*swy, c=swx*cwz;
/*
    m[0][0]=cwy*cwz;            // Original
    m[1][0]=-cwy*swz;
    m[2][0]=swy;
    m[0][1]=cwx*swz+c*swy;
    m[1][1]=cwx*cwz-a*swy;
    m[2][1]=-swx*cwy;
    m[0][2]=a-b*cwz;
    m[1][2]=c+b*swz;
    m[2][2]=cwx*cwy;
*/
    m[0][0]=cwy*cwz;			// Optimiert
    m[0][1]=-cwy*swz;
    m[0][2]=swy;
    m[1][0]=cwx*swz+c*swy;
    m[1][1]=cwx*cwz-a*swy;
    m[1][2]=-swx*cwy;
    m[2][0]=a-b*cwz;
    m[2][1]=c+b*swz;
    m[2][2]=cwx*cwy;
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::SetBase( _Vector3dC<T> &a, _Vector3dC<T> &b )
{
    _Vector3dC<T> c;

    m[0][0]=a.p[0];
    m[1][0]=a.p[1];
    m[2][0]=a.p[2];
    m[0][1]=b.p[0];
    m[1][1]=b.p[1];
    m[2][1]=b.p[2];

    c.CrossProd(a,b);
    m[0][2]=c.p[0];
    m[1][2]=c.p[1];
    m[2][2]=c.p[2];
}


// evtl. effizienter machbar
template <class T, class T2> inline
bool _Matrix34C<T,T2>::SetNormBase( const _Vector3dC<T> &_a, const _Vector3dC<T> &_b )
{
	_Vector3dC<T> a=_a;
	_Vector3dC<T> b=_b;

	a.Normalize();
	b.Normalize();

	T fak=a*b;

	if(fak-1>-FLOATEPSILON && fak-1<FLOATEPSILON)return(false);	// ungehr 1 = parallel

	b-=a*fak;	// senkrecht aufeinander

	b.Normalize();

    _Vector3dC<T> c;

    m[0][0]=a.p[0];
    m[1][0]=a.p[1];
    m[2][0]=a.p[2];
    m[0][1]=b.p[0];
    m[1][1]=b.p[1];
    m[2][1]=b.p[2];

    c.CrossProd(b,a);
    m[0][2]=-c.p[0];
    m[1][2]=-c.p[1];
    m[2][2]=-c.p[2];

	return(true);
}



template <class T,class T2> inline
void _Matrix34C<T,T2>::SetVec( float x, float y, float z )
{
    p[0]=x;p[1]=y;p[2]=z;
}

// evtl. effizienter machbar
// Die Orientierung wird nur anhand eines Vectors ausgerechnet - die Rotation um diesen ist eher zufllig
template <class T, class T2> inline
bool _Matrix34C<T,T2>::SetOriented( _Vector3dC<T> &a )
{
	Vector3dC b(0,0,1);

	if(a.p[0]==0 && a.p[1]==0)
	{
		b=Vector3dC(1,0,0);
	
		if(a.p[2]==0)return(false);
	}
	SetNormBase(a,b);
	return(true);
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::SetVec( const _Vector3dC<T> w )
{
    p[0]=w.p[0];p[1]=w.p[1];p[2]=w.p[2];
}


template <class T, class T2> inline
_Vector3dC<T> &_Matrix34C<T,T2>::GetVec( void ) const
{
    return(*((_Vector3dC<T> *)p));
}

template <class T, class T2> inline
_Vector3dC<T> _Matrix34C<T,T2>::GetWxVec( void ) const
{ 
	return(_Vector3dC<T>( (float)m[0][0],(float)m[1][0],(float)m[2][0])  ); 
}

template <class T, class T2> inline
_Vector3dC<T> _Matrix34C<T,T2>::GetWyVec( void ) const
{ 
	return(_Vector3dC<T>( (float)m[0][1],(float)m[1][1],(float)m[2][1])  ); 
}

template <class T, class T2> inline
_Vector3dC<T> _Matrix34C<T,T2>::GetWzVec( void ) const
{ 
	return(_Vector3dC<T>( (float)m[0][2],(float)m[1][2],(float)m[2][2])  ); 
}



template <class T, class T2> inline
void _Matrix34C<T,T2>::ApplyWxyz( double wx, double wy, double wz )
{
    _Matrix34C<T,T2> drot,drot2;

    drot.SetWxyz(wx,wy,wz);drot2.Pot32_onlyWxyz(drot);

    drot.Copy_onlyWxyz(*this);

    Mul_onlyWxyz(drot,drot2);
    GramSchmitt();
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::ApplyVec( float x, float y, float z )
{
    p[0]+=x;p[1]+=y;p[2]+=z;
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::ApplyVec( const _Vector3dC<T> &w )
{
    p[0]+=w.p[0];p[1]+=w.p[1];p[2]+=w.p[2];
}



// nach folgender Regel: NewVec=(Vec-Pos)Mat+Pos  (damit um Pos gedreht wird)
template <class T, class T2> inline
void _Matrix34C<T,T2>::ApplyMat( const _Matrix34C<T,T2> &a )
{
    _Matrix34C<T,T2> help;

    help.Copy_onlyWxyz(*this);
    Mul_onlyWxyz(help,a);
    ApplyVec(a.p[0],a.p[1],a.p[2] /*a->GetVec()*/);
}


// Optimierbar - indem Length benutzt wird (Matrix ist auch nur 3 Vektoren)
template <class T, class T2> inline
void _Matrix34C<T,T2>::GramSchmitt( void )
{
    double len;
    double k;
    int e;

    // Normierung
    len=1.0/sqrt(m[2][0]*m[2][0]+m[2][1]*m[2][1]+m[2][2]*m[2][2]);
    for(e=0;e<3;e++)m[2][e]*=len;

    // Orthogonalisierung
    k=m[2][0]*m[1][0]+m[2][1]*m[1][1]+m[2][2]*m[1][2];
    for(e=0;e<3;e++)m[1][e]-=m[2][e]*k;

    // Normierung
    len=1.0/sqrt(m[1][0]*m[1][0]+m[1][1]*m[1][1]+m[1][2]*m[1][2]);
    for(e=0;e<3;e++)m[1][e]*=len;

    // Orthogonalisierung
    k=m[0][0]*m[1][0]+m[0][1]*m[1][1]+m[0][2]*m[1][2];
    for(e=0;e<3;e++)m[0][e]-=m[1][e]*k;

    // Orthogonalisierung
    k=m[0][0]*m[2][0]+m[0][1]*m[2][1]+m[0][2]*m[2][2];
    for(e=0;e<3;e++)m[0][e]-=m[2][e]*k;

    // Normierung
    len=1.0/sqrt(m[0][0]*m[0][0]+m[0][1]*m[0][1]+m[0][2]*m[0][2]);
    for(e=0;e<3;e++)m[0][e]*=len;
}


template <class T, class T2> inline
void _Matrix34C<T,T2>::Normieren( void )
{
    int e;
    double len;
    
    len=1.0/sqrt(m[0][0]*m[0][0]+m[0][1]*m[0][1]+m[0][2]*m[0][2]);
    for(e=0;e<3;e++)m[0][e]*=len;

    len=1.0/sqrt(m[1][0]*m[1][0]+m[1][1]*m[1][1]+m[1][2]*m[1][2]);
    for(e=0;e<3;e++)m[1][e]*=len;

    len=1.0/sqrt(m[2][0]*m[2][0]+m[2][1]*m[2][1]+m[2][2]*m[2][2]);
    for(e=0;e<3;e++)m[2][e]*=len;
}


template <class T, class T2> inline
void _Matrix34C<T,T2>::ApplyScale_onlyWxyz( double mul )
{
    for(int i=0;i<3;i++)
        for(int e=0;e<3;e++) m[i][e]*=mul;
}


// Bei Orthogonalmatrizen reicht Transponieren
template <class T, class T2> inline
void _Matrix34C<T,T2>::Inverse_onlyWxyz( const _Matrix34C<T,T2> &b )
{
    for(int i=0;i<3;i++)
        for(int e=0;e<3;e++)
            m[e][i]=b.m[i][e];
}



// Bei Orthogonalmatrizen reicht Transponieren und angleichen des Vektors
template <class T, class T2> inline
void _Matrix34C<T,T2>::Inverse( const _Matrix34C<T,T2> &b )
{
    int i;
    
    for(i=0;i<3;i++)
        for(int e=0;e<3;e++)
            m[e][i]=b.m[i][e];

    for(i=0;i<3;i++)
        p[i]=(float)(-b.p[0]*m[i][0]
                     -b.p[1]*m[i][1]
                     -b.p[2]*m[i][2]);
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::Transp_onlyWxyz( void )
{
    for(int i=0;i<3;i++)
        for(int e=0;e<3;e++)
            if(i!=e)
            {
                double help=m[i][e];m[i][e]=m[e][i];m[e][i]=help;
            }
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::Daempfung( double fakxyz, double fak )
{
    p[0]*=(float)fak;p[1]*=(float)fak;p[2]*=(float)fak;
    
    m[0][0]=(m[0][0]-1.0)*fakxyz+1.0;
    m[1][1]=(m[1][1]-1.0)*fakxyz+1.0;
    m[2][2]=(m[2][2]-1.0)*fakxyz+1.0;
    m[0][1]*=fakxyz;m[0][2]*=fakxyz;
    m[1][0]*=fakxyz;m[1][2]*=fakxyz;
    m[2][0]*=fakxyz;m[2][1]*=fakxyz;
}


template <class T, class T2> inline
void _Matrix34C<T,T2>::MixWith_onlyWxyz( double fak, const _Matrix34C<float,double> &mat )
{
	// fak==0, keinen nderung
	// fak==1, *this=mat

	double ifak=1.0-fak;

	for(int i=0;i<3;i++)
		for(e=0;e<3;e++)
			m[e][i]=m[e][i]*ifak+mat.m[e][i]*fak;
}


inline double GetAngle( float x, float y )
{
	double aa;

	if(y!=0.0f)
	{
		aa=atan(x/y);
		if(y<0)
		{
			if(x>0)aa+=3.14159265;
				else aa-=3.14159265;
		}
	}
	else
	{
		if(x>0)aa=3.14159265/2;
			else aa=-3.14159265/2;
	}

	return(aa);
}


template <class T, class T2> inline
//_Matrix34C<float,float> _Matrix34C<float,float>::operator=( _Matrix34C<float,double> &mat )
const _Matrix34C<T,T2> _Matrix34C<T,T2>::operator=( const _Matrix34C<T,T2> &mat )
//const _Matrix34C<float,float> _Matrix34C<T,T2>::operator=( _Matrix34C<float,float> &mat )
{
	if(this==&mat)return(*this);

	for(int i=0;i<3;i++)
	{
		m[i][0]=mat.m[i][0];
		m[i][1]=mat.m[i][1];
		m[i][2]=mat.m[i][2];
	    p[i]=mat.p[i];
	}
	return(*this);
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::Convert( _Matrix34C<float,double> &mat )
{
	for(int i=0;i<3;i++)
	{
		m[i][0]=(float)mat.m[i][0];
		m[i][1]=(float)mat.m[i][1];
		m[i][2]=(float)mat.m[i][2];
	    p[i]=mat.p[i];
	}
}


template <class T, class T2> inline
void _Matrix34C<T,T2>::Load( FILE *in )
{
	p[0]=read_float(in);p[1]=read_float(in);p[2]=read_float(in);
   
	for(int i=0;i<3;i++)for(int e=0;e<3;e++)
		m[i][e]=(double)read_float(in);
}

template <class T, class T2> inline
void _Matrix34C<T,T2>::Save( FILE *out )
{
    write_float(out,p[0]);write_float(out,p[1]);write_float(out,p[2]);


	for(int i=0;i<3;i++)for(int e=0;e<3;e++)
		write_float(out,(float)m[i][e]);
}

// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************





template < class T > class _CCubicData
{
private:

	T m_para[4];

public:

	_CCubicData(){}
	_CCubicData( T a, T b, T c, T d) { m_para[0]=a;m_para[1]=b;m_para[2]=c;m_para[3]=d; }

	void SetData( T a, T b, T c, T d) { m_para[0]=a;m_para[1]=b;m_para[2]=c;m_para[3]=d; }

	float GetNearLength( float time )
	{
		T v1=CalcPos(0.0f),v2=CalcPos(time*0.5f),v3=CalcPos(time);

		return((v2-v1).Length()+(v3-v2).Length());
	}

	T CalcPos( float fak )		// fak=0..1
	{
		return( ((m_para[0]*fak + m_para[1])*fak + m_para[2])*fak + m_para[3] );
	}
};

// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************

// speichert einen normierten Vector in einem Byte (OktaederOben/Unten +xyPos flachgedrcktem Oktaeder)
// damit lassen nicht Normalenvektoren sehr effizient packen und evtl. einige
// Operationen damit ber Tabellen vereinfachen

// noch nicht fertig
class PackedDirection
{
private:
	UBYTE	dat;	// 242=2*11*11 Zustnde(0-121, 128-249) werden verwendet
public:
	PackedDirection( const PackedDirection &x ){ dat=x.dat; }

	UBYTE GetSide( void )	// 0..1
	{  
		return((UBYTE)(dat>=128?1:0));
	}

	UBYTE GetX( void )	// 0..10 ineffizient
	{  
		if(dat>=128)return((UBYTE)((dat-128)%11));
			else return((UBYTE)(dat%11));
	}

	UBYTE GetY( void )	// 0..10 ineffizient
	{  
		if(dat>=128)return((UBYTE)((dat-128)/11));
			else return((UBYTE)(dat/11));
	}

	void Set( UBYTE Side, UBYTE u, UBYTE v )	// Side=0..1 u=0..10 v=0..10    ineffizient
	{
		dat=(UBYTE)(Side*128+v*11+u);
	}
};


// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************
// ******************************************************************************************************


typedef _Vector3dC<float> Vector3dC;
typedef _Matrix34C<float,double> Matrix34C;
typedef _Matrix34C<float,float> Matrix34C_float;
typedef _CCubicData<float> CCubicData;
typedef _CCubicData<Vector3dC> CCubicData3;


class COrientation
{
public:
	Vector3dC m_pos;
	Vector3dC m_front;
	Vector3dC m_up;
	float	m_Zoom;

	COrientation(){}

	COrientation( Vector3dC pos, Vector3dC front, Vector3dC up, float Zoom )
	{
		m_pos=pos;m_front=front;m_up=up;m_Zoom=Zoom;
	}

	float Length( void )
	{
		return(m_pos.Length());
	}

	void GetMatrix( Matrix34C &mat )
	{
		mat.SetVec(m_pos);
		mat.SetNormBase(m_up,m_front);
	}

	COrientation operator+	( const COrientation &other )
	{
		return(COrientation(m_pos+other.m_pos,m_front+other.m_front,m_up+other.m_up,m_Zoom+other.m_Zoom));
	}

	COrientation operator-	( const COrientation &other )
	{
		return(COrientation(m_pos-other.m_pos,m_front-other.m_front,m_up-other.m_up,m_Zoom-other.m_Zoom));
	}

	COrientation operator*	( const float fak )
	{
		return(COrientation(m_pos*fak,m_front*fak,m_up*fak,m_Zoom*fak));
	}
};




inline ULONG irand( ULONG max ){ return(((ULONG)rand()*max)>>15); }

inline float frand( float max ){ return(rand()*(1.0f/32768.0f)*max); }
inline float frand2( float max ){ return(( rand()*(1.0f/16384.0f) - 1.0f )*max); }
inline float fsqr( float a ){ return(a*a); }

// Zufall zwischen w*3/4 und w*5/4
inline int iabout( int w ) { return(w+irand(w/2)-(w/4)); }




inline Vector3dC VectorRndCube( float max )
{
	return(Vector3dC(frand2(max),frand2(max),frand2(max)));
}


inline Vector3dC VectorRndSphere( float max )
{
	Vector3dC a;

	do
	{
		a.p[0]=frand2(1);
		a.p[1]=frand2(1);
		a.p[2]=frand2(1);

	} while(a.LengthQuad()>1.0f);

	return(a*max);
}


inline Vector3dC VectorRndSphereHull( float max )
{
	Vector3dC a;

	do
	{
		do
		{
			a.p[0]=frand2(1);
			a.p[1]=frand2(1);
			a.p[2]=frand2(1);

		} while(a.LengthQuad()>1.0f);
	} while(a.LengthQuad()<0.0001f);

	return(a*(max/a.Length()));
}


template <class T> inline
void _Vector3dC<T>::GetOtherBaseVec( _Vector3dC<T> &a, _Vector3dC<T> &b ) const
{
	if(p[2]<-0.5f || p[2]>0.5f)
	{
		a.p[0]=p[2];
		a.p[1]=p[1];
		a.p[2]=-p[0];
		a.Normalize();	// effizienter machbar
		b.CrossProd(*this,a);
	}
	else
	{
		a.p[0]=p[1];
		a.p[1]=-p[0];
		a.p[2]=p[2];
		a.Normalize();	// effizienter machbar
		b.CrossProd(*this,a);
	}
}


#endif
