// TList.h:
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TLIST1_H__BEA5D80A_F01C_4058_8831_D2897C6031BA__INCLUDED_)
#define AFX_TLIST1_H__BEA5D80A_F01C_4058_8831_D2897C6031BA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TMemPool.h"										// CTMemPool



template <class T>
class CTListElement
{
public:

	// *********************************************************************************
			CTListElement	()
	{
	}

	// *********************************************************************************
			CTListElement	( const T &el )
	{
		m_Data=el;
	}

	// *********************************************************************************
			~CTListElement	()
	{
	}

	CTListElement<T> *	m_Next;
	T					m_Data;
};

// *********************************************************************************
// *********************************************************************************
// *********************************************************************************



// unsorted, single linked, new or delete isnt used (uses CTMemPool)
// interface is similar to the STL
// this could be done only a little bit better (mem_needed)

template <class T>
class CTList  
{

	
public:

	// order: from front to end
	class iterator
	{
	public:
		// *********************************************************************************
		// default constructor
					iterator		()
		{
			m_NextIterator=0;m_Container=0;m_Ptr=0;				// optimizble
		}

		// *********************************************************************************
		// constructor
					iterator		( CTListElement<T> *el, CTList<T> *container )
		{
			m_NextIterator=0;m_Container=0;m_Ptr=el;			// optimizble
			container->AddIterator(this);
		}

		// *********************************************************************************
		// copy constructor
					iterator		( const iterator &el )
		{
			m_NextIterator=0;m_Container=0;m_Ptr=el.m_Ptr;		// optimizble
			el.m_Container->AddIterator(this);
		}

		// *********************************************************************************
		// destructor
					~iterator		()
		{
			if(m_Container)m_Container->SubIterator(this);
		}

		// *********************************************************************************
		const T &	operator*		( void ) const			// access to the data

		{
			if(m_Ptr->m_Next==0)			// last Element ( ==end() )
				assert(0);

			return((T &)(m_Ptr->m_Data));
		}

		// *********************************************************************************
		// assignment operator
		const iterator &	operator=		( const iterator &el )

		{
			if(m_Container)m_Container->SubIterator(this);

			m_NextIterator=0;m_Container=0;m_Ptr=el.m_Ptr;		// optimizble
			el.m_Container->AddIterator(this);

			return(*this);
		}

		// *********************************************************************************
		bool		operator==		( const iterator &rhs ) const
		{
			assert(m_Container=rhs.m_Container);			// you cant compare elements from differnt lists

			return(m_Ptr==rhs.m_Ptr);
		}

		// *********************************************************************************
		bool		operator!=		( const iterator &rhs ) const
		{
			assert(m_Container==rhs.m_Container);			// you cant compare elements from differnt lists

			return(m_Ptr!=rhs.m_Ptr);
		}

		// *********************************************************************************
		// it++
		iterator	operator++		( int i )
		{
			iterator ret=*this;

			m_Ptr=m_Ptr->m_Next;
			return(ret);
		}

		// *********************************************************************************
		// ++it
		iterator	operator++		( void )
		{
			m_Ptr=m_Ptr->m_Next;
			return(*this);
		}

	private:
		CTListElement<T> *	m_Ptr;
		iterator *			m_NextIterator;		// next iterator in container
		CTList<T> *			m_Container;		//

		friend CTList<T>;
	};

	// *********************************************************************************
	// *********************************************************************************
	// *********************************************************************************

	
	// *********************************************************************************
	// constructor
				CTList		( void )
	{
		m_Begin.m_Next=&m_End;
		m_End.m_Next=0;
		m_Pool=0;
		m_IteratorList=0;
	}

	// *********************************************************************************
	// destructor
				~CTList		( void )
	{
		SubAllIterators();
		assert(!m_Pool);		// before destructor call delete and SetPool(0);
	}

	// *********************************************************************************
	// insert element at the start of the list O(1)
	bool		push_front	( const T &e )
	{	
		CTListElement<T> *p=m_Pool->New();

		if(!p)return(false);

		p->m_Data=e;
		p->m_Next=m_Begin.m_Next;
		m_Begin.m_Next=p;

		return(true);
	}

	// *********************************************************************************
	// insert element at the end of the list O(n)
	bool		push_back	( const T &el )
	{	
		CTListElement<T> *p=m_Pool->New();

		if(!p)return(false);

		CTListElement<T> *prev=FindElementBefore(&m_End);

		p->m_Data=el;
		p->m_Next=prev->m_Next;
		prev->m_Next=p;

		return(true);
	}

	// *********************************************************************************
	// O(1)
	T			pop_front	( void )
	{
		assert(!empty());

		T					el=m_Begin.m_Next->m_Data;
		CTListElement<T>	*next=m_Begin.m_Next->m_Next;

		m_Pool->Delete(&m_Begin);

		m_Begin.m_Next=next;
		return(el);
	}

	// *********************************************************************************
	// O(n)
	T			pop_back	( void )
	{
		assert(0);			// todo
	}

	// *********************************************************************************
	// *** currently untested ***
	bool		erase		( iterator at )
	{
		CTListElement<T> *prev=FindElementBefore(at.m_Ptr);

		if(!prev)return(false);

		prev->m_Next=prev->m_Next->m_Next;

		m_Pool.Delete(at.m_Ptr):

		SubIterator(at);

		return(true);
	}

	// *********************************************************************************
	//
	bool		erase		( iterator first, iterator last )
	{
		assert(0);			// todo

//		InvalidateIterators();
	}

	// *********************************************************************************
	//
	void		SetPool		( CTMemPool< CTListElement<T> > *inPool )
	{
		m_Pool=inPool;
	}

	// *********************************************************************************
	//
	CTMemPool< CTListElement<T> > *	GetPool		( void )
	{
		return(m_Pool);
	}

	// *********************************************************************************
	void		clear		( void )
	{
		CTListElement<T> *p=m_Begin.m_Next;

		while(p!=&m_End)
		{
			CTListElement<T> *next=p->m_Next;

			m_Pool->Delete(p);

			p=next;
		}

		m_Begin.m_Next=&m_End;

		SubAllIterators();
	}

	// *********************************************************************************
	bool		empty		( void ) const
	{
		return(m_Begin.m_Next==&m_End);
	}

	// *********************************************************************************
	iterator	begin		( void )
	{
		return(iterator(m_Begin.m_Next,this));
	}

	// *********************************************************************************
	iterator	end			( void )
	{
		return(iterator(&m_End,this));
	}

	// *********************************************************************************
	void		debug		( void )
	{
		iterator i;

		for(i=begin();i!=end();++i)
			cout << *i << " ";

		cout << endl;

		cout.flush();
	}

private:

	// *********************************************************************************
	CTListElement<T> *FindElementBefore( CTListElement<T> *el )
	{
		CTListElement<T> *p=&m_Begin;
		CTListElement<T> *next=p->m_Next;

		while(next!=el)
		{
			p=next;next=p->m_Next;
		}
		return(p);
	}

	// *********************************************************************************
	// optimized, neccessary for destroying elements
	void		InvalidateIterators( CTListElement<T> *mem )
	{
		iterator *old=0,*p=m_IteratorList; 

		while(p)
		{
			if(p->m_Ptr==mem)
			{
				iterator *next=p->m_NextIterator; 

				if(old)old->m_NextIterator=it->m_NextIterator;
					else m_IteratorList=it->m_NextIterator;

				it->m_Container=0;
				it->m_NextIterator=0;
				it->m_Ptr=0;

				p=next;continue;
			}

			old=p;p=p->m_NextIterator;
		}
	}

	// *********************************************************************************
	void		AddIterator( iterator *it )
	{
		assert(!it->m_Container);

		it->m_Container=this;
		it->m_NextIterator=m_IteratorList;
		m_IteratorList=it;
	}

	// *********************************************************************************
	void		SubIterator( iterator *it )
	{
		iterator *old=0,*p=m_IteratorList; 

		while(p)
		{
			if(p==it)
			{
				if(old)old->m_NextIterator=it->m_NextIterator;
					else m_IteratorList=it->m_NextIterator;

				it->m_Container=0;
				it->m_NextIterator=0;
				it->m_Ptr=0;
				return;
			}

			old=p;p=p->m_NextIterator;
		}

		assert(0);		// iterator wasnt in the list
	}

	// *********************************************************************************
	void		SubAllIterators( void )
	{
		iterator *p=m_IteratorList;

		while(p)
		{
			p->m_Container=0;
			p->m_NextIterator=0;
			p->m_Ptr=0;

			p=p->m_NextIterator;
		}

		m_IteratorList=0;
	}

	// *********************************************************************************
	// *********************************************************************************
	// *********************************************************************************

	CTMemPool< CTListElement<T> > *		m_Pool;			// new and delete is not neccessary, because of this pool
	CTListElement<T>					m_Begin;		// this could be a pointer, but 
														// it is much easier (e.g. for FindElementBefore)
														// to handle it in a uniform way
	CTListElement<T>					m_End;			// points to no-where, the last Element in the List Points to this
	iterator *							m_IteratorList;	// for invalidate the iterator after erase or clear

	friend	iterator;
};





// *************************************************************************************************
// *************************************************************************************************
// *************************************************************************************************
// *************************************************************************************************
// *************************************************************************************************
// *************************************************************************************************






#endif // !defined(AFX_TLIST1_H__BEA5D80A_F01C_4058_8831_D2897C6031BA__INCLUDED_)
