// TTree.h:
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTREE1_H__BEA5D80A_F01C_4058_8831_D2897C6031BA__INCLUDED_)
#define AFX_TTREE1_H__BEA5D80A_F01C_4058_8831_D2897C6031BA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TMemPool.h"										// CTMemPool
#include <iostream>											// cout


// tree with elements stored in the nodes, not only in the leafes
// memory consumption is not minimal, but that makes the class handy and quick for
// most operations

template <class T, bool ConnectBidirectional=false>
class CTTreeElement
{
public:

	// order: this,down,next
	class iterator
	{
	public:
		// *********************************************************************************
		// default constructor
					iterator		()
		{
			m_NextIterator=0;
			m_Ptr=0;
		}

		// *********************************************************************************
		// constructor
					iterator		( CTTreeElement<T,ConnectBidirectional> *el )
		{
			m_Ptr=el;
			if(el)el->AddIterator(this);
		}

		// *********************************************************************************
		// copy constructor
					iterator		( const iterator &el )
		{
			m_Ptr=el.m_Ptr;
			if(m_Ptr)m_Ptr->AddIterator(this);
		}

		// *********************************************************************************
		// destructor
					~iterator		()
		{
			if(m_Ptr)m_Ptr->SubIterator(this);
		}

		// *********************************************************************************
		// cast
				operator bool	( void ) const
		{
			return(m_Ptr!=0);
		}
	
		// *********************************************************************************
		// access to the data
		T &	operator*			( void ) const
		{
			assert(this!=0);

			return((T &)(m_Ptr->m_Data));
		}

		// *********************************************************************************
		// access to the data
		CTTreeElement<T> *GetNode( void ) const
		{
			assert(this!=0);

			return(m_Ptr);
		}

		// *********************************************************************************
		// assignment operator
		const iterator &	operator=		( const iterator &el )

		{
			if(m_Ptr)m_Ptr->SubIterator(this);

			m_Ptr=el.m_Ptr;		// optimizble
			if(m_Ptr!=0)m_Ptr->AddIterator(this);

			return(*this);
		}

		// *********************************************************************************
		bool		operator==		( const iterator &rhs ) const
		{
			return(m_Ptr==rhs.m_Ptr);
		}

		// *********************************************************************************
		bool		operator!=		( const iterator &rhs ) const
		{
			return(m_Ptr!=rhs.m_Ptr);
		}

		// *********************************************************************************
		// it++
		iterator	operator++		( int i )
		{
			assert(m_Ptr);

			iterator ret=*this;

			CTTreeElement<T,ConnectBidirectional> *p=m_Ptr;

			m_Ptr->SubIterator(this);				// clear m_Ptr

			if(p->m_Down)
			{
				p=p->m_Down;						// one right

				if(p)p->AddIterator(this);			// set m_Ptr
				return(ret);
			}

			if(p->m_Next)
			{
				p=p->m_Next;						// one next

				if(p)p->AddIterator(this);			// set m_Ptr
				return(ret);
			}

			if(p->m_Parent)
			{
				while(p->m_Parent->m_Down==0)		// 
				{
					p=p->m_Parent;

					if(!p->m_Parent)				// end reached
						return(*this);
				}

				p=p->m_Parent->m_Next;				// one right
			}
			else p=0;								// end reached

			if(p)p->AddIterator(this);				// set m_Ptr

			return(ret);
		}

		// *********************************************************************************
		// ++it
		iterator	operator++		( void )
		{
//			CTTreeElement<T> *ret=m_Ptr;

//			m_Ptr=m_Ptr->m_Down;
//			return(ret);
			assert(0);		// todo
		}
/*
		// *********************************************************************************
		int	count_nextandright		( void )
		{
			assert(this);

			CTTreeElement<T> *p=m_Ptr;
			int	ret=-1;
			
			while(p)
			{
				p=p->m_Parent;

				ret++;
			}
		
			return(ret);
		}


		// *********************************************************************************
		int	count_next				( void )
		{
			assert(this);

			CTTreeElement<T> *p=m_Ptr;
			int	ret=0;
			
			while(p)
			{
				CTTreeElement<T> *pp=p->m_Parent;

				if(pp)if(pp->m_Next==p)ret++;

				p=pp;
			}
		
			return(ret);
		}

		// *********************************************************************************
		int	count_right				( void )
		{
			assert(this);

			CTTreeElement<T> *p=m_Ptr;
			int	ret=0;
			
			while(p)
			{
				CTTreeElement<T> *pp=p->m_Parent;

				if(pp)if(pp->m_Down==p)ret++;

				p=pp;
			}
		
			return(ret);
		}
*/
/*		// *********************************************************************************
		void		debug		( void )
		{
			assert(this);											// dont talk to me, when Im not here

			cout << "Tree:\n{\n";


			for(iterator i=begin();i!=end();i++)
			{
				int depth=i.count_left()+1;

				for(int e=0;e<depth;e++)cout << " ";				// could be done better
					
				cout << (*i);

				cout << endl;
			}

			cout << "}\n";
			cout.flush();
		}
*/
		// *********************************************************************************
		// insert behind the current, the current doesnt change
		bool		insertinto( CTTreeElement<T,ConnectBidirectional> *p )
		{
			assert(p!=0);
			assert(p->m_Down==0);
			assert(p->m_Parent==0);

			p->m_Down=m_Ptr->m_Down;
			p->m_Parent=m_Ptr;

			if(m_Ptr->m_Down)m_Ptr->m_Down->m_Parent=p;
			m_Ptr->m_Down=p;

			// inform all childs about the new parent
			while(p->m_Next!=0)
			{
				p=p->m_Next;
				assert(p->m_Parent==0);
				p->m_Parent=m_Ptr;
			}
			return(true);
		}

		// *********************************************************************************
		// insert a child under the current, the current iterator doesnt change
		bool		append( CTTreeElement<T,ConnectBidirectional> *p )
		{
			assert(p!=0);
//			assert(p->m_Down==0);
			assert(p->m_Next==0);
			assert(p->m_Parent==0);

//			p->m_Down=0;
			p->m_Next=m_Ptr->m_Next;
			p->m_Parent=m_Ptr;

			if(m_Ptr->m_Next)m_Ptr->m_Next->m_Parent=p;
			m_Ptr->m_Next=p;
			return(true);
		}


		// *********************************************************************************
		// insert a child on the current, afterwards points the iterator to the new objet
		bool		insert	( CTTreeElement<T,ConnectBidirectional> *p, bool subtreetoleft=true )
		{
			assert(p!=0);
			assert(p->m_Down==0);
			assert(p->m_Next==0);
			assert(p->m_Parent==0);

			CTTreeElement<T,ConnectBidirectional> *old=m_Ptr;

			if(m_Ptr)m_Ptr->SubIterator(this);			// clear m_Ptr

			if(subtreetoleft)
			{
				p->m_Down=0;
				p->m_Next=old;
			}
			else
			{
				p->m_Down=old;
				p->m_Next=0;
			}
			
			if(m_Ptr)
			{
				p->m_Parent=old->m_Parent;
				old->m_Parent=p;
			}

			p->AddIterator(this);

			return(true);
		}
		// *********************************************************************************
		iterator	right		( void )
		{
			return(m_Ptr->m_Down);
		}

		// *********************************************************************************
		iterator	left		( void )
		{
			return(m_Ptr->m_Next);
		}

			
		// *********************************************************************************
		iterator	begin		( void )
		{
			return(m_Ptr);
		}

		// *********************************************************************************
		iterator	end			( void )
		{
			return(0);
		}

	private:
		CTTreeElement<T,ConnectBidirectional> *	m_Ptr;					// Pointer to the element (container for iterators)
		iterator *								m_NextIterator;			// next iterator in container

		friend CTTreeElement<T,ConnectBidirectional>;
	};

	// *********************************************************************************
	// *********************************************************************************
	// *********************************************************************************


	// *********************************************************************************
			CTTreeElement	()
	{
		m_Next=0;m_Down=0;m_Parent=0;m_IteratorList=0;
	}

	// *********************************************************************************
			CTTreeElement	( T &el )
	{
		m_Data=el;m_IteratorList=0;
		m_Next=0;m_Down=0;m_Parent=0;

		if(ConnectBidirectional)
			m_Data->Client_ContainerChanged(this);
	}

	// *********************************************************************************
			~CTTreeElement	()
	{
		SubAllIterators();

		if(ConnectBidirectional)
			m_Data->Client_ContainerChanged(0);
	}

private:
	// *********************************************************************************
	// optimized, neccessary for destroying elements
	void		InvalidateIterators( CTTreeElement<T,ConnectBidirectional> *mem )
	{
		iterator *old=0,*p=m_IteratorList; 

		while(p)
		{
			if(p->m_Ptr==mem)
			{
				iterator *next=p->m_NextIterator; 

				if(old)old->m_NextIterator=it->m_NextIterator;
					else m_IteratorList=it->m_NextIterator;

				it->m_NextIterator=0;
				it->m_Ptr=0;

				p=next;continue;
			}

			old=p;p=p->m_NextIterator;
		}
	}

	// *********************************************************************************
	void		AddIterator( iterator *it )
	{
		assert(this);

		it->m_NextIterator=m_IteratorList;
		it->m_Ptr=this;
		m_IteratorList=it;
	}

	// *********************************************************************************
	void		SubIterator( iterator *it )
	{
		assert(this);

		iterator *old=0,*p=m_IteratorList; 

		while(p)
		{
			if(p==it)
			{
				if(old)old->m_NextIterator=it->m_NextIterator;
					else m_IteratorList=it->m_NextIterator;

				it->m_NextIterator=0;
				it->m_Ptr=0;
				return;
			}

			old=p;p=p->m_NextIterator;
		}

		assert(0);		// iterator wasnt in the list
	}

	// *********************************************************************************
	void		SubAllIterators( void )
	{
		assert(this);

		iterator *p=m_IteratorList;

		while(p)
		{
			p->m_NextIterator=0;
			p->m_Ptr=0;

			p=p->m_NextIterator;
		}

		m_IteratorList=0;
	}

	// ********************************************************************************
	// ********************************************************************************
	// ********************************************************************************

	CTTreeElement<T,ConnectBidirectional> *	m_Next;			// first the next pointer
	CTTreeElement<T,ConnectBidirectional> *	m_Down;			// pointer to the child elements
	CTTreeElement<T,ConnectBidirectional> *	m_Parent;		// redundant, for faster access and easier itertors
	iterator *								m_IteratorList;	// for invalidate the iterator after erase or clear

	T										m_Data;			// the the data

	friend iterator;
};


#endif // !defined(AFX_TTREE1_H__BEA5D80A_F01C_4058_8831_D2897C6031BA__INCLUDED_)
