// Nicht vergesssen:
//
// keine strings in interfaces verwenden, wg. Freigabeproblematik mit DLLs

#ifndef _INTERFACES_H
#define _INTERFACES_H

#include "Datatypes.h"
#include "handle.h"					// CHandle
#include <objbase.h>
#include <string>
#include "Features\TTree.h"			// CTTree
//#include <iostream>
using namespace std;

class CInstanceFactory;
class I2DMemoryDC;
class IMessageHandler;
class CWnd;
class CSerializeArchivePointer;
class IEditorDocument;
class CGUIAppData;
class IReferenceGuard;
class IWinGUI;
class CPoint2I;
class CRect2I;
class IHistoryCommand;
class IHistoryManager;
class I2DServerInteraction;


// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************

// Baumstruktur, wird derzeit von SimpleBox und WinGui verwendet
// recht stabil, Freigabe der Objekte durch normales Ref-Counting
// vereinfacht Serialize-Methoden von hierarchischen Strukturen

// Interface Name
class ISceneGraphObject : public IUnknown 
{ 
public:
	// Interface Methods
	virtual bool						InsertObject		( ISceneGraphObject *p )=0;		// Unterobjekt einfgen, vorne, O(1)
	virtual bool						InsertObjectAtEnd	( ISceneGraphObject *p )=0;		// Unterobjekt einfgen, hinten O(n), n=Anz. Elemente in der Liste
	virtual void						ReleaseObject		( void )=0;						// Austragen
	virtual void						ReleaseSubObjects	( void )=0;						// Unterobjekte freigeben
	virtual void						OnChildReleased		( ISceneGraphObject *p )=0;		// 

	// fr die interne Benutzung
	virtual CHandle<ISceneGraphObject>	GetParent			( void )=0;						// Vater Objekt
	virtual CHandle<ISceneGraphObject>	GetNext				( void )=0;						// nachfolgende Objekte
	virtual CHandle<ISceneGraphObject>	GetIn				( void )=0;						// enthaltene Objekte
	virtual void						SetParent			( ISceneGraphObject *p )=0;		// Vater Objekt
	virtual void						SetNext				( ISceneGraphObject *p )=0;		// nachfolgende Objekte
	virtual void						SetIn				( ISceneGraphObject *p )=0;		// enthaltene Objekte
};

// {A05625F3-3695-4673-ACEA-10DEEF9341F2}
DEFINE_GUID(IID_ISceneGraphObject, 0xa05625f3, 0x3695, 0x4673, 0xac, 0xea, 0x10, 0xde, 0xef, 0x93, 0x41, 0xf2);





// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************


// Interface Name
class IPopUpMenu :public IUnknown 
{ 
public:
	// Interface Methods
	virtual	void			SetPos				( int x, int y )=0;
	virtual	bool			Init				( CWnd *wnd )=0;

	// Ist fr Untermens, gefllt mir nicht
	virtual IPopUpMenu *	Create				( void )=0;
	virtual	void			AppendItem			( char *text, char *command, int mode=0 )=0;	// mode=1 -> mit Haken
	virtual	void			AppendMenu			( char *text, IPopUpMenu *entry )=0;
	virtual string			GetCommand			( int id )=0;
	virtual void			TrackPopUpMenu		( void )=0;
};

// {AE7C26F9-A142-4c2f-A45B-E40C4AC8AFD4}
DEFINE_GUID(IID_IPopUpMenu, 
0xae7c26f9, 0xa142, 0x4c2f, 0xa4, 0x5b, 0xe4, 0xc, 0x4a, 0xc8, 0xaf, 0xd4);


// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************


// Interface Name
class IEditorApplication :public IUnknown 
{ 
public:
	// Interface Methods
	virtual ULONG					GetVersion				( void )=0;
	virtual IMessageHandler *		GetErrorHandler			( void )=0;
	virtual IMessageHandler *		GetMessageHandler		( void )=0;
	virtual CInstanceFactory *		GetFactory				( void )=0;
	virtual IPopUpMenu *			GetPopupHandler			( void )=0;
	virtual IWinGUI *				GetWinGUI				( void )=0;
	virtual IReferenceGuard *		GetReferenceGuard		( void )=0;
//	virtual HINSTANCE				GetInstanceHandle		( void )=0;		// nur vorbergehend
	virtual bool					GetGUIAppData			( CGUIAppData &ret )=0;		// damit man der Applikation Dialoge/Toolbar/... anhngen kann
};

// {3839DB86-012D-455f-B21F-0B53D1F925B4}
DEFINE_GUID(IID_IEditorApplication, 
0x3839db86, 0x12d, 0x455f, 0xb2, 0x1f, 0xb, 0x53, 0xd1, 0xf9, 0x25, 0xb4);

// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************


// Interface Name
class IEditorDocument :public IUnknown 
{ 
public:
	// Interface Methods
	virtual void				InvalidateAllViews	( void )=0;
};

// {13281724-6A49-4b25-841C-AA90C359D129}
DEFINE_GUID(IID_IEditorDocument, 
0x13281724, 0x6a49, 0x4b25, 0x84, 0x1c, 0xaa, 0x90, 0xc3, 0x59, 0xd1, 0x29);


// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************



// Interface Name
class ISerialize : public IUnknown 
{ 
public:
	// Interface Methods
	virtual void				Serialize			( CSerializeArchivePointer &ar )=0;
	virtual void				SerializeRecursive	( CSerializeArchivePointer &ar )=0;
	virtual const GUID & 		GetGUID				( void )=0;		// individuell fr jedes SGO
	virtual void				OnAfterLoad			( void )=0;		// wird nach dem Laden aller Objekte fr alle Objekte aufgerufen
																	// in der Reihenfolge des Serialisierens
};

// {1FDC617A-4B84-42fa-A17A-909DD288ABBE}
DEFINE_GUID(IID_ISerialize, 0x1fdc617a, 0x4b84, 0x42fa, 0xa1, 0x7a, 0x90, 0x9d, 0xd2, 0x88, 0xab, 0xbe);

// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************




class I2DClientInteraction :public IUnknown 
{
public:
	// Interface Methods

	// MouseMove wird for Frame nur einmal aufgerufen
	virtual	bool				Client_OnMouseMove		( CPoint2I xy, ULONG dwOldFlags, ULONG dwNewFlags, CPoint2I pos, int inMagnify )=0;
	// MouseButton wird unter Umstnden in einem Frame mehrmals aufgerufen
	// return: true-Nachricht wurde behandelt, false-Nachricht an Objekt darunter weitergeben
	virtual	bool				Client_OnMouseButton	( CPoint2I xy, ULONG dwOldFlags, ULONG dwNewFlags, CPoint2I pos, int inMagnify )=0;

	// return: true, Key was processed
	virtual	void				Client_SrvChanged		( I2DServerInteraction *inSrv )=0;
	virtual	bool				Client_OnKey			( DWORD nChar )=0;
	virtual void				Client_Draw				( I2DMemoryDC *pMC, CPoint2I pos, int inMagnify )=0;
	virtual void				Client_MarkedDraw		( I2DMemoryDC *pMC, int r, CPoint2I pos, int inMagnify )=0;	//wird zusttzlich zu Draw aufgerufen, wenn Objekt markiert ist
	virtual	CRect2I				Client_GetRect			( void )=0;
	virtual	void				Client_RecalcRect		( const CPoint2I &inPos, const CPoint2I &inChildSize )=0;
	virtual bool				Client_Init				( const CRect2I &inArea )=0;
	virtual void				Client_OnMarkStateChanged( bool inMark )=0;

	virtual void				Client_ContainerChanged	( CTTreeElement<CHandle<I2DClientInteraction>,true> * )=0;
};


// {C237F4A4-4118-43e2-8DF7-A8048FAE48AA}
DEFINE_GUID(IID_I2DClientInteraction, 0xc237f4a4, 0x4118, 0x43e2, 0x8d, 0xf7, 0xa8, 0x4, 0x8f, 0xae, 0x48, 0xaa);

// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************





class I2DServerInteraction :public IUnknown 
{
public:
	// Interface Methods

	virtual	void				Server_Mark				( I2DClientInteraction *p, bool inNewState, bool inClearSelectionBefore=true )=0;
	virtual	void				Server_RedrawArea		( const CRect2I &inArea )=0;	// if inArea==0, do full Update 
	virtual	void				Server_UpdateRect		( const I2DClientInteraction *p, const CRect2I &inArea )=0;
	virtual	void				Server_AddObject		( I2DClientInteraction *p )=0;
	virtual	void				Server_SubObject		( I2DClientInteraction *p )=0;


	virtual IHistoryManager *	GetHistoryManager		( void )=0;
//	virtual void				Server_HistoryAddAndDo	( IHistoryCommand *inCmd, bool inIsTemp )=0;
//	virtual void				SecureTemp				( void );
//	virtual bool				UnDo					( void );						//
	
};

// {C9CDD8CA-1900-4524-A386-F5BE7DF1FAF0}
DEFINE_GUID(IID_I2DServerInteraction, 0xc9cdd8ca, 0x1900, 0x4524, 0xa3, 0x86, 0xf5, 0xbe, 0x7d, 0xf1, 0xfa, 0xf0);

// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************
// ****************************************************************************************



#endif // _INTERFACES_H
