// SerializeArchiv.cpp: Implementierung der Klasse CSerializeArchiv.
//
//////////////////////////////////////////////////////////////////////

#pragma warning( disable : 4786 )		// STL warning ausschalten

// #include "stdafx.h"

#include "SerializeArchive.h"
#include "SerializeArchivePointer.h"
#include "interfaces.h"
#include "Datatypes.h"

#include "CInstanceFactory.h"

#include "..\PlugInSource\MessageHandler\IMessageHandler.h"		// IMessageObject

#include "GuidTypDef.h"		// CGuidTypDef

extern IEditorApplication *g_App;
extern CInstanceFactory *_g_Factory;


#include <map>
using namespace std;


CSerializeArchivePointer	CSerializeArchive::ar;


#define ERR if(g_App->GetErrorHandler())(g_App->GetErrorHandler())		// fr Fehlerausgabe



// Zeichenkette in 4 Byte Chunknamen umwandeln
ULONG CNAME( const char *txt )
{
	assert(strlen(txt)==4);	// mu so sein

	return((((ULONG)txt[3])<<24)|(((ULONG)txt[2])<<16)|(((ULONG)txt[1])<<8)|((ULONG)txt[0]));
}





bool operator<( const GUID &_a, const GUID &_b )		// GUID vergleichen
{
	ULONG *a=(ULONG *)&_a;
	ULONG *b=(ULONG *)&_b;

	for(int i=0;i<4;i++)
	{
		if(*a<*b)return(true);
			else if(*a>*b)return(false);

		a++;b++;
	}
	return(false);
}






/*
void write_word( FILE *out, UWORD data )
{
    fwrite(&data,2,1,out);
}
*/

void write_block( FILE *handle, const void *ptr, ULONG len )
{
    fwrite(ptr,1,len,handle);
}

// 4 Byte alignment
void write_block_align( FILE *handle, UBYTE *ptr, ULONG len )
{
	UBYTE dummy[3]={0,0,0};
    fwrite(ptr,1,len,handle);
	if(((len+3)&0xfffffffc)-len!=0)fwrite(dummy,1,((len+3)&0xfffffffc)-len,handle);
}

// ohne Fehlerberprfung
void write_long( FILE *out, ULONG data )
{
    fwrite(&data,4,1,out);
}



// ohne Fehlerberprfung
ULONG read_long( FILE *in )
{
    ULONG data;

    fread(&data,4,1,in);
    return(data);
}
/*
// ohne Fehlerberprfung
UWORD read_word( FILE *in )
{
    UWORD data;

    fread(&data,2,1,in);
    return(data);
}
*/
int read_block( FILE *handle, UBYTE *ptr, ULONG len )
{
    return(fread(ptr,1,len,handle));
}

// 4 Byte alignment
int read_block_align( FILE *handle, UBYTE *ptr, ULONG len )
{
	int ret=fread(ptr,1,len,handle);	if(ret!=len)return(0);

	UBYTE dummy[3];

	if(((len+3)&0xfffffffc)==len)return(len);
    return(fread(dummy,1,((len+3)&0xfffffffc)-len,handle));
}

ULONG seek_pos( FILE *hnd )
{
	return((ULONG)ftell(hnd));
}

void write_long_at( FILE *out, ULONG value, ULONG pos )
{
	ULONG d=(ULONG)ftell(out);

	fseek(out,pos,SEEK_SET);	// neue position
	write_long(out,value);		// Wert schreiben
	fseek(out,d,SEEK_SET);		// zurck
}



//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CSerializeArchive::CSerializeArchive()
{
	m_accessptr=0;m_actualattribdef=0;
	m_filehandle=0;
}

CSerializeArchive::~CSerializeArchive()
{
	assert(m_filehandle==0);		// file mu geschlossen sein
}








ULONG MakeHashNumber( const char *txt )
{
	ULONG ret=0x35673412;
	int len=strlen(txt);

	for(int i=0;i<len;i++)
	{
		ret=(ret<<2)^ret^i^(i<<6)^(ULONG)(txt[i]);
	}
	return(ret);
}


const char *GetFilenameOnly( const char *path_with_file )
{
	char const *ret=path_with_file;

	char const *p=path_with_file;
	while(*p!=0)
	{
		if(*p=='\\' || *p==':')ret=p+1;
		p++;
	}

	return(ret);
}




ISerialize *CSerializeArchive::Load( const char *filename )
{
	const char *filename_only=GetFilenameOnly(filename);		// ohne pfad

	// Open
    FILE *in=fopen(filename,"rb");		
	if(!in)							
		{ ERR->Add("CSerializeArchive::Load '%s' failed",filename);return(false); }

	// Dateiformatkennung (Scene Graph Data base)
    if(read_long(in)!=CNAME("SGDb"))
		{ fclose(in);ERR->Add("CSerializeArchive::Load Filetype failed");return(false); }

	// Versionsnummer
	ULONG version=read_long(in);
    if(version!=0x00000002)				
		{ fclose(in);ERR->Add("CSerializeArchive::Load Wrong Version");return(false); }

	// Filename manipulation
	if(read_long(in)!=MakeHashNumber(filename_only))
	{ 
		fclose(in);
		ERR->Add("Filename manipulation");
		return(false);
	}

	ar.SetCallback(this);		// ISerializePointerCorrection

	// ********************************************************************

#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("Serialize Load ADEF -----------------------------");
#endif
	if(read_long(in)!=CNAME("ADEF"))		// Chunkname
	{ 
		fclose(in);
		ERR->Add("Chunk ADEF not found");
		return(false);
	}

	read_long(in);							// Chunklnge wird hier nicht bentigt
	ULONG typcount=read_long(in);			// Anzahl der Objekttypen
	assert(typcount!=0);

#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("    Objecttypecount: %4d",typcount);
#endif

	CGuidTypDef *typCollection=new CGuidTypDef[typcount];	// Id <-> Guid/ADEF Zuordnung



	{
		for(ULONG i=0;i<typcount;i++)	// Alle Objekttypen lesen
		{
			ULONG size=read_long(in);	// 16(GUID) + AttribDef

			assert(size>=16);			// mindestens die GUID mu drin sein

			{
				GUID typ;

				read_block(in,(UBYTE *)&(typ),16);				// GUID einlesen
				typCollection[i].SetGuid(typ);

#ifdef DEBUGSERIALIZE
				{
					char str[40];	// GUID hat feste Lnge, 40 mu reichen

					GetStringFromGUID(typ,str);
					g_App->GetMessageHandler()->Add("  %4d. GUID: [%s]",i,str);
				}
#endif
			}

			{
				char str[64*1024];	assert(size<64*1024-1);		// 64Kb, sollte reichen

				read_block_align(in,(UBYTE *)str,size-16);		// Attributdefinition einlesen
				str[size-16]=0;									// 0 terminieren
				typCollection[i].SetAttribDef(str);

#ifdef DEBUGSERIALIZE
					g_App->GetMessageHandler()->Add("        ADEF: [%s] ",str);
#endif
			}
		}
	}




	// ********************************************************************

#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("Serialize Load APTR -----------------------------");
#endif
	if(read_long(in)!=CNAME("APTR"))		// Chunkname
	{ 
		delete [] typCollection;

		fclose(in);
		ERR->Add("Chunk APTR not found");
		return(false);
	}

	read_long(in);							// Chunklnge wird hier nicht bentigt
	ULONG objcount=read_long(in);			// Anzahl der Objektinstanzen
	assert(objcount!=0);					// denkbar, aber das Root sollte sich immer serialisieren

#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("    Objectcount: %4d",objcount);
#endif

	IUnknown ** objCollection=new IUnknown *[objcount];	// neue Speicherposition indiziert in der Reihenfolge der Speicherung

	map<ULONG,IUnknown *> mapCollection;				// alte Speicherpos <-> neue Speicherpos Zuordnung fr Interfacepointer angeleichen
	CGuidTypDef ** tobjCollection=new CGuidTypDef *[objcount];	// Objekt -> Typ Zuordnung

	ar.SetLoadModeADEF();


	{
		for(ULONG i=0;i<objcount;i++)	// Alle Objektinstanzen lesen
		{
			ULONG type_id=read_long(in);
			ULONG old_ptr=read_long(in);

			IUnknown *ptr=_g_Factory->CreateInstance(typCollection[type_id].GetGuid());

#ifdef DEBUGSERIALIZE
			g_App->GetMessageHandler()->Add("  %4d. Type:%4d    OldPtr: %p   NewPtr:%p",i,type_id,old_ptr,ptr);
#endif
			if(typCollection[type_id].GetState()==GTDS_notset)		// recover/fastread ermitteln
			{
				ISerialize *ser;

				ptr->QueryInterface(IID_ISerialize,(void **)&ser);
				assert(ser);

				ser->Serialize(ar);

				ULONG siz;
				char *buff=(char *)ar.GetBufferAndSize(siz);

				if(typCollection[type_id].SameType(buff))
					typCollection[type_id].SetState(GTDS_fastread);
				  else
					typCollection[type_id].SetState(GTDS_recover);		

#ifdef DEBUGSERIALIZE
				g_App->GetMessageHandler()->Add("        Mode: [%s] ",typCollection[type_id].GetState()==GTDS_fastread?"fastread":"recover");
#endif
				ar.FlushBuffer();

				ser->Release();
			}

			tobjCollection[i]=&typCollection[type_id];
			objCollection[i]=ptr;
			mapCollection[old_ptr]=ptr;
		}
	}





	// ********************************************************************

#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("Serialize Load ADAT -----------------------------");
#endif

	m_accessptr=&mapCollection;

	if(read_long(in)!=CNAME("ADAT"))		// Chunkname
	{ 
		delete tobjCollection;
		delete objCollection;
		delete []typCollection;

		fclose(in);
		ERR->Add("Chunk ADAT not found");
		return(false);
	}

	read_long(in);	// Chunklnge wird hier nicht bentigt

	{
		for(ULONG i=0;i<objcount;i++)	// Alle Objektinstanzen lesen
		{
			IUnknown *p2=objCollection[i];
			assert(p2);	// dieses sollte mal einer korrekten Fehlerbehandlung weichen

			CGuidTypDef *typ_id_ptr=tobjCollection[i];

			m_actualattribdef=typ_id_ptr->GetAttribDef();	// mu 0-terminiert sein

//			ULONG siz=typ_id_ptr->GetByteSize();

			ULONG siz=read_long(in);

#ifdef DEBUGSERIALIZE
			g_App->GetMessageHandler()->Add("  %4d. Ptr: %p  Data: %4d Bytes",i,p2,siz);
#endif

			if(typ_id_ptr->GetState()==GTDS_fastread) ar.SetLoadModeADAT_f();
				else ar.SetLoadModeADAT_r();	// recover liee sich auch schneller machen
			
			void *buff=ar.SetBufferAndInit();

			read_block(in,(UBYTE *)buff,siz);

			ISerialize *p;

			p2->QueryInterface(IID_ISerialize,(LPVOID*)&p);
			assert(p);

			p->Serialize(ar);

			p->Release();
		}
	}

	m_accessptr=0;

	// ********************************************************************

	if(read_long(in)!=CNAME("AFIN"))		// Chunkname
	{ 
		delete tobjCollection;
		delete objCollection;
		delete []typCollection;

		fclose(in);
		ERR->Add("Chunk AFIN not found");
		return(false);
	}


	// ********************************************************************

	fclose(in);

	// OnAfterLoad aufrufen

	{
		for(ULONG i=0;i<objcount;i++)	// Alle Objektinstanzen durchgehen
		{
			IUnknown *p2=objCollection[i];

			ISerialize *p;

			p2->QueryInterface(IID_ISerialize,(LPVOID*)&p);
			assert(p);

			p->OnAfterLoad();

			p->Release();
		}
	}

	// erstes Objekt ist das Rootobjekt
	ISerialize *ret;
	objCollection[0]->QueryInterface(IID_ISerialize,(LPVOID*)&ret);
	assert(ret);

	delete tobjCollection;
	delete objCollection;
	delete []typCollection;

	return(ret);
}


















// seek liee sich etwas besser machen.
// Auch knnte man die Datei im Speicher aufbauen
bool CSerializeArchive::Save( const char *filename, ISerialize *what )
{
	assert(g_App);

	const char *filename_only=GetFilenameOnly(filename);		// ohne pfad

	// Open
	assert(m_filehandle==0);
    m_filehandle=fopen(filename,"wb");
    
	if(!m_filehandle)
	{
		ERR->Add("CSerializeArchive::Save '%s' failed",filename);
		return(false);
	}

	// Magic Number
    write_long(m_filehandle,CNAME("SGDb"));				// Dateiformatkennung (Scene Graph Data base)

	// Versionsnummer
    write_long(m_filehandle,0x00000002);     

	write_long(m_filehandle,MakeHashNumber(filename_only));	// fr Filename Manipulation Erkennung

	ar.SetCallback(this);		// ISerializePointerCorrection






#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("Serialize Save ADEF -----------------------------");
#endif
	// Attributdefinition (pro Objekttyp)
	{
		ULONG no;								// Anzahl der Objekttypen

		ar.SetSaveModeADEF();				// setzt Counter auch auf 0
		write_long(m_filehandle,CNAME("ADEF"));			// Chunkname
		
		ULONG oldseekpos=seek_pos(m_filehandle);
		write_long(m_filehandle,0);						// Chunksize, wird mittels write_long_at spter korrigiert
		write_long(m_filehandle,0);						// Anzahl der Objekttypen, wird mittels write_long_at spter korrigiert

		what->SerializeRecursive(ar);			// Counter wird fr jeden Typ hochgezhlt

		no=ar.GetCounter();						// Counter auslesen

#ifdef DEBUGSERIALIZE
		g_App->GetMessageHandler()->Add("    Objecttypecount: %d",no);
#endif
		write_long_at(m_filehandle,no,oldseekpos+4);	//  Anzahl der Objekttypen korrigieren

		write_long_at(m_filehandle,seek_pos(m_filehandle)-oldseekpos+4,oldseekpos);	// Chunkgre korrigieren
	}







#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("Serialize Save APTR -----------------------------");
#endif
	{
		ULONG no;								// Anzahl der Objektinstanzen

		// Pointer der Objekte zum Zeitpunkt des Speicherns und deren Typid pro Objektinstanz
		write_long(m_filehandle,CNAME("APTR"));			// Chunkname

		ULONG oldseekpos=seek_pos(m_filehandle);
		write_long(m_filehandle,0);						// Chunksize, wird mittels write_long_at spter korrigiert
		write_long(m_filehandle,0);						// Anzahl der Instanzen, wird mittels write_long_at spter korrigiert

		ar.SetSaveModeAPTR();					// setzt Counter auch auf 0

		what->SerializeRecursive(ar);			// Counter wird fr jeden Instanz hochgezhlt

		no=ar.GetCounter();						// Counter auslesen

#ifdef DEBUGSERIALIZE
		g_App->GetMessageHandler()->Add("    Objectcount: %d",no);
#endif
		write_long_at(m_filehandle,no,oldseekpos+4);	//  Anzahl der Instanzen korrigieren

		write_long_at(m_filehandle,seek_pos(m_filehandle)-oldseekpos+4,oldseekpos);	// Chunkgre korrigieren
	}







#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("Serialize Save ADAT -----------------------------");
#endif
	{
		// Attributdaten pro Objektinstanz
		ar.SetSaveModeADAT();
		write_long(m_filehandle,CNAME("ADAT"));			// Chunkname

		ULONG oldseekpos=seek_pos(m_filehandle);
		write_long(m_filehandle,0);						// Chunksize, wird mittels write_long_at spter korrigiert
	
		what->SerializeRecursive(ar);

		write_long_at(m_filehandle,seek_pos(m_filehandle)-oldseekpos+4,oldseekpos);	// Chunkgre korrigieren
	}


#ifdef DEBUGSERIALIZE
	g_App->GetMessageHandler()->Add("Serialize Save End ------------------------------");
#endif
    write_long(m_filehandle,CNAME("AFIN"));			// Endmarker

	fclose(m_filehandle);
	m_filehandle=0;
	return(true);
}







bool CSerializeArchive::IsGUIDRegistered( const GUID &g )
{
	return(m_mCollection.count(g)!=0);	// Objekttyp noch nicht dagewesen, also serialisieren
}



void CSerializeArchive::RegisterGUID( const GUID &g, ULONG keyid )
{
	m_mCollection[g]=keyid;
}

ULONG CSerializeArchive::GetKeyFromGUID( const GUID &g )	// liefert key-id zurck
{
	return(m_mCollection[g]);
}

IUnknown *CSerializeArchive::Convert( ULONG old )
{
	return((*m_accessptr)[old]);
}

const char *CSerializeArchive::GetAttribDef( void )		// fr Recovery load
{
	return(m_actualattribdef);	// mu 0-terminiert sein
}


void CSerializeArchive::WriteObjHeader( const ULONG guid_id, const IUnknown *oldmempos )	// fr saveAPTR
{
	assert(m_filehandle);

	write_long(m_filehandle,guid_id);
	write_long(m_filehandle,(ULONG)oldmempos);
}

void CSerializeArchive::WriteObjDataBlock( const void *data, const ULONG size )
{
	write_long(m_filehandle,size);				// optimierbar, wenn man die Zahl mit in den Block einschlsselt
	write_block(m_filehandle,data,size);		// Speicherblock mit den Daten
}

void CSerializeArchive::WriteObjDefBlock( const void *def, const ULONG size, const GUID &guid )
{
	assert(size<64*1024);

	write_long(m_filehandle,size+16);							// size=16+defsize
	write_block(m_filehandle,(UBYTE *)&(guid),16);

	if(size!=0)write_block_align(m_filehandle,(UBYTE *)def,size);
}


