// SerializeArchivePointer.cpp: Implementierung der Klasse CSerializeArchivePointer.
//
//////////////////////////////////////////////////////////////////////
//#ifdef _AFXDLL				// MFC
//  #include "stdafx.h"
//#endif

#include "SerializeArchivePointer.h"
#include "interfaces.h"
#include <string.h>

#ifdef DEBUGSERIALIZE
 #include "..\PlugInSource\MessageHandler\IMessageHandler.h"		// IMessageObject
#endif

extern IEditorApplication *g_App;
#define ERR if(g_App->GetErrorHandler())(g_App->GetErrorHandler())		// fr Fehlerausgabe
#define MESS if(g_App->GetMessageHandler())(g_App->GetMessageHandler())		// fr Fehlerausgabe

void write_long( FILE *out, ULONG data );		// aus SerializeArchive.cpp


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

// AttributeString: (nicht Nullterminiert, sondern Lnge in m_attribdefsize )
//
// Aufbau: Abfolge von Speicherbedarf, Typ und Name, durch / getrennt, Typ und Name durch ' getrennt
// (der Name mu nicht angegeben werden, dann entfllt das ' Zeichen )
// (der Name darf bis zu 32 Zeichen lang sein)
// (der Speicherbedarf mu fr alle Attribute des gleichen Typs konstant sein, oder
//  es wird statt einer Zahl A angegeben. Das heit dem Attribut ist ein Byte vorangestellt, das
//  die Lnge des Attributes enthlt (z.B. bei Strings), spter soll es auch ein B oder C geben, 
//  das Lngen ber 255 und 65K Bytes zult) 
//
//           SizeTyp'Name/SizeTyp/SizeTyp/SizeTyp'Name/SizeTyp'Name
//
// z.B.      4I'test/4I/4I/AS'filename/4F'dummyf
// bedeutet: 4ByteInteger(Name test), 4ByteInteger, 4ByteInteger, 4ByteInteger(Name dummy), 4ByteFloat(name dummyf)
//
// nicht erlaubte Zeichen im Namen: / und '  
//
// Mgliche Typen(mit Grenangabe): (drfen bis zu 8 Buchstaben sein - ist aber weniger effizient)
//
// I4 ..... bool		4 Bytes wg. Alignment
// I4 ..... char		4 Bytes wg. Alignment
// I4 ..... short		4 Bytes wg. Alignment
// I4 ..... int			4 Bytes
// R4 ..... float		4 Bytes
// R8 ..... double		8 Bytes
// AS ..... string      1 Byte Lnge, dann Daten, max 255 Zeichen und erweitert um bis zu 3 Bytes wg. 4 Byte Alignment (schnellerer Zugriff)





// funktioniert vorwrts und rckwrts
// soll nur einfache Hackversuche abwehren
void crpytString( UBYTE *ptr, ULONG len )
{
	ULONG code=0x62454782;

	for(int i=0;i<len;i++)
	{
		UBYTE q=*ptr;

		code+=(code<<13)+i+(len<<7)+len;

		*ptr++=((UBYTE)code) ^ q;
	}
}




CSerializeArchivePointer::CSerializeArchivePointer()
{
	m_mode=CSAM_notset;
	FlushBuffer();
	m_aktObject=0;
	m_CallbackPtr=0;
}

CSerializeArchivePointer::~CSerializeArchivePointer()
{
}


bool CSerializeArchivePointer::BeforeSerialize( ISerialize *what )				// vor jedem Serialize() aufrufen
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			break;
		case CSAM_loadADAT_r:
			break;
		case CSAM_saveADAT:
			break;
		case CSAM_loadADEF:
			break;
		case CSAM_saveADEF:
			{
				if(!m_CallbackPtr->IsGUIDRegistered(what->GetGUID()))	// Objekttyp noch nicht dagewesen, also serialisieren
				{
#ifdef DEBUGSERIALIZE
					{
						char str[40];	// GUID hat feste Lnge, 40 mu reichen
	
						GetStringFromGUID(what->GetGUID(),str);
						MESS->Add("  %4d. GUID: [%s]",m_counter,str);
					}
#endif
				} else return(false);
			}
			break;
		case CSAM_saveAPTR:
			{
				ULONG id=m_CallbackPtr->GetKeyFromGUID(what->GetGUID());


			{
				IUnknown *i;
				what->QueryInterface(IID_IUnknown,(LPVOID*)&i);assert(i);

				m_CallbackPtr->WriteObjHeader(id,i);	// id=0.. ,  Speicherpos, fr Handle/Zeiger Auflsung

#ifdef DEBUGSERIALIZE
				MESS->Add("  %4d.  Type:%4d    Ptr: %p",m_counter,id,i);
#endif
			}

				m_counter++;		// Objektinstanz hinzuzhlen
			}
			break;
		default:	assert(0);return(false);
	}

	return(true);
}


void CSerializeArchivePointer::AfterSerialize( ISerialize *what )				// nach jedem Serialize() aufrufen
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			break;
		case CSAM_loadADAT_r:
			break;
		case CSAM_saveADAT:
			{
				assert(m_CallbackPtr);

				{
					ULONG siz;
					char *buff=(char *)GetBufferAndSize(siz);

					m_CallbackPtr->RegisterGUID(what->GetGUID(),m_counter);

#ifdef DEBUGSERIALIZE
					MESS->Add("  [Type:%4d]   Data: %4d Bytes",m_counter,siz);
#endif
					if(siz!=0)m_CallbackPtr->WriteObjDataBlock((void *)buff,siz);
				}

#ifdef DEBUGSERIALIZE
				m_counter++;			// nur frs Debugging
#endif
				FlushBuffer();
//				p=p->GetNextRecursive();
			}

			break;
		case CSAM_loadADEF:
			break;
		case CSAM_saveADEF:
			{
				assert(m_CallbackPtr);					
				assert(!m_CallbackPtr->IsGUIDRegistered(what->GetGUID()));	// Objekttyp noch nicht dagewesen, also serialisieren, wurde vorher schon gecheckt

				ULONG siz;
				char *buff=(char *)GetBufferAndSize(siz);

#ifdef DEBUGSERIALIZE
				MESS->Add("        ADEF: [%s] ",(buff!=0) ? buff:"");
#endif
				m_CallbackPtr->WriteObjDefBlock((void *)buff,siz,what->GetGUID());

				FlushBuffer();

				m_CallbackPtr->RegisterGUID(what->GetGUID(),m_counter++);		// Typ hinzuzhlen
			}

			break;
		case CSAM_saveAPTR:
			break;
		default:	assert(0);
	}
}






bool CSerializeArchivePointer::Dbool( bool &value, char *name )
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];
	
				value=(*p==1);m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADAT_r:
			{
				ULONG *p=(ULONG *)FindInAttrString("I4",name);	// kann spter auch Typen casten
	
				if(p)					// nur wenn es gefunden wurde
					value=(*p==1);			
			}
			break;
		case CSAM_saveADAT:
			{			// 0x00000001
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				*p=value?1:0;m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("I4",name);		// wird als 1 Byte gespeichert
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}





bool CSerializeArchivePointer::Dchar( char &value, char *name )
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{			// 0x000000ff
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				value=(UBYTE)(*p);m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADAT_r:
			{			// 0x000000ff
				ULONG *p=(ULONG *)FindInAttrString("I4",name);	// kann spter auch Typen casten

				if(p)					// nur wenn es gefunden wurde
					value=(UBYTE)(*p);
			}
			break;
		case CSAM_saveADAT:		
			{			// 0x000000ff
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				*p=(ULONG)((UBYTE)value);m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("I4",name);
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}




// noch nicht getestet
bool CSerializeArchivePointer::Dstring( string &value, char *name )
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{			// 0xff * k
				UBYTE *p=(UBYTE *)&m_attribdat[m_attribdatsize];	// Lnge im ersten Byte

				ULONG len=*p;
				char str[260];

				strncpy(str,(char *)(p+1),len);str[len]=0;

				crpytString((UBYTE *)str,len);
				value=str;m_attribdatsize+=((len+1+3)/4)*4;
			}
			break;
		case CSAM_loadADAT_r:
			{			// 0xff * k
				UBYTE *p=(UBYTE *)FindInAttrString("AS",name);	// kann spter auch Typen casten

				if(p)					// nur wenn es gefunden wurde
				{
					ULONG len=*p;
					char str[260];

					strncpy(str,(char *)(p+1),len);str[len]=0;
					crpytString((UBYTE *)str,len);
					value=str;
				}
			}
			break;
		case CSAM_saveADAT:		
			{			// 0xff * k 
				UBYTE *p=(UBYTE *)&m_attribdat[m_attribdatsize];
				ULONG len=value.length();

				if(len>255)
				{
					len=255;		// String wird abgeschnitten
					ERR->Add("Saving String>255 characters, limited zu 255");
				}		

				*p=len;
				strncpy((char *)(p+1),value.c_str(),len);
				crpytString((UBYTE *)(p+1),len);
				// das Ausnullen spar ich mir
				m_attribdatsize+=((len+1+3)/4)*4;
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("AS",name);
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}



bool CSerializeArchivePointer::Dshort( short &value, char *name )
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{			// 0x0000ffff
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				value=(UWORD)(*p);m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADAT_r:
			{			// 0x0000ffff
				ULONG *p=(ULONG *)FindInAttrString("I4",name);	// kann spter auch Typen casten

				if(p)					// nur wenn es gefunden wurde
					value=(UWORD)(*p);
			}
			break;
		case CSAM_saveADAT:		
			{			// 0x0000ffff
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				*p=(ULONG)((UWORD)value);m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("I4",name);
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}

bool CSerializeArchivePointer::Dint( int &value, char *name )
{
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{			// 0xffffffff
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				value=*p;m_attribdatsize+=4;

#ifdef DEBUGSERIALIZE
				MESS->Add("	Int %d",value);
#endif
			}
			break;
		case CSAM_loadADAT_r:
			{			// 0xffffffff
				ULONG *p=(ULONG *)FindInAttrString("I4",name);	// kann spter auch Typen casten

				if(p)					// nur wenn es gefunden wurde
					value=*p;

#ifdef DEBUGSERIALIZE
				MESS->Add("	Int %d recover: %c",value,p?'1':'0');
#endif
			}
			break;
		case CSAM_saveADAT:		
			{			// 0xffffffff
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				*p=(ULONG)(value);m_attribdatsize+=4;

#ifdef DEBUGSERIALIZE
				MESS->Add("	Int %d",value);
#endif
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("I4",name);
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}


bool CSerializeArchivePointer::Dfloat( float &value, char *name )
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{			// 0xffffffff
				float *p=(float *)&m_attribdat[m_attribdatsize];

				value=*p;m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADAT_r:
			{			// 0xffffffff
				float *p=(float *)FindInAttrString("R4",name);	// kann spter auch Typen casten

				if(p)					// nur wenn es gefunden wurde
					value=*p;
			}
			break;
		case CSAM_saveADAT:
			{			// 0xffffffff
				float *p=(float *)&m_attribdat[m_attribdatsize];

				*p=value;m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("R4",name);
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}






bool CSerializeArchivePointer::Ddouble( double &value, char *name )
{
	assert(g_App);
	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{			// 0xffffffff ffffffff
				double *p=(double *)&m_attribdat[m_attribdatsize];

				value=*p;m_attribdatsize+=8;
			}
			break;
		case CSAM_loadADAT_r:
			{			// 0xffffffff ffffffff
				double *p=(double *)FindInAttrString("R8",name);	// kann spter auch Typen casten

				if(p)					// nur wenn es gefunden wurde
					value=*p;
			}
			break;
		case CSAM_saveADAT:		
			{			// 0xffffffff ffffffff
				double *p=(double *)&m_attribdat[m_attribdatsize];

				*p=value;m_attribdatsize+=8;
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("R8",name);
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}




bool CSerializeArchivePointer::DHandle( const GUID &IId, IUnknown * &value, char *name )
{
	assert(g_App);

	// Check, ob IId zu CHandle passt
#ifdef DEBUG
	{
			T *p;
			value->QueryInterface(IId,(LPVOID*)&p);
			assert(value.GetPtr()==p);
	}
#endif

	switch(m_mode)
	{ 
		case CSAM_loadADAT_f:
			{			// 0xffffffff
				assert(m_CallbackPtr!=0);

				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				if(*p!=0)
				{
					value=m_CallbackPtr->Convert(*p);
				
					assert(value!=0);							// Fehler in den internen Datenstrukturen
				}

#ifdef DEBUGSERIALIZE
				MESS->Add("	OldPtr %p   NewPtr: %p",*p,value);
#endif
				m_attribdatsize+=4;
			}
			break;
		case CSAM_loadADAT_r:
			{			// 0xffffffff
				assert(m_CallbackPtr!=0);

				ULONG *p=(ULONG *)FindInAttrString("H4",name);	// kann spter auch Typen casten

				if(p)
					value=m_CallbackPtr->Convert(*p);

#ifdef DEBUGSERIALIZE
				MESS->Add("	OldPtr %p   NewPtr: %p recover:%c",p?*p:0,value,p?'1':'0');
#endif
			}
			break;
		case CSAM_saveADAT:		
			{			// 0xffffffff
				ULONG *p=(ULONG *)&m_attribdat[m_attribdatsize];

				*p=(ULONG)value;m_attribdatsize+=4;

#ifdef DEBUGSERIALIZE
				MESS->Add("	Ptr %p",value);
#endif
			}
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			AppendAttribDef("H4",name);			// H=Handle
			break;
		case CSAM_saveAPTR:	
			break;
		default:	assert(0);return(false);
	}

	return(false);
}


void CSerializeArchivePointer::SetCallback( ISerializePointerCorrection *callback )
{
	m_CallbackPtr=callback;
}



void *CSerializeArchivePointer::GetBufferAndSize( ULONG &size )
{
	switch(m_mode)
	{ 
//		case CSAM_loadADAT:
//			break;
		case CSAM_saveADAT:		
			size=m_attribdatsize;
			return(m_attribdatsize!=0?m_attribdat:0);
			break;
		case CSAM_loadADEF:
		case CSAM_saveADEF:
			m_attribdef[m_attribdefsize]=0;	// Endterminator

			size=m_attribdefsize!=0?m_attribdefsize-1:0;
			return(m_attribdefsize!=0?m_attribdef+1:0);
			break;
		default:	assert(0);return(false);
	}
}


void *CSerializeArchivePointer::SetBufferAndInit( void )
{
	m_attribdatsize=0;
	return(m_attribdat);
}


void CSerializeArchivePointer::FlushBuffer( void )						// nach jeder Objektdef		
{
	m_attribdefsize=0;
	m_attribdatsize=0;
}


void CSerializeArchivePointer::SetSaveModeADEF( void )					// darf nur CSerializeArchive machen
{
	m_mode=CSAM_saveADEF;
	m_counter=0;	// Objekttypen zhlen
}


void CSerializeArchivePointer::SetSaveModeAPTR( void )					// darf nur CSerializeArchive machen
{
	m_mode=CSAM_saveAPTR;
	m_counter=0;	// Objektinstanzen zhlen
}


void CSerializeArchivePointer::SetSaveModeADAT( void )					//  darf nur CSerializeArchive machen
{
	m_mode=CSAM_saveADAT;
	m_counter=0;	// Objekttypen zhlen
}

void CSerializeArchivePointer::SetLoadModeADAT_f( void )					// darf nur CSerializeArchive machen
{
	m_mode=CSAM_loadADAT_f;
}

void CSerializeArchivePointer::SetLoadModeADAT_r( void )					// darf nur CSerializeArchive machen
{
	m_mode=CSAM_loadADAT_r;
}

void CSerializeArchivePointer::SetLoadModeADEF( void )					// darf nur CSerializeArchive machen
{
	m_mode=CSAM_loadADEF;
}


bool CSerializeArchivePointer::AppendAttribDef( char *type, char *name )
{
	int lent=strlen(type);
	int lenn=strlen(name);

#ifdef _DEBUG
	{						// TYPE Check
		char *p=type;
		while(*p)
		{
			// illegale Zeichen, werden anderweitig verwendet
			if(*p=='-' || *p=='/' || *p=='\''){ assert(0);return(false); }	
			p++;
		}
	}
	{						// NAME Check
		char *p=name;
		while(*p)
		{
			// illegale Zeichen, werden anderweitig verwendet
			if(*p=='-' || *p=='/' || *p=='\''){ assert(0);return(false); }	
			p++;
		}
	}
#endif

	if(lent+lenn+m_attribdefsize+2>=MEM_ATTRIBDEF)return(false);

	m_attribdef[m_attribdefsize++]='/';
	strcpy(&m_attribdef[m_attribdefsize],type);m_attribdefsize+=lent;

	m_attribdef[m_attribdefsize++]='-';
	strcpy(&m_attribdef[m_attribdefsize],name);m_attribdefsize+=lenn;
	return(true);
}


// Recovery
void CSerializeArchivePointer::RearrangeMemory( char *intype, char *outtype, void *inmem, void *outmem )
{
	assert(strcmp(intype,outtype)!=0);	// nur ntig, wenn AttributeStrings verschieden sind

	char *p=0;		
	
	while(*p!=0)
	{
		if(*p=='A')			{}
		else if(*p=='B')	{}
		else if(*p=='C')	{}
		else
		{
			int size=0;

			while(*p>='0' && *p<='9'){ size=size*10 + (int)(*p-'0');p++; }

			char type[9],*_type=type;				// bis zu 8 Buchstaben

			while(*p!='\'' && *p!='/' && *p!=0){ *_type++=*p++; }	*_type=0;

			char name[33],*_name=name;				// bis zu 32 Zeichen lang

			if(*p=='\'')
			{
				p++;
				while(*p!='/' && *p!=0){ *_name++=*p++; }			*_name=0;
			}
			else name[0]=0;	// kein Name

			// size, type, name sind nun ermittelt




			if(*p=='/')p++;	// zum nchsten
		}

		p++;
	}
}

// schenller implementierbar
void *CSerializeArchivePointer::FindInAttrString( char *in_type, char *in_name )
{
	assert(m_mode==CSAM_loadADAT_r);

	const char *def=m_CallbackPtr->GetAttribDef();		// mu 0-terminiert sein
	ULONG offset=0;

	if(*def!=0)
	for(;;)
	{
		if(strncmp(def,in_type,strlen(in_type))==0)						// richtiger Typ
		{
			const char *def2=def;
			def2+=strlen(in_type);

			if(*def2=='-')
			{
				def2++;
				ULONG slen=strlen(in_name);

				if(strncmp(def2,in_name,slen)==0)					// richtiger Name
				{
					char fin=*(def2+slen);

					if(fin=='/' || fin==0)
						return((void *)((ULONG)m_attribdat+offset));
				}
			}
		}

		def++;											// Typ-Char berspringen

		ULONG len= (ULONG)(*def++)-(ULONG)'0';			// Gre in Bytes

		while(*def!=(char)'-')		
		{
			len=len*10 + (ULONG)(*def++)-(ULONG)'0';	// Gre in Bytes

			assert(*def);
		}

		offset+=len;

		while(*def!=0 && *def!='/')def++;

		if(*def==0)break;								// Abbruchbedingung
		def++;											// '/' berspringen
	}

	return(0);
}


/*bool CSerializeArchivePointer::FindInAttrString( char *in_data, ULONG &out_relindex, ULONG &out_attribsize )
{
	char *p=0;
	
	while(*p!=0)
	{
		if(*p=='A')			{}
		else if(*p=='B')	{}
		else if(*p=='C')	{}
		else
		{
			int size=0;

			while(*p>='0' && *p<='9'){ size=size*10 + (int)(*p-'0');p++; }

			char type[9],*_type=type;				// bis zu 8 Buchstaben

			while(*p!='\'' && *p!='/' && *p!=0){ *_type++=*p++; }	*_type=0;

			char name[33],*_name=name;				// bis zu 32 Zeichen lang

			if(*p=='\'')
			{
				p++;
				while(*p!='/' && *p!=0){ *_name++=*p++; }			*_name=0;
			}
			else name[0]=0;	// kein Name

			// size, type, name sind nun ermittelt




			if(*p=='/')p++;	// zum nchsten
		}

		p++;
	}

	return(false);
}
*/

ULONG CSerializeArchivePointer::GetCounter( void )
{
	return(m_counter);
}
